% evalresults.m
%
%     Part 1  -  Compute the cohort of workers that is laid off every period.
%     Part 2  -  Compute evolution of consumption of nonemployed workers.
%     Part 3  -  Dispersion of consumption growth rates: Extended model.
%     Part 4  -  Dispersion of consumption growth rates: Baseline model.


%%   Part 1. Compute the cohort of workers that is laid off every period.

SEPY = zeros(na,T);                                        % Distribution of endogenously-separated workers (young)
SEPO = zeros(na,T);                                        % Distribution of endogenously-separated workers (older)

fye1 = zeros(na,N,T);   foe1 = zeros(na,N,T);

fye1(:,z0idx,1) = (1-gamma)*q*theta*fyu;
fyu1 = (1-gamma)*(1-q*theta)*fyu;
fyu1(1,1) = fyu1(1,1) + newb;
fou1 = (1-chi)*fou + gamma*fyu;

for itau = 2 : T-1
  fye1(:,:,itau) = fye(:,:,itau-1) * P_a;
  tmp = repmat(split(itau,:),na,1);
  sepy = sum(fye1(:,:,itau).*(tmp == 1),2);
  SEPY(:,itau) = sepy;
  fye1(:,:,itau) = (tmp < 1).*fye1(:,:,itau);
  foe1(:,:,itau) = ( (1-chi) * foe(:,:,itau-1) + gamma * fye(:,:,itau-1) )* P_b;
  tmp = repmat(splito(itau,:),na,1);
  sepo = sum(foe1(:,:,itau).*(tmp == 1),2);
  SEPO(:,itau) = sepo;
  foe1(:,:,itau) = (tmp < 1).*foe1(:,:,itau);
  for ia = 1 : na
    fyu1(idxSP(ia,itau),1) = fyu1(idxSP(ia,itau),1) + squeeze((1-gamma)*delta*sum(fye(ia,:,itau-1),2)) + sepy(ia);
    fou1(idxSP(ia,itau),1) = fou1(idxSP(ia,itau),1) + squeeze((1-chi)*delta*sum(foe(ia,:,itau-1),2))   + sepo(ia) + ...
                             squeeze(gamma*delta*sum(fye(ia,:,itau-1),2));
  end
end

fye1(:,:,T) = (fye(:,:,T-1) + fye(:,:,T)) * P_a;
tmp = repmat(split(T,:),na,1);
sepy = sum(fye1(:,:,T).*(tmp == 1),2);
SEPY(:,T) = sepy;
fye1(:,:,T) = (tmp < 1).*fye1(:,:,T);
foe1(:,:,T) = (1-chi) * (1-delta) * (foe(:,:,T-1) + foe(:,:,T)) * P + gamma * (1-delta) * (fye(:,:,T-1) + fye(:,:,T)) * P;
tmp  = repmat(splito(T,:),na,1);
sepo = sum(foe1(:,:,T).*(tmp == 1),2);
SEPO(:,T) = sepo;
foe1(:,:,T) = (tmp < 1).*foe1(:,:,T);  
for ia = 1 : na
  fyu1(idxSP(ia,T),1) = fyu1(idxSP(ia,T),1) + squeeze((1-gamma)*delta*(sum(fye(ia,:,T-1),2) + sum(fye(ia,:,T),2))) + sepy(ia);
  fou1(idxSP(ia,T),1) = fou1(idxSP(ia,T),1) + squeeze((1-chi)*delta*(sum(foe(ia,:,T-1),2)   + sum(foe(ia,:,T),2))) + sepo(ia) + ...
                        squeeze(gamma*delta*(sum(fye(ia,:,T-1),2) + sum(fye(ia,:,T),2)));
end

YY = SEPY+delta*squeeze(sum(fye,2));                       % Add exogenous separations --> YY and OO are the cohorts of
OO = SEPO+delta*squeeze(sum(foe,2));                       % laid-off young and older workers respectively


%%   Part 2. Compute evolution of consumption of nonemployed workers.

[ aidxyu , prob1yu , prob2yu ] = stataux(adecyu,agrid);           aidxyu(aidxyu == 1) = 2;
[ aidxou , prob1ou , prob2ou ] = stataux(adecou,agrid);           aidxou(aidxou == 1) = 2;
[ aidxr  , prob1r  , prob2r  ] = stataux(adecr,agrid);            aidxr(aidxr == 1)   = 2;

cdecye = zeros(na,N,T);  cdecyu = zeros(na,1);  cdecoe = zeros(na,N,T);  cdecou = zeros(na,1);  cdecr  = zeros(na,1);

for ia = 1 : na
  for itau = 1 : T
    for iz = 1 : N
      cdecye(ia,iz,itau) = (1+r)*agrid(ia) + w(itau,iz)  + beqy + (-1000000)*split(itau,iz) - adecye(ia,iz,itau);
      cdecoe(ia,iz,itau) = (1+r)*agrid(ia) + wo(itau,iz) + (-1000000)*splito(itau,iz) - adecoe(ia,iz,itau);
    end
  end
  cdecyu(ia,1) = (1+r)*agrid(ia) + b  + beqy - adecyu(ia);
  cdecou(ia,1) = (1+r)*agrid(ia) + bo - adecou(ia);
  cdecr(ia,1)  = (1+r)*agrid(ia) + br - adecr(ia);
end

YYY = sum(YY,2);     Ycum = cumsum(YYY/sum(YYY));
OOO = sum(OO,2);     Ocum = cumsum(OOO/sum(OOO));

idxY25 = sum(Ycum < 0.40);     Y25 = YYY;   Y25(idxY25+1:end) = 0;
idxO25 = sum(Ocum < 0.40);     O25 = YYY;   O25(idxO25+1:end) = 0;

CONSyavg_ = zeros(25,1);   CONSyavg_(1) = sum(sum(cdecyu.*YYY))/sum(sum(YYY));
CONSoavg_ = zeros(25,1);   CONSoavg_(1) = sum(sum(cdecou.*OOO))/sum(sum(OOO));
CONSy25_  = zeros(25,1);   CONSy25_(1)  = sum(sum(cdecyu.*Y25))/sum(sum(Y25));
CONSo25_  = zeros(25,1);   CONSo25_(1)  = sum(sum(cdecou.*O25))/sum(sum(O25));

for spelluidx = 2 : 25
  YYY1tmp = (1-gamma)*(1-q*theta)*YYY;     YYY1 = zeros(na,1);
  OOO1tmp = (1-chi)*OOO;                   OOO1 = zeros(na,1);
  Y251tmp = (1-gamma)*(1-q*theta)*Y25;     Y251 = zeros(na,1);
  O251tmp = (1-chi)*O25;                   O251 = zeros(na,1);
  for ia = 1 : na
    YYY1(aidxyu(ia,1)-1,1) = YYY1(aidxyu(ia,1)-1,1) + prob1yu(ia,1)*YYY1tmp(ia,1);
    YYY1(aidxyu(ia,1),1)   = YYY1(aidxyu(ia,1),1)   + prob2yu(ia,1)*YYY1tmp(ia,1);
    OOO1(aidxou(ia,1)-1,1) = OOO1(aidxou(ia,1)-1,1) + prob1ou(ia,1)*OOO1tmp(ia,1);
    OOO1(aidxou(ia,1),1)   = OOO1(aidxou(ia,1),1)   + prob2ou(ia,1)*OOO1tmp(ia,1);
    Y251(aidxyu(ia,1)-1,1) = Y251(aidxyu(ia,1)-1,1) + prob1yu(ia,1)*Y251tmp(ia,1);
    Y251(aidxyu(ia,1),1)   = Y251(aidxyu(ia,1),1)   + prob2yu(ia,1)*Y251tmp(ia,1);
    O251(aidxou(ia,1)-1,1) = O251(aidxou(ia,1)-1,1) + prob1ou(ia,1)*O251tmp(ia,1);
    O251(aidxou(ia,1),1)   = O251(aidxou(ia,1),1)   + prob2ou(ia,1)*O251tmp(ia,1);
  end
  CONSyavg_(spelluidx) = sum(sum(cdecyu.*YYY1))/sum(sum(YYY1));    YYY = YYY1;
  CONSoavg_(spelluidx) = sum(sum(cdecou.*OOO1))/sum(sum(OOO1));    OOO = OOO1;
  CONSy25_(spelluidx)  = sum(sum(cdecyu.*Y251))/sum(sum(Y251));    Y25 = Y251;  
  CONSo25_(spelluidx)  = sum(sum(cdecou.*O251))/sum(sum(O251));    O25 = O251;
end

dcyavg_1 = 100*(CONSyavg_(2)/CONSyavg_(1)-1);
dcyavg_4 = 100*(CONSyavg_(5)/CONSyavg_(1)-1);
dcy25_1  = 100*(CONSy25_(2)/CONSy25_(1)-1);
dcy25_4  = 100*(CONSy25_(5)/CONSy25_(1)-1);

dcoavg_1 = 100*(CONSoavg_(2)/CONSoavg_(1)-1);
dcoavg_4 = 100*(CONSoavg_(5)/CONSoavg_(1)-1);
dco25_1  = 100*(CONSo25_(2)/CONSo25_(1)-1);
dco25_4  = 100*(CONSo25_(5)/CONSo25_(1)-1);

fprintf('\n                                        After 1 quarter       After 1 year      \n')
fprintf('   Young workers:   Average                 %5.1f%%              %5.1f%%           \n',dcyavg_1,dcyavg_4)
fprintf('                    40%% asset-poorest       %5.1f%%              %5.1f%%           \n',dcy25_1,dcy25_4)
fprintf('   Older workers:   Average                 %5.1f%%              %5.1f%%           \n',dcoavg_1,dcoavg_4)
fprintf('                    40%% asset-poorest       %5.1f%%              %5.1f%%           \n',dco25_1,dco25_4)


%%   Part 3. Dispersion of consumption growth rates: Extended model.

aidxye  = zeros(na,N,T);     aidxoe  = zeros(na,N,T);
prob1ye = zeros(na,N,T);     prob1oe = zeros(na,N,T);    prob2ye = zeros(na,N,T);     prob2oe = zeros(na,N,T);

for itau = 1 : T
  [ aidxyetmp  , prob1ye(:,:,itau)  , prob2ye(:,:,itau)  ] = stataux(adecye(:,:,itau),agrid);
  [ aidxoetmp  , prob1oe(:,:,itau)  , prob2oe(:,:,itau)  ] = stataux(adecoe(:,:,itau),agrid);
  aidxyetmp(aidxyetmp == 1) = 2;     aidxye(:,:,itau) = aidxyetmp;
  aidxoetmp(aidxoetmp == 1) = 2;     aidxoe(:,:,itau) = aidxoetmp;
end

popfac = 1e-6;   vvye = zeros(500000,1);   count  = 1;     % Part 1: Young and employed

for itau = 1 : T
for iz = 1 : N
for ia = 1 : na
  if fye(ia,iz,itau) > 0

    f0 = fye(ia,iz,itau);                                  % Period 0: Population mass, consumption, savings
    c0 = cdecye(ia,iz,itau);
    a1 = aidxye(ia,iz,itau);

    f1ye = f0 * P_a(iz,:);                                 % Period 1: Employed
    tmp  = split(min(itau+1,T),:);
    sepy = sum(f1ye.*(tmp == 1),2);
    f1ye = (tmp < 1).*f1ye;
    c1ye = cdecye(a1,:,min(itau+1,T));
    dcye = log(max(c1ye,0.00001)) - log(max(c0,0.00001));

    tmp = popsamp(dcye',f1ye',popfac);
    countstep = length(tmp);
    vvye(count:count+countstep-1,1) = tmp;
    count = count + countstep;
    
    f1yu = sepy + (1-gamma)*delta*f0;                      % Period 1: Unemployed
    c1yu = cdecyu(idxSP(a1,min(itau+1,T)),1);
    dcyu = log(max(c1yu,0.00001)) - log(max(c0,0.00001));
    
    tmp = popsamp(dcyu',f1yu',popfac);
    countstep = length(tmp);
    vvye(count:count+countstep-1,1) = tmp;
    count = count + countstep;
    
%     f1oe = f0 * gamma * P_a(iz,:);                       % Age shock (ignored, very small prob)
%     tmp  = splito(min(itau+1,T),:);
%     sepo = sum(f1oe.*(tmp == 1),2);
%     f1oe = (tmp < 1).*f1oe;
%     c1oe = cdecoe(a1,:,min(itau+1,T));
%     dcoe = log(max(c1oe,0.00001)) - log(max(c0,0.00001));
%     tmp = popsamp(dcoe',f1oe',popfac);   countstep = length(tmp);
%     vvye(count:count+countstep-1,1) = tmp;   count = count + countstep;
    
  end
  
end
end
if mod(itau,20) == 0
  fprintf('%3.0d',itau)
end
end
vvye(count+1:end) = [];                                    % Remove obsolete cells

vvyu = zeros(500000,1);   count = 1;                       % Part 1: Young and unemployed

for ia = 1 : na

  if fyu(ia,1) > 0

    f0 = fyu(ia,1);
    c0 = cdecyu(ia,1);
    a1 = aidxyu(ia,1);

    f1ye = (1-gamma)*q*theta*f0;
    c1ye = cdecye(a1,z0idx,1);
    dcye = log(max(c1ye,0.00001)) - log(max(c0,0.00001));
    tmp  = popsamp(dcye',f1ye',popfac);   countstep = length(tmp);
    vvyu(count:count+countstep-1,1) = tmp;   count = count + countstep;
    
    f1yu = (1-gamma)*(1-q*theta)*f0;
    c1yu = cdecyu(a1,1);
    dcyu = log(max(c1yu,0.00001)) - log(max(c0,0.00001));
    tmp = popsamp(dcyu',f1yu',popfac);   countstep = length(tmp);
    vvyu(count:count+countstep-1,1) = tmp;   count = count + countstep;
    
  end
  
end
vvyu(count+1:end) = [];

vvy = [ vvye ; vvyu ];
stdyExtd = std(vvy);

vvoe = zeros(500000,1);   count = 1;                       % Part 3: Older and employed

for itau = 1 : T
for iz = 1 : N
for ia = 1 : na
  if foe(ia,iz,itau) > 0

    f0 = foe(ia,iz,itau);
    c0 = cdecoe(ia,iz,itau);
    a1 = aidxoe(ia,iz,itau);

    f1oe = f0 * (1-chi) * P_b(iz,:);
    tmp  = splito(min(itau+1,T),:);
    sepo = sum(f1oe.*(tmp == 1),2);
    f1oe = (tmp < 1).*f1oe;
    c1oe = cdecoe(a1,:,min(itau+1,T));
    dcoe = log(max(c1oe,0.00001)) - log(max(c0,0.00001));
    tmp = popsamp(dcoe',f1oe',popfac);   countstep = length(tmp);
    vvoe(count:count+countstep-1,1) = tmp;   count = count + countstep;
    
    f1ou = sepo + (1-chi)*delta*f0;
    c1ou = cdecou(idxSP(a1,min(itau+1,T)),1);
    dcou = log(max(c1ou,0.00001)) - log(max(c0,0.00001));
    
    tmp = popsamp(dcou',f1ou',popfac);   countstep = length(tmp);
    vvoe(count:count+countstep-1,1) = tmp;   count = count + countstep;
    
  end
  
end
end
if mod(itau,20) == 0
  fprintf('%3.0d',itau)
end
end
vvoe(count+1:end) = [];

vvou = zeros(500000,1);   count = 1;                       % Part 4: Older and nonemployed

for ia = 1 : na

  if fou(ia,1) > 0

    f0 = fou(ia,1);
    c0 = cdecou(ia,1);
    a1 = aidxou(ia,1);

    f1ou = (1-chi)*f0;
    c1ou = cdecou(a1,1);
    dcou = log(max(c1ou,0.00001)) - log(max(c0,0.00001));
    tmp = popsamp(dcou',f1ou',popfac);   countstep = length(tmp);
    vvou(count:count+countstep-1,1) = tmp;   count = count + countstep;
    
%     f1r = chi*f0;
%     c1r = cdecr(a1,1);
%     dcr = log(max(c1r,0.00001)) - log(max(c0,0.00001));
%     tmp = popsamp(dcr',f1r',popfac);   countstep = length(tmp);
%     vvou(count:count+countstep-1,1) = tmp;   count = count + countstep;
    
  end
  
end
vvou(count+1:end) = [];

vvo = [ vvoe ; vvou ];
stdoExtd = std(vvo);
vv = [ vvy ; vvo ];
stdExtd = std(vv);


%%   Part 4. Dispersion of consumption growth rates: Baseline model.

vvye = zeros(1000000,1);   count = 1;

for itau = 1 : T
for iz = 1 : N
  if ZZye(itau,iz) > 0

    f0 = ZZye(itau,iz);
    c0 = w(itau,iz) + ra;

    f1ye = f0 * P_a(iz,:);
    tmp  = split(min(itau+1,T),:);
    sepy = sum(f1ye.*(tmp == 1),2);
    f1ye = (tmp < 1).*f1ye;
    c1ye = w(min(itau+1,T),:) + ra;
    dcye = log(max(c1ye,0.00001)) - log(max(c0,0.00001));
    tmp = popsamp(dcye',f1ye',popfac);   countstep = length(tmp);
    vvye(count:count+countstep-1,1) = tmp;   count = count + countstep;
    
    f1yu = sepy + (1-gamma)*delta*f0;
    c1yu = btildey(min(itau+1,T),1) + ra;
    dcyu = log(max(c1yu,0.00001)) - log(max(c0,0.00001));
    tmp = popsamp(dcyu',f1yu',popfac);   countstep = length(tmp);
    vvye(count:count+countstep-1,1) = tmp;   count = count + countstep;
    
%     f1oe = f0 * gamma * P_a(iz,:);
%     tmp = splito(min(itau+1,T),:);
%     sepo = sum(f1oe.*(tmp == 1),2);
%     f1oe = (tmp < 1).*f1oe;
%     c1oe = cdecoe(a1,:,min(itau+1,T));
%     dcoe = log(max(c1oe,0.00001)) - log(max(c0,0.00001));
%     tmp = popsamp(dcoe',f1oe',popfac);   countstep = length(tmp);
%     vvye(count:count+countstep-1,1) = tmp;   count = count + countstep;
    
  end
  
end
end
vvye(count+1:end) = [];

vvyu = zeros(500000,1);   count = 1;

for itau = 1 : T

  if ZZyu(itau,1) > 0

    f0 = ZZyu(itau,1);
    c0 = btildey(itau,1) + ra;

    f1ye = (1-gamma)*q*theta*f0;
    c1ye = w(1,z0idx) + ra;
    dcye = log(max(c1ye,0.00001)) - log(max(c0,0.00001));
    tmp = popsamp(dcye',f1ye',popfac);   countstep = length(tmp);
    vvyu(count:count+countstep-1,1) = tmp;   count = count + countstep;
    
    f1yu = (1-gamma)*(1-q*theta)*f0;
    c1yu = btildey(itau,1) + ra;
    dcyu = log(max(c1yu,0.00001)) - log(max(c0,0.00001));
    tmp = popsamp(dcyu',f1yu',popfac);   countstep = length(tmp);
    vvyu(count:count+countstep-1,1) = tmp;   count = count + countstep;
    
  end
  
end
vvyu(count+1:end) = [];

vvy = [ vvye ; vvyu ];
stdyBase = std(vvy);

vvoe = zeros(1000000,1);   count = 1;

for itau = 1 : T
for iz = 1 : N
  if ZZoe(itau,iz) > 0

    f0 = ZZoe(itau,iz);
    c0 = wo(itau,iz) + ra;

    f1oe = f0 * (1-chi) * P_b(iz,:);
    tmp  = splito(min(itau+1,T),:);
    sepo = sum(f1oe.*(tmp == 1),2);
    f1oe = (tmp < 1).*f1oe;
    c1oe = wo(min(itau+1,T),:) + ra;
    dcoe = log(max(c1oe,0.00001)) - log(max(c0,0.00001));
    tmp = popsamp(dcoe',f1oe',popfac);   countstep = length(tmp);
    vvoe(count:count+countstep-1,1) = tmp;   count = count + countstep;
    
    f1ou = sepo + (1-chi)*delta*f0;
    c1ou = btildeoA(min(itau+1,T),1) + ra;
    dcou = log(max(c1ou,0.00001)) - log(max(c0,0.00001));
    tmp = popsamp(dcou',f1ou',popfac);   countstep = length(tmp);
    vvoe(count:count+countstep-1,1) = tmp;   count = count + countstep;
    
  end
  
end
end
vvoe(count+1:end) = [];

vvou  = zeros(500000,1);   count = 1;

for itau = 1 : T

  if ZZou(itau,1) > 0

    f0 = ZZou(itau,1);
    c0 = btildeoA(itau,1) + ra;

    f1ou = (1-chi)*f0;
    c1ou = btildeoA(itau,1) + ra;
    dcou = log(max(c1ou,0.00001)) - log(max(c0,0.00001));
    tmp = popsamp(dcou',f1ou',popfac);   countstep = length(tmp);
    vvou(count:count+countstep-1,1) = tmp;   count = count + countstep;
    
  end
  
end

vvou(count+1:end) = [];
stdvvou = std(vvou);

vvo = [ vvoe ; vvou ];
stdoBase = std(vvo);
vv = [ vvy ; vvo ];
stdBase = std(vv);

fprintf('\n   Dispersion of consumption growth rates      Extended model     Baseline model');
fprintf('\n   -----------------------------------------------------------------------------');
fprintf('\n   On average among        - All workers          %5.3f               %5.3f     ',stdExtd,stdBase);
fprintf('\n                           - Young workers        %5.3f               %5.3f     ',stdyExtd,stdyBase);
fprintf('\n                           - Older workers        %5.3f               %5.3f     \n',stdoExtd,stdoBase);