% evalwelfare.m
%
% To run this script, proceed as follows. First solve the model with the benchmark EPL scheme and save the results as a
% file in a generic directory (it is enough to save the variables that are needed and listed below). Then solve the
% model with the unified EPL scheme and do the same. Then run this script. I do not save the 2 files "benchsavings.mat"
% and "optimsavings.mat" because they occupy >50 MB of space.

clearvars

load benchsavings.mat vye vyu voe vou fye fyu foe fou r gamma chi eta phi T na N

vye0 = vye;   vyu0 = vyu;   voe0 = voe;   vou0 = vou;   phi0 = phi;
fye0 = fye;   fyu0 = fyu;   foe0 = foe;   fou0 = fou;

load optimsavings.mat vye vyu voe vou fye fyu foe fou phi

vye1 = vye;   vyu1 = vyu;   voe1 = voe;   vou1 = vou;   phi1 = phi;
fye1 = fye;   fyu1 = fyu;   foe1 = foe;   fou1 = fou;

clear vye vyu voe vou fye fyu foe fou phi

tmp = 1/(1-(1/(1+r))*(1-gamma))*(1+(1/(1+r))*gamma/(1-(1/(1+r))*(1-chi)));

dWnewb = - (100*(((vyu0(1,1) + (1/(1-eta))*tmp)/(vye1(1,1) + (1/(1-eta))*tmp))^(1/(1-eta)) - 1))

% The following part takes care of the fact that workers who are currently employed under the old scheme retain their
% entitlements to SP. Hence, I need to find the tenure that corresponds to the same SP. Since the new EPL scheme is less
% generous, I cannot proceed by going from 0 (old scheme) to 1 (new scheme), because for long-tenured workers there is
% no corresponding tenure that has the same SP under the new scheme. Therefore, I simply go in the opposite direction
% from 1 to 0. This gives approximately the same welfare effects. Importantly, the welfare effects are based on a
% short-run perspective, i.e. using the stationary distribution of the pre-reform steady state.

vye0x = zeros(na,N,T);
voe0x = zeros(na,N,T);

for itau = 1 : T
  idx = interp1(phi0(1:T-5),1:T-5,phi1(itau),'nearest');   % Look for tenure that has the same SP
  vye0x(:,:,itau) = vye0(:,:,idx);
  voe0x(:,:,itau) = voe0(:,:,idx);  
end

dWye = 100*(((vye0x + (1/(1-eta))*tmp)./(vye1 + (1/(1-eta))*tmp)).^(1/(1-eta)) - 1);     % Young employed
dWyu = 100*(((vyu0  + (1/(1-eta))*tmp)./(vyu1 + (1/(1-eta))*tmp)).^(1/(1-eta)) - 1);     % Young unemployed
dWoe = 100*(((voe0x + (1/(1-eta))*tmp)./(voe1 + (1/(1-eta))*tmp)).^(1/(1-eta)) - 1);     % Old employed
dWou = 100*(((vou0  + (1/(1-eta))*tmp)./(vou1 + (1/(1-eta))*tmp)).^(1/(1-eta)) - 1);     % Old nonemployed (ignored)

dWavg   = - (sum(sum(sum(dWye.*fye0))) + sum(dWyu.*fyu0) + sum(sum(sum(dWoe.*foe0))))/...
            (sum(sum(sum(fye0))) + sum(fyu0)+sum(sum(sum(foe0))))
dWavgy  = - (sum(sum(sum(dWye.*fye0))) + sum(dWyu.*fyu0)/(sum(sum(sum(fye0))) + sum(fyu0)))
dWavgo  = - (sum(sum(sum(dWoe.*foe0))) + sum(dWou.*fou0)/(sum(sum(sum(foe0))) + sum(fou0)));   % Ignored
dWavgoe = -  sum(sum(sum(dWoe.*foe0)))/sum(sum(sum(foe0)))     % Consider only employed workers