% Comparative statics.

clearvars
tic

nz     = 101;                stdz  = 0.044;                % Productivity process: Grid size and standard deviation
z0idx  = 29;                 kappa = 9.5000;               % Initial productivity index / Payroll tax in percent
b      = 0.2203;             k     = 0.2204;               % Unemployment income younger workers / Vacancy cost
bo     = 0.1617;             wavg0 = 0.3754;               % Unemployment income older workers / Guess for avg wage

uniEPL = 0;                  QUITS = 1;                    % Benchmark (0) or Unified EPL (1) / Quits: Entitlement to SP
chi    = 1/40;               gamma = 1/120;                % May be adjusted
upsil  = 1;                  rry   = 0.65;                 % May be adjusted / Replacement rate: May be adjusted

if upsil == 0.5
  b = 0.2432;  bo = 0.1444;  k = 0.2624;  kappa = 8.37;  wavg0 = 0.399;  stdz = 0.055;  z0idx = 30;   % tau_u = 17,rho_u = 33
elseif QUITS == 0
  b = 0.2370;  bo = 0.1862;  k = 0.2246;  kappa = 9.84;  wavg0 = 0.409;  stdz = 0.044;  z0idx = 32;   % tau_u = 6, rho_u = 16
elseif (chi == 1/20)
  b = 0.2600;  bo = 0.2283;  k = 0.2280;  kappa = 6.90;  wavg0 = 0.454;  stdz = 0.044;  z0idx = 37;   % tau_u = 6, rho_u = 12
elseif (chi == 1/60)    
  b = 0.1948;  bo = 0.1337;  k = 0.2354;  kappa = 14.10; wavg0 = 0.325;  stdz = 0.044;  z0idx = 25;   % tau_u = 7, rho_u = 34
elseif (rry == 0.50)
  b = 0.1636;  bo = 0.1482;  k = 0.2184;  kappa = 9.83;  wavg0 = 0.327;  stdz = 0.044;  z0idx = 23;   % tau_u = 2, rho_u = 17
elseif (rry == 0.65)
  b = 0.2803;  bo = 0.1752;  k = 0.2239;  kappa = 9.76;  wavg0 = 0.422;  stdz = 0.044;  z0idx = 35;   % tau_u = 8, rho_u = 24
end

init;                                                      % Initialization
vfi;                                                       % Solve value function iteration
simul;                                                     % Compute stationary distribution

for calibrationit = 1 : 3
  critgov = 1;                                             % Initialize
  while abs(critgov) > 1e-5                                % Balance the government budget
    initx;   vfi;   simul;                                 % Solve the model
    critgov = (Rev-Ben)/Rev;                               % Excess revenues
    kappa = kappa - 0.10 * critgov;                        % Adjust payroll tax rate
    wavg0 = 0.5*wavg + 0.5*wavg0;                          % Adjust guess for average wage
  end
  b  = b  - 0.28 * (b/wavgy  - rry);                       % Calibration: Set new value for b
  bo = bo - 0.28 * (bo/wavgo - 0.4540);   fprintf('x ')    % Calibration: Set new value for bo
end
  
showstats;
toc