!---------------------------------------------------------------------------------------------------------------------------------------------------
!	module distribution: compute the time-invariant distribution
!---------------------------------------------------------------------------------------------------------------------------------------------------

module distribution

	use globals
	use misc_functions
	
	implicit none
	
contains


!	compute next period distribution

subroutine update_distribution()

	double precision :: NewLE(NT,NZ,NC,2), NewLU(NT,NC,3), norm
	
	!	initialize size distribution
	
	norm = sum(LE) + sum(LU)
	LE  = LE/norm
	LU  = LU/norm
	
	!	transpose transition matrix
	
	P = transpose(P)
	
	!	set new measures to zero
	
	NewLE = 0.0d+0
	NewLU = 0.0d+0
		
	!	job-finding rate this period
	
	jfind = min(A*theta(ip)**(1.0d+0-psi),theta(ip),1.0d+0)
	
	!	employment
	
	NewLE(1,iz0,NC,1) = (1.0d+0-gamma)*jfind*sum(LU(:,:,1))
	
	do ic = 1, NC
				
		do it = 2, NT-1
		
			NewLE(it,:,ic,1) = (1.0d+0-delta)*(1.0d+0-gamma)*matmul(P, LE(it-1,:,ic,1))
			NewLE(it,:,ic,2) = (1.0d+0-delta)*(gamma*matmul(P, LE(it-1,:,ic,1)) + (1.0d+0-khi)*matmul(P, LE(it-1,:,ic,2)))
			
		end do
		
		NewLE(NT,:,ic,1) = (1.0d+0-delta)*(1.0d+0-gamma)*matmul(P, LE(NT-1,:,ic,1) + LE(NT,:,ic,1))
		NewLE(NT,:,ic,2) = (1.0d+0-delta)*(gamma*matmul(P, LE(NT-1,:,ic,1) + LE(NT,:,ic,1)) &
												+ (1.0d+0-khi)*matmul(P, LE(NT-1,:,ic,2) + LE(NT,:,ic,2)))
		
		
		!	separations
		
		do it = 1, NT
		
			if (sepy(ip,it,ic) .le. NZ) then
			
				NewLU(it,ic,1) = NewLU(it,ic,1) + sum(NewLE(it,sepy(ip,it,ic):NZ,ic,1))
				NewLE(it,sepy(ip,it,ic):NZ,ic,1) = 0.0d+0
				
			end if
			
			if (sepo(ip,it,ic) .le. NZ) then
			
				NewLU(it,ic,3) = NewLU(it,ic,3) + sum(NewLE(it,sepo(ip,it,ic):NZ,ic,2))
				NewLE(it,sepo(ip,it,ic):NZ,ic,2) = 0.0d+0
			
			end if
			
		end do
		
	end do
		
	!	unemployment
	
	NewLU(1,NC,1) = NewLU(1,NC,1) + khi*(sum(LU(:,:,2:3))+sum(LE(:,:,:,2))) + (1.0d+0-gamma)*delta*sum(LE(:,:,:,1))*(1.0d+0-delta_sp)
	NewLU(:,NC,1) = NewLU(:,NC,1) + (1.0d+0-gamma)*((1.0d+0-jfind)*sum(LU(:,:,1), dim = 2) &
								  + delta*sum(sum(LE(:,:,:,1), dim = 3), dim = 2)*delta_sp)
	NewLU(:,:,2) = NewLU(:,:,2) + gamma*LU(:,:,1) + (1.0d+0-khi)*LU(:,:,2)
	
	do ic = 1, NC
	
		NewLU(1,ic,3) = NewLU(1,ic,3) + (gamma*delta*sum(LE(:,:,ic,1)) + (1.0d+0-khi)*delta*sum(LE(:,:,ic,2)))*(1.0d+0-delta_sp)
		NewLU(:,ic,3) = NewLU(:,ic,3) + (1.0d+0-khi)*LU(:,ic,3) &
								+ (gamma*delta*sum(LE(:,:,ic,1), dim = 2) + (1.0d+0-khi)*delta*sum(LE(:,:,ic,2), dim = 2))*delta_sp
								
	end do
	
	!	update distribution
	
	LE = NewLE
	LU = NewLU
	
	!	transpose transition matrix back
	
	P = transpose(P)
	
end subroutine

end module