% Computes steady-state equilibrium for benchmark model.

clearvars
tic

nz     = 101;                stdz  = 0.044;                % Productivity process: Grid size and standard deviation
z0idx  = 29;                 kappa = 9.5000;               % Initial productivity index / Payroll tax in percent
b      = 0.2203;             k     = 0.2204;               % Unemployment income younger workers / Vacancy cost
bo     = 0.1617;             wavg0 = 0.3754;               % Unemployment income older workers / Guess for avg wage
uniEPL = 0;                  QUITS = 1;                    % Benchmark (0) or Unified EPL (1) / Quits entitle to EPL

init;                                                      % Initialization
vfi;                                                       % Value function iteration
simul;                                                     % Compute stationary distribution

for calibrationit = 1 : 3                                  % Uncomment to calibrate b and bo
  critgov = 1;                                             % Initialize
  while abs(critgov) > 1e-5                                % Balance the government budget
    initx;   vfi;   simul;                                 % Solve the model
    critgov = (Rev-Ben)/Rev;                               % Excess revenues
    kappa   = kappa - 0.10 * critgov;                      % Adjust payroll tax rate
    wavg0   = 0.5*wavg + 0.5*wavg0;                        % Adjust guess for average wage
  end
  b  = b  - 0.28 * (b/wavgy  - 0.58);                      % Calibration: Set new value for b
  bo = bo - 0.28 * (bo/wavgo - 0.4540);   fprintf('x ')    % Calibration: Set new value for bo
end

showstats;
toc