!---------------------------------------------------------------------------------------------------------------------------------------------------
!	module misc_functions: various functions and subroutines
!---------------------------------------------------------------------------------------------------------------------------------------------------

module misc_functions

	use globals
	
	implicit none

contains


!	load model parameters

subroutine initial_parameters()

	double precision :: PInv(NZ,NZ), zmin, zmax, zinit, step, zGridInv(NZ), skip
	integer :: iz_p

	!	model specification
	
	open(1, file = 'results/reform.txt', status = 'old')
	read(1, *) reform_sp
	close(1)
		
	!	read parameters
		
	open(1, file = 'results/params.txt', status = 'old')
	
	read(1, *) delta_sp
	read(1, *) young_sp
	read(1, *) model_period
	read(1, *) rho
	rho = 1.0d+0/(1.0d+0 + rho)
	
	read(1, *) sigma
	read(1, *) leis
	
	!	life-cycle
	
	read(1, *) gamma
	read(1, *) khi
	
	gamma = 1.0d+0/(dble(model_period)*gamma)
	khi   = 1.0d+0/(dble(model_period)*khi)
		
	read(1, *) nu
	read(1, *) By
	read(1, *) Bo
	read(1, *) k
	
	!	retirement income
	
	read(1, *) R
	if (R .gt. 0.0d+0) R  = (utils(R) + leis)/(1.0d+0-rho*(1.0d+0-1.0d+0/dble(model_period*20)))
	
	!	productivity process

	read(1, *) zinit
	read(1, *) prob_z
	read(1, *) sig_z
		
	zmin  = 0.0d+0
	zmax  = 1.0d+0
	step  = (zmax-zmin)/dble(NZ-1)
	do iz = 1, NZ
		zGridInv(iz) = zmin + step*dble(iz-1)
	end do
		
	iz0 = locate(zinit, zGridInv)

	print *, ' '
	print *, 'Note: initial z', iz0, real(zGridInv(iz0))
	print *, ' '
	
	PInv = 0.0d+0
	do iz = 1, NZ
		PInv(iz,iz) = 1.0d+0 - prob_z
  		PInv(iz, 1) = PInv(iz,1)+ prob_z*normal_cdf(0.5d+0*(zGridInv(1)+zGridInv(2))-zGridInv(iz),0.0d+0,sig_z)
  		do iz_p = 2, NZ-1
    		PInv(iz,iz_p) = PInv(iz,iz_p) + prob_z*(normal_cdf(0.5d+0*(zGridInv(iz_p)+zGridInv(iz_p+1)) - zGridInv(iz), 0.0d+0 ,sig_z) &
    											  - normal_cdf(0.5d+0*(zGridInv(iz_p-1)+zGridInv(iz_p)) - zGridInv(iz) ,0.0d+0 ,sig_z))
  		end do
  		PInv(iz,NZ) = PInv(iz,NZ) + prob_z*(1.0d+0-normal_cdf(0.5d+0*(zGridInv(NZ-1)+zGridInv(NZ))-zGridInv(iz),0.0d+0,sig_z))
	end do
	

	!	inverse order
	do iz = 1, NZ
		zGrid(NZ+1-iz) = zGridInv(iz)
	end do	
	iz0 = NZ+1-iz0
	
	do iz = 1, NZ
	do iz_p = 1, NZ
		P(NZ+1-iz,NZ+1-iz_p) = PInv(iz,iz_p)
	end do
	end do
	
	
	!	matching parameters
	
	read(1, *) delta
	read(1, *) A
	read(1, *) psi
	read(1, *) beta
	
	close(1)
	
	!	annuity factor for old workers
	
	annfacO = (1.0d+0-rho)/(1.0d+0-rho**(1.0d+0/khi))
	
	!	value of posting vacancies
	
	open(1, file = 't0/eq_tuple.txt', status = 'old')
	read(1, *) skip
	read(1, *) skip
	read(1, *) skip
	read(1, *) skip
	read(1, *) k
	close(1)	
		
end subroutine


!	locate x in xgrid

integer function locate(x, xgrid)
	implicit none

	double precision, intent(in) :: x, xgrid(:)
	
	integer :: N, ju, jm, jl, jndx
	
	N  = size(xgrid)
	jl = 0
	ju = N+1
	do while (ju - jl > 1)
		jm = (ju + jl)/2
		if (x .ge. xgrid(jm)) then
			jl = jm
		else
			ju = jm
		end if
	end do

	if (x .eq. xgrid(1)) then 
		jndx = 1
	else if (x .eq. xgrid(N)) then
		jndx = N - 1
	else 
		jndx = jl
	end if
	locate = jndx

end function


!	cdf of normal distribution (N(mu,std^2) evaluated at x)

double precision function normal_cdf(x, mu, std)
	implicit none
	double precision, intent(in) :: x, mu, std	
	normal_cdf = 1.0d+0 - erfc((x-mu)/(std*sqrt(2.0d+0)))/2.0d+0

end function


!	set the layoff payments functions

subroutine initialize_layoff_payments()
	
	double precision :: skip, phi_tmp(NT)
	integer :: elig

	!	at t0
	ic = 1
	open(1, file = 't0/phi_function.txt', status = 'old')
	do it = 1, NT
		read(1, *) phi(it,ic), skip
	end do
	close(1)
		
	!	at t1
	ic = NC
	open(1, file = 't1/phi_function.txt', status = 'old')
	do it = 1, NT
		read(1, *) phi(it,ic), skip
	end do
	close(1)
	
	if (reform_sp .gt. 0.0d+0) then
	
		elig = minloc(phi(:, NC), dim = 1, mask = phi(:, NC) .gt. 0.0d+0)
		elig = max(1, min(elig, NT))
		
		phi_tmp = phi(:,1)
		
		do it = 1, NT-1
		
			phi_tmp(it) = phi(max(1, it-(ip-1)),1)
			if (it-(ip-1) .ge. elig) phi_tmp(it) = phi_tmp(it) + phi(min(NT, elig+(ip-1)),NC)
			
		end do
		phi_tmp   = min(phi(NT,1), phi_tmp)
		phi(:, 1) = phi_tmp
	
	end if
		
	!	value function for old workers
	
	do ic = 1, NC
	do it = 1, NT
		Uo(it,ic) = (utils(Bo + nu*annfacO*phi(it,ic)) + leis + rho*khi*R)/(1.0d+0-rho*(1.0d+0-khi))
	end do
	end do
		
end subroutine


!	initialize a time-path for kappa

subroutine initialize_time_path()
	
	double precision :: skip

	open(1, file = 't0/eq_tuple.txt', status = 'old')
	read(1, *) skip
	read(1, *) kappa(1)
	close(1)
	
	open(1, file = 't1/eq_tuple.txt', status = 'old')
	read(1, *) skip
	read(1, *) kappa(NP)
	close(1)

	kappa(2) = kappa(NP)*0.50d+0 + kappa(1)*0.50d+0
	kappa(3) = kappa(NP)*0.75d+0 + kappa(1)*0.25d+0
	kappa(4:NP-1) = kappa(NP)
	
	open(1, file = 'results/theta_kappa_path.txt', status = 'old')
	do ip = 1, NP
		read(1, *) skip, kappa(ip)
	end do
	close(1)
	
	
end subroutine


!	initialize wage and value functions

subroutine initialize_various_functions()

	double precision :: skip1, skip2
	
	!	load contract at t1

	ip = NP
	ic = NC
	
	!	wages
	
	open(1, file = 't1/wage_y.txt', status = 'old')
	open(2, file = 't1/wage_o.txt', status = 'old')
	do it = 1, NT
		read(1, *) (wwy(ip,it,iz,ic), iz = NZ, 1, -1)
		read(2, *) (wwo(ip,it,iz,ic), iz = NZ, 1, -1)
	end do
	close(1)
	close(2)
		
	!	value functions

	ip = NP
	ic = NC
		
	open(1, file = 't1/value_Wy.txt', status = 'old')
	open(2, file = 't1/value_Wo.txt', status = 'old')
	open(3, file = 't1/value_Jy.txt', status = 'old')
	open(4, file = 't1/value_Jo.txt', status = 'old')
	do it = 1, NT
		read(1, *) (Wy(it,iz,ic), iz = NZ, 1, -1)
		read(2, *) (Wo(it,iz,ic), iz = NZ, 1, -1)
		read(3, *) (Jy(it,iz,ic), iz = NZ, 1, -1)
		read(4, *) (Jo(it,iz,ic), iz = NZ, 1, -1)
	end do
	close(1)
	close(2)
	close(3)
	close(4)
	
	open(1, file = 't1/value_U.txt', status = 'unknown')
	do it = 1, NT
		read(1, *) UyP(it,ic), skip1, skip2
	end do
	close(1)

	!	equilibrium tuple
	
	open(1, file = 't1/eq_tuple.txt', status = 'old')
	read(1, *) theta(NP)
	read(1, *) kappa(NP)
	read(1, *) EdurP
	close(1)
	
	!	value of starting a job at t1
	
	Wn = Wy(1,iz0,NC)
	
	!	load contract at t0
		
	ip = NP
	ic = 1
	
	!	wages
	
	open(1, file = 't0_in_t1/wage_y.txt', status = 'old')
	open(2, file = 't0_in_t1/wage_o.txt', status = 'old')
	do it = 1, NT
		read(1, *) (wwy(ip,it,iz,ic), iz = NZ, 1, -1)
		read(2, *) (wwo(ip,it,iz,ic), iz = NZ, 1, -1)
	end do
	close(1)
	close(2)
		
	!	value functions

	ip = NP
	ic = 1
		
	open(1, file = 't0_in_t1/value_Wy.txt', status = 'old')
	open(2, file = 't0_in_t1/value_Wo.txt', status = 'old')
	open(3, file = 't0_in_t1/value_Jy.txt', status = 'old')
	open(4, file = 't0_in_t1/value_Jo.txt', status = 'old')
	do it = 1, NT
		read(1, *) (Wy(it,iz,ic), iz = NZ, 1, -1)
		read(2, *) (Wo(it,iz,ic), iz = NZ, 1, -1)
		read(3, *) (Jy(it,iz,ic), iz = NZ, 1, -1)
		read(4, *) (Jo(it,iz,ic), iz = NZ, 1, -1)
	end do
	close(1)
	close(2)
	close(3)
	close(4)
	
	open(1, file = 't0_in_t1/value_U.txt', status = 'unknown')
	do it = 1, NT
		read(1, *) Uy(it,ic), skip1, skip2
	end do
	close(1)
		
	!	initialize distribution function

	ip = 1
	ic = 1
	
	LE = 0.0d+0
	LU = 0.0d+0
	open(1, file = 't0/dist_LEy.txt', status = 'old')
	open(2, file = 't0/dist_LEo.txt', status = 'old')
	open(3, file = 't0/dist_LU.txt', status = 'old')
	do it = 1, NT
		read(1, *) (LE(it,iz,ic,1), iz = NZ, 1, -1)
		read(2, *) (LE(it,iz,ic,2), iz = NZ, 1, -1)
		read(3, *) LU(it,ic,1), LU(it,ic,2), LU(it,ic,3)
	end do
	close(1)
	close(2)
	close(3)

end subroutine


!	utility function

double precision function utils(cc)
	
	double precision, intent(in) :: cc
	if (sigma .eq. 1.0d+0) utils = log(max(cc,TINY))
	if (sigma .gt. 1.0d+0) utils = (max(cc,TINY)**(1.0d+0-sigma) - 1.0d+0)/(1.0d+0-sigma)
	
end function


!	marginal utility

double precision function umarg(cc)
	
	double precision, intent(in) :: cc
	umarg = max(cc,TINY)**(-sigma)
	
end function


!	inverse utility function

double precision function inv_utils(uu)
	
	double precision, intent(in) :: uu
	if (sigma .eq. 1.0d+0) inv_utils = exp(uu)
	if (sigma .gt. 1.0d+0) inv_utils = (1.0d+0+(1.0d+0-sigma)*uu)**(1.0d+0/(1.0d+0-sigma))
	
end function


end module