% modelplot.m
%
% This is a script that creates a bunch of figures. Select only one part at a time.

clearvars

DO_PART1 = 1;                % EPL scheme, wage profiles for young and older workers (benchmark)
DO_PART2 = 0;                % Average wages and productivity thresholds (benchmark)
DO_PART3 = 0;                % Wage profiles for young and old workers (unified EPL); EPL scheme (benchmark vs unified)
DO_PART4 = 0;                % Average wage for young and older workers (unified EPL)
DO_PART5 = 0;                % Productivity thresholds (unified EPL)


%%   Part 1:   EPL scheme, wage profiles for young and older workers. (Benchmark economy)

if DO_PART1 == 1

  load Results\bench.mat

  rect  = [80 80 1200 700];
  figure('Position',rect)
  plot(1:T,phi(1:T)./(wavg0*4),'b-','LineWidth',3);   grid on
  set(gca,'FontSize',18,'TickLabelInterpreter', 'latex');
  title({'\textbf{Benchmark EPL scheme}'}, 'Interpreter', 'latex','FontSize',21)
  xlabel({'Tenure in quarters'}, 'Interpreter', 'latex', 'Fontsize', 20)
  ylabel({'Multiple of avg yearly wage (young)'}, 'Interpreter', 'latex', 'Fontsize', 20)
  axis([ 0 120 0 4])
  
  zidx1 = 29;   zidx2 = 50;   zidx3 = 101;
  
  figure('Position',rect)
  plot(1:T,w(1:T,zidx1),'r-',1:T,w(1:T,zidx2),'b:','LineWidth',4);   hold on
  plot(1:T,w(1:T,zidx3),'m-','LineWidth',2);   grid on
  set(gca,'FontSize',18,'TickLabelInterpreter', 'latex');
  title({'\textbf{Wage function - Young workers}'}, 'Interpreter', 'latex','FontSize',21)
  legend({'$~z = 0.32~$','$~z = 0.50~$','$~z = 1.00~$'},'Interpreter', 'latex','Location','Southeast','FontSize',20)
  xlabel({'Tenure in quarters'}, 'Interpreter', 'latex', 'Fontsize', 20)
  axis([ 0 120 0.15 0.6])  

  figure('Position',rect)
  plot(1:T,wo(1:T,zidx1),'r-',1:T,wo(1:T,zidx2),'b:','LineWidth',4);   hold on
  plot(1:T,wo(1:T,zidx3),'m-','LineWidth',2);   grid on
  set(gca,'FontSize',18,'TickLabelInterpreter', 'latex');
  title({'\textbf{Wage function - Older workers}'}, 'Interpreter', 'latex','FontSize',21)
  legend({'$~z = 0.32~$','$~z = 0.50~$','$~z = 1.00~$'},'Interpreter', 'latex','Location','Southeast','FontSize',20)
  xlabel({'Tenure in quarters'}, 'Interpreter', 'latex', 'Fontsize', 20)
  axis([ 0 120 0.15 0.6])

end


%%   Part 2:   Average wages and productivity thresholds (benchmark).

if DO_PART2 == 1

  load Results\bench.mat

  rect  = [80 80 1200 700];
  figure('Position',rect)
  plot(1:T,sum(w.*ZZye,2)./sum(ZZye,2),'LineWidth',3);   grid on
  set(gca,'FontSize',18,'TickLabelInterpreter', 'latex');
  title({'\textbf{Average wage - Young workers}'}, 'Interpreter', 'latex','FontSize',21)
  xlabel({'Tenure in quarters'}, 'Interpreter', 'latex', 'Fontsize', 20)
  axis([ 0 120 0.15 0.5])
  
  figure('Position',rect)
  plot(1:T,sum(wo.*ZZoe,2)./sum(ZZoe,2),'LineWidth',3);   grid on
  set(gca,'FontSize',18,'TickLabelInterpreter', 'latex');
  title({'\textbf{Average wage - Older workers}'}, 'Interpreter', 'latex','FontSize',21)
  xlabel({'Tenure in quarters'}, 'Interpreter', 'latex', 'Fontsize', 20)
  axis([ 0 120 0.15 0.5])
  
  zthres = zeros(T,2);
  for t = 1 : T
    zthres(t,1) = z(1+sum(split(t,:),2));
    zthres(t,2) = z(1+sum(splito(t,:),2));
  end

  rect  = [80 80 1200 700];
  figure('Position',rect)
  plot(1:T,zthres(:,1),'b-','LineWidth',4);   hold on  
  plot(1:T,zthres(:,2),'r-','LineWidth',2);   grid on
  set(gca,'FontSize',18,'TickLabelInterpreter', 'latex');
  title({'\textbf{Separation thresholds}'}, 'Interpreter', 'latex','FontSize',21)
  xlabel({'Tenure in quarters'}, 'Interpreter', 'latex', 'Fontsize', 20)
  ylabel({'Productivity $z$'}, 'Interpreter', 'latex', 'Fontsize', 20)
  axis([ 0 120 0 0.3 ])
  legend({'Young worker','Older worker'},'Interpreter', 'latex','Location','Southeast','FontSize',20)

end


%%   Part 3: Wage profiles for young and older workers; Firing cost function (Benchmark vs SOEC).

if DO_PART3 == 1
  
  load Results\bench.mat phi T wavg
  phiold  = phi;
  wavgold = wavg;
  load Results\optim.mat phi wavg
  
  rect  = [80 80 1200 700];
  figure('Position',rect)
  plot(1:T,phiold(1:T)./(wavgold*4),'r:','LineWidth',4);   hold on
  plot(1:T,phi./(wavg*4),'b-','LineWidth',3);   grid on
  set(gca,'FontSize',18,'TickLabelInterpreter', 'latex');
  title({'\textbf{EPL scheme}'}, 'Interpreter', 'latex','FontSize',21)
  legend({'Benchmark','Unified EPL'},'Interpreter', 'latex','Location','Southeast','FontSize',20)
  xlabel({'Tenure in quarters'}, 'Interpreter', 'latex', 'Fontsize', 20)
  ylabel({'Multiple of avg yearly wage (young)'}, 'Interpreter', 'latex', 'Fontsize', 20)
  axis([ 0 T 0 4 ])
  
  load Results\optim.mat
  
  zidx1 = 29;   zidx2 = 50;   zidx3 = 101;
  
  rect  = [80 80 1200 700];
  figure('Position',rect)
  plot(1:T,w(1:T,zidx1),'r-',1:T,w(1:T,zidx2),'b:',1:T,w(1:T,zidx3),'m--','LineWidth',3);   grid on
  set(gca,'FontSize',18,'TickLabelInterpreter', 'latex');
  title({'\textbf{Wage for young workers}'}, 'Interpreter', 'latex','FontSize',21)
  legend({'$~z = 0.32~$','$~z = 0.50~$','$~z = 1.00~$'},'Interpreter', 'latex','Location','Southeast','FontSize',20)
  xlabel({'Tenure in quarters'}, 'Interpreter', 'latex', 'Fontsize', 20)
  axis([ 0 120 0.2 0.60 ])

  figure('Position',rect)
  plot(1:T,wo(1:T,zidx1),'r-',1:T,wo(1:T,zidx2),'b:',1:T,wo(1:T,zidx3),'m--','LineWidth',3);   grid on
  set(gca,'FontSize',18,'TickLabelInterpreter', 'latex');
  title({'\textbf{Wage for older workers}'}, 'Interpreter', 'latex','FontSize',21)
  legend({'$~z = 0.32~$','$~z = 0.50~$','$~z = 1.00~$'},'Interpreter', 'latex','Location','Southeast','FontSize',20)
  xlabel({'Tenure in quarters'}, 'Interpreter', 'latex', 'Fontsize', 20)
  axis([ 0 120 0.2 0.60 ])
  
end


%%   Part 4: Average wage for young and older workers. (Unified EPL)

if DO_PART4 == 1
  
  load Results\bench.mat;   wI  = w;   woI  = wo;   ZZIye  = ZZye;   ZZIoe  = ZZoe;
  load Results\optim.mat;   wII = w;   woII = wo;   ZZIIye = ZZye;   ZZIIoe = ZZoe;
  
  wavgyI    = sum(wI.*ZZIye,2)./sum(ZZIye,2);
  wavgyII   = sum(wII.*ZZIIye,2)./sum(ZZIIye,2);
  wavgoI    = sum(woI.*ZZIoe,2)./sum(ZZIoe,2);
  wavgoII   = sum(woII.*ZZIIoe,2)./sum(ZZIIoe,2);
  wavgallI  = (sum(wI.*ZZIye,2)+sum(woI.*ZZoe,2))./(sum(ZZIye,2)+sum(ZZIoe,2));
  wavgallII = (sum(wII.*ZZIIye,2)+sum(woII.*ZZoe,2))./(sum(ZZIIye,2)+sum(ZZIIoe,2));
  
  rect  = [80 80 1200 700];
  figure('Position',rect)
  plot(1:T,wavgyI,'r--',1:T,wavgyII,'b-','LineWidth',3);   grid on
  set(gca,'FontSize',18,'TickLabelInterpreter', 'latex');
  title({'\textbf{Average wage - Young workers}'}, 'Interpreter', 'latex','FontSize',21)
  xlabel({'Tenure in quarters'}, 'Interpreter', 'latex', 'Fontsize', 20)
  legend({'Benchmark EPL','Unified EPL'},'Interpreter', 'latex','Location','Southeast','FontSize',20)
  axis([ 0 120 0.15 0.50 ])
  
  figure('Position',rect)
  plot(1:T,wavgoI,'r--',1:T,wavgoII,'b-','LineWidth',3);   grid on
  set(gca,'FontSize',18,'TickLabelInterpreter', 'latex');
  title({'\textbf{Average wage - Older workers}'}, 'Interpreter', 'latex','FontSize',21)
  xlabel({'Tenure in quarters'}, 'Interpreter', 'latex', 'Fontsize', 20)
  legend({'Benchmark EPL','Unified EPL'},'Interpreter', 'latex','Location','Southeast','FontSize',20)
  axis([ 0 120 0.15 0.50 ])
    
end


%%   Part 5: Productivity thresholds. (Unified EPL)

if DO_PART5 == 1
  
  load Results\bench.mat

  zthres = zeros(T,2);
  for t = 1 : T
    zthres(t,1) = z(1+sum(split(t,:),2));
    zthres(t,2) = z(1+sum(splito(t,:),2));
  end  
  zthresI = zthres;
  
  load Results\optim.mat

  zthres = zeros(T,2);
  for t = 1 : T
    zthres(t,1) = z(1+sum(split(t,:),2));
    zthres(t,2) = z(1+sum(splito(t,:),2));
  end  
  zthresII = zthres;

  rect  = [80 80 1200 700];
  figure('Position',rect)
  plot(1:T,zthresI(:,1),'r:','LineWidth',4);   hold on
  plot(1:T,zthresII(:,1),'b-','LineWidth',3);   grid on
  set(gca,'FontSize',18,'TickLabelInterpreter', 'latex');
  title({'\textbf{Separation thresholds - Young worker}'}, 'Interpreter', 'latex','FontSize',21)
  xlabel({'Tenure in quarters'}, 'Interpreter', 'latex', 'Fontsize', 20)
  ylabel({'Productivity $z$'}, 'Interpreter', 'latex', 'Fontsize', 20)
  axis([ 0 120 0 0.35])
  legend({'Benchmark EPL','Unified EPL'},'Interpreter', 'latex','Location','Southeast','FontSize',20)
  
end