% montecarlo.m
%
% This script runs a Monte Carlo simulation for the benchmark model. (not used)

clearvars
load bench.mat
tic

WW = 20000;                                      % Number of workers
TT = 300;                                        % Number of periods

ageWW = zeros(TT,WW);                            % 1: young; 2: old; 9: retired
zWW   = zeros(TT,WW);                            % Productivity
eWW   = zeros(TT,WW);                            % Employment status: 1: empl; 2: nonempl; 3: old nonempl type B
tenWW = zeros(TT,WW);                            % Tenure
cWW   = zeros(TT,WW);                            % Consumption

Pc    = cumsum(P,2);                             % Form cumulative probabilities
JDc   = zeros(TT,1);

ageWW(1,:) = 1;                                  % Initial value for all newborn workers: 1 (young)
zWW(1,:)   = 1;                                  % Initial value for all newborn workers: Irrelevant
eWW(1,:)   = 2;                                  % Initial value for all newborn workers: 2 (unemployed)
tenWW(1,:) = 0;                                  % Initial value for all newborn workers: 0 (no tenure)
cWW(1,:)   = b;                                  % Initial value for all newborn workers: Unemployment benefits

for tt = 2 : TT                                  % Simulation period
  for i = 1 : WW                                 % Identity of worker

    if ageWW(tt-1,i) == 9                        % AGE: Already retired

      ageWW(tt,i) = 1;                           % Retired workers are replaced by newborns
      zWW(tt,i)   = 1;
      eWW(tt,i)   = 2;
      tenWW(tt,i) = 0;

    elseif ageWW(tt-1,i) == 2                    % AGE: Older worker

      if eWW(tt-1,i) == 2                        % ---EMPLOYMENT: Early retired (type A)

        if rand < chi                            % ------RETIREMENT SHOCK: yes
          ageWW(tt,i) = 1;                       % Retired workers are replaced by newborns
          zWW(tt,i)   = 1;
          eWW(tt,i)   = 2;
          tenWW(tt,i) = 0;
          cWW(tt,i)   = b;
        else                                     % ------RETIREMENT SHOCK: no
          ageWW(tt,i) = 2;
          zWW(tt,i)   = 1;
          eWW(tt,i)   = 2;
          tenWW(tt,i) = tenWW(tt-1,i);
          cWW(tt,i)   = btildeoA(min(1+tenWW(tt,i),T));
        end
        
      elseif eWW(tt-1,i) == 3                    % ---EMPLOYMENT: early retired (type B)

        if rand < chi                            % ------RETIREMENT SHOCK: yes
          ageWW(tt,i) = 1;                       % Retired workers are replaced by newborns
          zWW(tt,i)   = 1;
          eWW(tt,i)   = 2;
          tenWW(tt,i) = 0;
          cWW(tt,i)   = b;
        else                                     % ------RETIREMENT SHOCK: no
          ageWW(tt,i) = 2;
          zWW(tt,i)   = 1;
          eWW(tt,i)   = 3;
          tenWW(tt,i) = tenWW(tt-1,i);
          cWW(tt,i)   = btildeoB(min(1+tenWW(tt,i),T));
        end
        
      else                                       % ---EMPLOYMENT: employed

        if rand < chi                            % ------RETIREMENT SHOCK: yes
          ageWW(tt,i) = 1;                       % Retired workers are replaced by newborns
          zWW(tt,i)   = 1;
          eWW(tt,i)   = 2;
          tenWW(tt,i) = 0;
          cWW(tt,i)   = b;
        else                                     % ------RETIREMENT SHOCK: no
          ageWW(tt,i) = 2;
          eWW(tt,i)   = 1;
          zWW(tt,i)   = 1 + sum(rand > Pc(zWW(tt-1,i),:));
          tenWW(tt,i) = min(T,tenWW(tt-1,i)+1);
          cWW(tt,i)   = wo(tenWW(tt,i),zWW(tt,i));
          if (splito(tenWW(tt,i),zWW(tt,i)) == 1) || (rand < delta),
            zWW(tt,i) = 1;
            eWW(tt,i) = 2;
            cWW(tt,i) = btildeoA(min(1+tenWW(tt,i),T));
            JDc(tt,1) = JDc(tt,1) + 1;
          end
        end

      end

    else                                         % AGE: Young worker

      if eWW(tt-1,i) == 2                        % ---EMPLOYMENT: unemployed

        if rand < gamma                          % ------AGE SHOCK: yes
          ageWW(tt,i) = 2;
          zWW(tt,i)   = 1;
          eWW(tt,i)   = 3;
          tenWW(tt,i) = tenWW(tt-1,i);
          cWW(tt,i)   = btildeoB(min(1+tenWW(tt,i),T));
        else                                     % ------AGE SHOCK: no
          ageWW(tt,i) = 1;
          if rand < q*theta                      % Does not find job
            zWW(tt,i)   = 1;
            eWW(tt,i)   = 2;
            tenWW(tt,i) = tenWW(tt-1,i);
            cWW(tt,i)   = btildey(min(1+tenWW(tt,i),T));
          else                                   % Finds a job
            zWW(tt,i)   = z0idx;
            eWW(tt,i)   = 1;
            tenWW(tt,i) = 1;
            cWW(tt,i)   = w(1,z0idx);
          end
        end

      else                                       % ---EMPLOYMENT: employed

        if rand < gamma                          % ------AGE SHOCK: yes
          ageWW(tt,i) = 2;
          eWW(tt,i)   = 1;
          zWW(tt,i)   = 1 + sum(rand > Pc(zWW(tt-1,i),:));
          tenWW(tt,i) = min(T,tenWW(tt-1,i)+1);
          cWW(tt,i)   = wo(tenWW(tt,i),zWW(tt,i));
          if (splito(tenWW(tt,i),zWW(tt,i)) == 1) || (rand < delta)
            zWW(tt,i) = 1;
            eWW(tt,i) = 2;
            cWW(tt,i) = btildeoA(min(1+tenWW(tt,i),T));
            JDc(tt,1) = JDc(tt,1) + 1;
          end
        else                                     % ------AGE SHOCK: no
          ageWW(tt,i) = 1;
          eWW(tt,i)   = 1;
          zWW(tt,i)   = 1 + sum(rand > Pc(zWW(tt-1,i),:));
          tenWW(tt,i) = min(T,tenWW(tt-1,i)+1);
          cWW(tt,i)   = w(tenWW(tt,i),zWW(tt,i));
          if (split(tenWW(tt,i),zWW(tt,i)) == 1) || (rand < delta)
            zWW(tt,i) = 1;
            eWW(tt,i) = 2;
            cWW(tt,i) = btildey(min(1+tenWW(tt,i),T));
            JDc(tt,1) = JDc(tt,1) + 1;
          end
        end

      end

    end

  end
end


%%   Compute individual wage volatility for long-tenured workers.

sample = find((tenWW(TT,:)>=80).*(tenWW(TT,:)<120).*(eWW(TT,:)==1) == 1);   % Select employed workers with long tenure

cWW1   = cWW(1:TT,sample);                                 % Income history
tenWW1 = tenWW(1:TT,sample);                               % Tenure history
vecstd = zeros(length(sample),1);                          % Create vectors of log wages for long-tenured workers

for i = 1 : length(sample)                                 % TT-tenWW1(TT,i)+1 is the 1st period of tenure for worker i
  vecstd(i) = std(log(cWW1(TT-tenWW1(TT,i)+1:TT,i)));      % Compute standard deviation of log wages during tenure
end

mean(vecstd)                                               % Average across all workers in the sample