% Create matrix UU containing the welfare gain for different combinations of initial eligibility and slope.

clearvars
tic
load Results\bench.mat

DO_GRID   = 0;                                             % Simulate reforms for a grid of tau_u and rho_u
DO_SINGLE = 1;                                             % Simulate reform  for a specific combination

if     DO_GRID == 1,      tau_u_g =  0 : 1 : 12;           % Entry phase (in months)
                          rho_u_g = 12 : 1 : 24;           % Slope (in dwys)
elseif DO_SINGLE == 1,    tau_u_g =  5;                    % Entry phase (in months)
                          rho_u_g = 20;                    % Slope (in dwys)
end

UU = zeros(length(tau_u_g),length(rho_u_g));               % Initialize matrix of results (UU contains welfare gain)
U0 = U(1);   uniEPL = 1;                                   % Save benchmark welfare and initialize EPL reform

for tau_u_idx = 1 : length(tau_u_g)                        % Iterate over grid of entry phase
  
  kappa = 0.01 * 8.9;                                      % Initial guess for payroll tax
  fprintf('       tau_u_idx = %2.0d / %2.0d          Time = %2.1f min\n',tau_u_idx,length(tau_u_g),toc/60)

  for rho_u_idx = 1 : length(rho_u_g)                      % Iterate over grid of slopes
    tau_u = tau_u_g(tau_u_idx);
    rho_u = rho_u_g(rho_u_idx);
    wavg0 = wavg;
    kappa = 100 * kappa;                                   % Use previous tax rate as initial guess (except for 1st it)
    init;   critgov = 1;
    while abs(critgov) > 1e-5 
      initx;   vfi;   simul;
      critgov = (Rev-Ben)/Rev;
      kappa   = kappa - 0.10 * critgov;
      wavg0   = 0.5*wavg + 0.5*wavg0;
    end    
    tmp  = 1/(1-(1/(1+r))*(1-gamma))*(1+(1/(1+r))*gamma/(1-(1/(1+r))*(1-chi)));
    UU(tau_u_idx,rho_u_idx) = 100*((U0 - tmp)/(U(1) - tmp) - 1);
    
  end

end
toc

if DO_GRID == 1
  UU
  [maxA,ind]      = max(UU(:));                            % Show best combination
  [maxrow,maxcol] = ind2sub(size(UU),ind);
  tau_u_g(maxrow)
  rho_u_g(maxcol)
  xelig = tau_u_g;
  xslop = rho_u_g;
  [ xmeshX , xmeshY ] = meshgrid(xelig,xslop);
  surf(xmeshX,xmeshY,UU')                                  % Visualize function values
else
  showstats;
end

% save optim.mat                                           % Save results