!---------------------------------------------------------------------------------------------------------------------------------------------------
!	module results_welfare: characterize welfare change associated with the reform
!---------------------------------------------------------------------------------------------------------------------------------------------------

module results_welfare

	use globals

	implicit none
	
contains


!	compute various quantities along the transition path

subroutine compute_welfare_figures()

	double precision :: WyTILDE(NT,NZ), WoTILDE(NT,NZ), UyTILDE(NT), UyPOST, skip1, skip2, norm
	double precision :: pos_w_y, pos_w_o, pos_w_temp, pos_w_perm, pos_u_y
	double precision :: ind_w_y, ind_w_o, ind_w_temp, ind_w_perm, ind_u_y
	double precision :: CE_u_y(NT), CE_w_y(NT,NZ), CE_w_o(NT,NZ)
	double precision :: wce_w_y, wce_w_o, wce_u_y, wce_a, wce_w, wce_u, wce_y, wce_o
	double precision :: wce_temp, wce_perm
	
	!	only one contract in t0
	
	ic = 1
	
	!	distribution
	
	LE = 0.0d+0
	LU = 0.0d+0
	open(1, file = 't0/dist_LEy.txt', status = 'old')
	open(2, file = 't0/dist_LEo.txt', status = 'old')
	open(3, file = 't0/dist_LU.txt', status = 'old')
	do it = 1, NT
		read(1, *) (LE(it,iz,ic,1), iz = NZ, 1, -1)
		read(2, *) (LE(it,iz,ic,2), iz = NZ, 1, -1)
		read(3, *) LU(it,ic,1), LU(it,ic,2), LU(it,ic,3)
	end do
	close(1)
	close(2)
	close(3)
	
	!	value functions
	
	open(1, file = 't0/value_Wy.txt', status = 'old')
	open(2, file = 't0/value_Wo.txt', status = 'old')
	open(3, file = 't0/value_U.txt', status = 'old')
	do it = 1, NT
		read(1, *) (WyTILDE(it,iz), iz = NZ, 1, -1)
		read(2, *) (WoTILDE(it,iz), iz = NZ, 1, -1)
		read(3, *) UyTILDE(it), skip1, skip2
	end do
	close(1)
	close(2)
	close(3)

	!	fraction of positive changes
	
	pos_w_y = 0.0d+0
	pos_w_o = 0.0d+0
	ind_w_y = 0.0d+0
	ind_w_o = 0.0d+0	
	do it = 1, NT
	do iz = 1, NZ
	
		if (Wy(it,iz,ic) .gt. WyTILDE(it,iz)) pos_w_y = pos_w_y + LE(it,iz,ic,1)
		if (Wo(it,iz,ic) .gt. WoTILDE(it,iz)) pos_w_o = pos_w_o + LE(it,iz,ic,2)
		if (Wy(it,iz,ic) .ge. WyTILDE(it,iz)) ind_w_y = ind_w_y + LE(it,iz,ic,1)
		if (Wo(it,iz,ic) .ge. WoTILDE(it,iz)) ind_w_o = ind_w_o + LE(it,iz,ic,2)
		
	end do
	end do
		
	pos_w_temp = 0.0d+0
	ind_w_temp = 0.0d+0
	do it = 1, 9
	do iz = 1, NZ
	
		if (Wy(it,iz,ic) .gt. WyTILDE(it,iz)) pos_w_temp = pos_w_temp + LE(it,iz,ic,1)
		if (Wo(it,iz,ic) .gt. WoTILDE(it,iz)) pos_w_temp = pos_w_temp + LE(it,iz,ic,2)
		if (Wy(it,iz,ic) .ge. WyTILDE(it,iz)) ind_w_temp = ind_w_temp + LE(it,iz,ic,1)
		if (Wo(it,iz,ic) .ge. WoTILDE(it,iz)) ind_w_temp = ind_w_temp + LE(it,iz,ic,2)
		
	end do
	end do
	
	pos_w_perm = pos_w_y + pos_w_o - pos_w_temp
	ind_w_perm = ind_w_y + ind_w_o - ind_w_temp
	
	pos_u_y = 0.0d+0
	ind_u_y = 0.0d+0
	
	do it = 1, NT
	
		if (Uy(it,ic) .gt. UyTILDE(it)) pos_u_y = pos_u_y + LU(it,ic,1)
		if (Uy(it,ic) .ge. UyTILDE(it)) ind_u_y = ind_u_y + LU(it,ic,1)
		
	end do
	
	norm = sum(LE(:,:,:,:))+sum(LU(:,:,1))
	
	print *, ' '
	print *, '% of the population benefiting STRICTLY from the policy change'
	print *, ' '
	write(*, '(A,X,F7.3)')  'overall', 1.0d+2*(pos_w_y + pos_w_o + pos_u_y)/norm
	print *, ' '
	write(*, '(A,X,F7.3)')  'young, overall',  1.0d+2*(pos_w_y + pos_u_y)/(sum(LE(:,:,:,1))+sum(LU(:,:,1)))
	write(*, '(A,X,F7.3)')  'young, empl.  ',  1.0d+2*pos_w_y/sum(LE(:,:,:,1))
	write(*, '(A,X,F7.3)')  'young, unemp. ',  1.0d+2*pos_u_y/sum(LU(:,:,1))
	write(*, '(A,X,F7.3)')	'older, empl.  ',  1.0d+2*pos_w_o/sum(LE(:,:,:,2))
	print *, ' '
	write(*, '(A,X,F7.3)') 'temporary ', 1.0d+2*pos_w_temp/(sum(LE(1:9,:,:,1))+sum(LE(1:9,:,:,2)))
	write(*, '(A,X,F7.3)') 'permanent ', 1.0d+2*pos_w_perm/(sum(LE(10:NT,:,:,1))+sum(LE(10:NT,:,:,2)))
	print *, ' '
	print *, ' '
	print *, '% of the population benefiting WEAKLY from the policy change'
	print *, ' '
	write(*, '(A,X,F7.3)')  'overall', 1.0d+2*(ind_w_y + ind_w_o + ind_u_y)/norm
	print *, ' '
	write(*, '(A,X,F7.3)')  'young, overall ',  1.0d+2*(ind_w_y + ind_u_y)/(sum(LE(:,:,:,1))+sum(LU(:,:,1)))
	write(*, '(A,X,F7.3)')  'young, empl.   ',  1.0d+2*ind_w_y/sum(LE(:,:,:,1))
	write(*, '(A,X,F7.3)')  'young, unemp.  ',  1.0d+2*ind_u_y/sum(LU(:,:,1))
	write(*, '(A,X,F7.3)')  'older, empl.   ',  1.0d+2*ind_w_o/sum(LE(:,:,:,2))
	print *, ' '
	write(*, '(A,X,F7.3)') 'temporary ', 1.0d+2*ind_w_temp/(sum(LE(1:9,:,:,1))+sum(LE(1:9,:,:,2)))
	write(*, '(A,X,F7.3)') 'permanent ', 1.0d+2*ind_w_perm/(sum(LE(10:NT,:,:,1))+sum(LE(10:NT,:,:,2)))
	print *, ' '
	
	!	welfare: consumption equivalent
	
	do it = 1, NT
	do iz = 1, NZ
	
		CE_w_y(it,iz) = welfare_ce(WyTILDE(it,iz),Wy(it,iz,ic),1)
		CE_w_o(it,iz) = welfare_ce(WoTILDE(it,iz),Wo(it,iz,ic),2)
		
	end do
	end do
	
	do it = 1, NT
	
		CE_u_y(it) = welfare_ce(UyTILDE(it), Uy(it,ic), 1)
		
	end do
	
	wce_w_y = 0.0d+0
	wce_w_o = 0.0d+0
	wce_temp = 0.0d+0
	wce_perm = 0.0d+0
	
	
	do it = 1, NT
	do iz = 1, NZ
		wce_w_y = wce_w_y + CE_w_y(it,iz)*LE(it,iz,ic,1)
		wce_w_o = wce_w_o + CE_w_o(it,iz)*LE(it,iz,ic,2)
	end do
	end do
	
	do it = 1, NT
	do iz = 1, NZ
		if (it .le. 9) wce_temp = wce_temp + CE_w_y(it,iz)*LE(it,iz,ic,1) + CE_w_o(it,iz)*LE(it,iz,ic,2)
		if (it .gt. 9) wce_perm = wce_perm + CE_w_y(it,iz)*LE(it,iz,ic,1) + CE_w_o(it,iz)*LE(it,iz,ic,2)
	end do
	end do
	
	do it = 1, NT
	
		wce_u_y = wce_u_y + CE_u_y(it)*LU(it,ic,1)
		
	end do

	!	compute averages
		
	wce_a = 1.0d+2*(wce_w_y + wce_w_o + wce_u_y)/norm
	wce_w = 1.0d+2*(wce_w_y + wce_w_o)/sum(LE)
	wce_y = 1.0d+2*(wce_w_y + wce_u_y)/(sum(LE(:,:,:,1)) + sum(LU(:,:,1)))
	wce_w_y = 1.0d+2*wce_w_y/sum(LE(:,:,:,1))
	wce_w_o = 1.0d+2*wce_w_o/sum(LE(:,:,:,2))
	wce_temp = 1.0d+2*wce_temp/(sum(LE(1:9,:,:,1))+sum(LE(1:9,:,:,2)))
	wce_perm = 1.0d+2*wce_perm/(sum(LE(10:NT,:,:,1))+sum(LE(10:NT,:,:,2)))
	wce_u_y = 1.0d+2*wce_u_y/sum(LU(:,:,1))
	wce_u   = wce_u_y
	wce_o   = wce_w_o
	
	print *, ' '
	print *, 'welfare change in consumption equivalent unit'
	print *, ' '
	write(*, '(A,3(X,F8.3))') 'overall, empl and unemp', wce_a, wce_w, wce_u
	print *, ' '
	write(*, '(A,3(X,F8.3))') 'young, overall ',  wce_y
	write(*, '(A,3(X,F8.3))') 'young, empl.   ',  wce_w_y,  1.0d+2*minval(CE_w_y, mask = LE(:,:,ic,1) .gt. 0.0d+0), &
															1.0d+2*maxval(CE_w_y, mask = LE(:,:,ic,1) .gt. 0.0d+0)
	write(*, '(A,3(X,F8.3))') 'young, unemp.  ',  wce_u_y,  1.0d+2*minval(CE_u_y, mask = LU(:,ic,1) .gt. 0.0d+0), &
															1.0d+2*maxval(CE_u_y, mask = LU(:,ic,1) .gt. 0.0d+0)
	write(*, '(A,3(X,F8.3))') 'older, empl.   ',  wce_w_o,  1.0d+2*minval(CE_w_o, mask = LE(:,:,ic,2) .gt. 0.0d+0), &
															1.0d+2*maxval(CE_w_o, mask = LE(:,:,ic,2) .gt. 0.0d+0)
	write(*, '(A,2(X,F8.3))') 'temp and perm ',  wce_temp, wce_perm
	print *, ' '

end subroutine


!	welfare functions

double precision function welfare_ce(V0,V1,id)

	double precision :: fact
	double precision, intent(in) :: V0, V1
	integer, intent(in) :: id
	
	!	factor for E(rho)
	
	if (sigma .eq. 1.0d+0) then
	
		if (id .eq. 1) fact = (1.0d+0-rho*(1.0d+0-gamma))*(1.0d+0-rho*(1.0d+0-khi))/(1.0d+0-rho*(1.0d+0-gamma-khi))
		if (id .eq. 2) fact = (1.0d+0-rho*(1.0d+0-khi))
		
	end if
	if (sigma .gt. 1.0d+0) then
	
		if (id .eq. 1) fact =(1.0d+0/(1.0d+0-sigma))*(1.0d+0/(1.0d+0-rho*(1.0d+0-gamma)))*(1.0d+0+rho*gamma/(1.0d+0-rho*(1.0d+0-khi)))
		if (id .eq. 2) fact =(1.0d+0/(1.0d+0-sigma))*(1.0d+0/(1.0d+0-rho*(1.0d+0-khi)))
		
	end if
	
	!	welfare formula
	
	if (sigma .eq. 1.0d+0) welfare_ce = exp(fact*(V1-V0)) - 1.0d+0
	if (sigma .gt. 1.0d+0) welfare_ce = ((V1+fact)/(V0+fact))**(1.0d+0/(1.0d+0-sigma)) - 1.0d+0
	
end function

end module	