% showstats.m

DO_PART1 = 1;                          % Compute job destruction rates
DO_PART2 = 1;                          % Display statistics from the model (e.g. for calibration)
DO_PART3 = 1;                          % Display parameters and statistics from the model (e.g. to create a txt-file)
DO_PART4 = 0;                          % Model fit for the tenure distribution (vs data)


%%   PART 1:   Compute job destruction rates.

if DO_PART1 == 1
  
  qo = sum(sum(ZZoe))+sum(sum(ZZou));   uo   = sum(sum(ZZou))/qo;      % Population share and unemployment rate (o)
  qy = 1-qo;                            uy   = sum(sum(ZZyu))/qy;      % Population share and unemployment rate (y)
  u  = qy*uy+qo*uo;                     outp = sum(ZZye+ZZoe)*z;       % Total unemployment rate // Total output

  zavgy = (sum(ZZye)*z)/sum(sum(ZZye));                                % Average productivity (young)
  zavgo = (sum(ZZoe)*z)/sum(sum(ZZoe));                                % Average productivity (old)
  zavg  = (sum(sum(ZZye))*zavgy + sum(sum(ZZoe))*zavgo)/(1-u);         % Average productivity (all)

  septemp = 0;                                                         % Compute separation rate for temporary jobs   
  for i = 2 : 9
    ZZye1(i,:) = (1-gamma)*(1-delta)*ZZye(i-1,:)*P;
    septemp    = septemp + sum(ZZye1(i,split(i,:)==1));
    ZZoe1(i,:) = (1-chi)*(1-delta)*ZZoe(i-1,:)*P + gamma*(1-delta)*ZZye(i-1,:)*P;
    septemp    = septemp + sum(ZZoe1(i,splito(i,:)==1)) + chi*sum(ZZoe(i-1,:));
  end
  septemp = delta + septemp/(sum(sum(ZZye(1:8,:)))+sum(sum(ZZoe(1:8,:))));

  sepperm = 0;                                                         % Compute separation rate for permanent jobs
  for i = 10 : T-1
    ZZye1(i,:) = (1-gamma)*(1-delta)*ZZye(i-1,:)*P;
    sepperm    = sepperm + sum(ZZye1(i,split(i,:)==1));
    ZZoe1(i,:) = (1-chi)*(1-delta)*ZZoe(i-1,:)*P + gamma*(1-delta)*ZZye(i-1,:)*P;
    sepperm    = sepperm + sum(ZZoe1(i,splito(i,:)==1)) + chi*(1-delta)*sum(ZZoe(i-1,:));
  end

  ZZoe1(T,:) = sum((1-chi)*(1-delta)*ZZoe(T-1:T,:)*P) + sum(gamma*(1-delta)*ZZye(T-1:T,:)*P);

  sepperm  = sepperm + sum(ZZoe1(T,splito(T,:)==1)) + chi*(1-delta)*sum(ZZoe(T,:));
  sepperm  = delta + sepperm/sum(sum(ZZye(9:T,:)+ZZoe(9:T,:)));
  septotal = septemp*sum(sum(ZZye(1:8,:))+sum(ZZoe(1:8,:))) + sepperm*sum(sum(ZZye(9:T,:)+ZZoe(9:T,:)));

  clear aux calibrationit C8 convtmp i io1 it iy1 iy2 ZZoe1 ZZou1 ZZye1 ZZyu1 xx sepy sepo annfaco RRe0 RRJ0 pjob1 ...
        philag mu1 inval0 CV_J0 CV_W0 CVF0 CVW0 tmp wF wF0 woF woW wW wW0 w01 resF0 resW0 annfacy
      
end


%%   PART 2:   Display statistics from the model (e.g. for calibration).

if DO_PART2 == 1
  
  if exist('z0disp','var') == 0
       findprob = q*theta;
  else findprob = alfa  * q * theta * (sum(sum(repmat(ZZyu(:,1),1,N).*(split0 == 0).*repmat(z0distrib,T,1))))/sum(ZZyu(:,1)) + ...
               (1-alfa) * q * theta;    % Average job finding probability
  end
  
  fprintf('\n       replrate y :   0.5800 - %6.4f             find prob :   0.4000 - %6.4f',b/wavgy,findprob)
  fprintf('\n       replrate o :   0.4540 - %6.4f             unemp old :   0.4000 - %6.4f',bo/wavgo,uo)
  fprintf('\n       destrtemp  :   7.4367 - %6.4f             destrperm :   2.0949 - %6.4f',septemp*100,sepperm*100)
  fprintf('\n       avg wage   :   %6.4f - %6.4f             kappa (%%) :   %6.4f\n',wavg0,wavg,100*kappa)
  
  if exist('alfa','var') == 1
    fprintf('       theta      :   1.0000 - %6.4f\n',theta)
  end
  
  if exist('z0disp','var') == 1
    jobfindprob = alfa*q*theta*pjob + (1-alfa)*q*theta;
    survivalmatrix = zeros(T,40);
    for itau = 1 : T
      for iperiod = 1 : 40
        survivalmatrix(itau,iperiod) = 100 * (1-jobfindprob(itau))^iperiod;
      end
    end
    pi_u_2_to_4 = survivalmatrix(:,1) - survivalmatrix(:,3);         % Unemployed 2 <= x < 4 quarters (%, by tenure)
    pi_u_4_to_8 = survivalmatrix(:,3) - survivalmatrix(:,7);         % Unemployed 4 <= x < 8 quarters (%, by tenure)
    pi_u_8_plus = survivalmatrix(:,7);                               % Unemployed >= 8 quarters       (%, by tenure)    
    pi_u_2_to_4_aggr = mu*pi_u_2_to_4;                     % Unemployed 2 <= x < 4 quarters (%, overall)
    pi_u_4_to_8_aggr = mu*pi_u_4_to_8;                     % Unemployed 4 <= x < 8 quarters (%, overall)
    pi_u_8_plus_aggr = mu*pi_u_8_plus;                     % Unemployed >= 8 quarters       (%, overall)
  end

end


%%   PART 3:   Display statistics from the model (e.g. for calibration).

if DO_PART3 == 1
  
  if uniEPL == 1                                           % Reference file for comparison (unified vs dual EPL)
    
    if exist('z0disp','var') == 0
      oo = load('Results\bench.mat');                      % <----------------------------------------------------------
    elseif alfa == 0.25,   oo = load('Results\stochz25.mat');
    elseif alfa == 0.50,   oo = load('Results\stochz50.mat');
    elseif alfa == 0.75,   oo = load('Results\stochz75.mat');
    end      
    
    duy       = 100 * (uy - oo.uy)/oo.uy;
    duo       = 100 * (uo - oo.uo)/oo.uo;
    du        = 100 * (u  - oo.u)/oo.u;
    dwavgy    = 100 * (wavgy - oo.wavgy)/oo.wavgy;
    dwavgo    = 100 * (wavgo - oo.wavgo)/oo.wavgo;
    dwavg     = 100 * (wavg  - oo.wavg)/oo.wavg;
    dzavgy    = 100 * (zavgy - oo.zavgy)/oo.zavgy;
    dzavgo    = 100 * (zavgo - oo.zavgo)/oo.zavgo;
    dseptemp  = 100 * (septemp - oo.septemp)/oo.septemp;
    dsepperm  = 100 * (sepperm - oo.sepperm)/oo.sepperm;
    dfindprob = 100 * (findprob - oo.findprob)/oo.findprob;
    dtheta    = 100 * (theta - oo.theta)/oo.theta;
    dkappa    = 100 * (kappa - oo.kappa)/oo.kappa;
    if exist('z0disp','var')
      dpi1    = 100 * (pi_u_2_to_4_aggr - oo.pi_u_2_to_4_aggr)/oo.pi_u_2_to_4_aggr;
      dpi2    = 100 * (pi_u_4_to_8_aggr - oo.pi_u_4_to_8_aggr)/oo.pi_u_4_to_8_aggr;
      dpi3    = 100 * (pi_u_8_plus_aggr - oo.pi_u_8_plus_aggr)/oo.pi_u_8_plus_aggr;
    end
  end

  fprintf('\n\n   PARAMETERS:\n\n')
  fprintf('   by    = %6.4f        -        Unemployment benefit (young worker)\n',b)
  fprintf('   bo    = %6.4f        -        Unemployment benefit (older worker)\n',bo)
  fprintf('   k     = %6.4f        -        Vacancy posting cost\n',k)
  fprintf('   A     = %6.4f        -        Matching efficiency\n',A)
  fprintf('   upsil = %6.4f        -        One minus red-tape costs\n',upsil)
  fprintf('   delta = %6.4f        -        Exogenous job destruction rate\n',delta)
  if exist('z0idx','var'),   fprintf('   z0    = %6.4f        -        Initial match productivity\n',z(z0idx));
  else                       fprintf('   z0    = %6.4f        -        Initial match productivity\n',z(z0centeridx));
  end
  fprintf('   sigz  = %6.4f        -        Standard deviation for z\n',stdz)
  if exist('pjob','var'),    fprintf('   alpha = %6.4f        -        Probability of drawing stochastic z0\n',alfa);
  end
  fprintf('\n\n')

  fprintf('   EPL SCHEME:\n\n')
  if uniEPL == 0
    fprintf('   Benchmark scheme\n\n\n')
  else
    fprintf('   Entry phase (months) :            %2.0f\n',tau_u)
    fprintf('   Slope (in dwys)      :            %2.0f\n\n\n',rho_u)
  end

  fprintf('   STATISTICS:\n\n')
  fprintf('   Unemployment young (%%) :     %7.4f',100*uy);        if uniEPL == 1, fprintf('    ( %5.1f %% )',duy); end
  fprintf('\n   Non-employment old (%%) :     %7.4f',100*uo);      if uniEPL == 1, fprintf('    ( %5.1f %% )',duo); end
  fprintf('\n   Non-employment all (%%) :     %7.4f',100*u);       if uniEPL == 1, fprintf('    ( %5.1f %% )',du); end
  fprintf('\n   Average wage young     :     %7.4f',wavgy);        if uniEPL == 1, fprintf('    ( %5.1f %% )',dwavgy); end
  fprintf('\n   Average wage old       :     %7.4f',wavgo);        if uniEPL == 1, fprintf('    ( %5.1f %% )',dwavgo); end
  fprintf('\n   Average wage all       :     %7.4f',wavg);         if uniEPL == 1, fprintf('    ( %5.1f %% )',dwavg); end
  fprintf('\n   Average prod young     :     %7.4f',zavgy);        if uniEPL == 1, fprintf('    ( %5.1f %% )',dzavgy); end
  fprintf('\n   Average prod old       :     %7.4f',zavgo);        if uniEPL == 1, fprintf('    ( %5.1f %% )',dzavgo); end
  fprintf('\n   JD rate <= 2 years (%%) :     %7.4f',100*septemp);    if uniEPL == 1, fprintf('    ( %5.1f %% )',dseptemp); end
  fprintf('\n   JD rate  > 2 years (%%) :     %7.4f',100*sepperm);    if uniEPL == 1, fprintf('    ( %5.1f %% )',dsepperm); end
  fprintf('\n   Job finding rate (%%)   :     %7.4f',100*findprob);   if uniEPL == 1, fprintf('    ( %5.1f %% )',dfindprob); end
  if exist('z0disp','var')
    fprintf('\n   Unempl spell (quarters, %%)');
    fprintf('\n      2 <= x < 4          :     %7.4f', pi_u_2_to_4_aggr);   if uniEPL == 1, fprintf('    ( %5.1f %% )',dpi1); end
    fprintf('\n      4 <= x < 8          :     %7.4f', pi_u_4_to_8_aggr);   if uniEPL == 1, fprintf('    ( %5.1f %% )',dpi2); end
    fprintf('\n      8 <= x              :     %7.4f', pi_u_8_plus_aggr);   if uniEPL == 1, fprintf('    ( %5.1f %% )',dpi3); end
  end
  fprintf('\n   Labor market tightness :     %7.4f',theta);          if uniEPL == 1, fprintf('    ( %5.1f %% )',dtheta); end
  fprintf('\n   Payroll tax (%%)        :     %7.4f',100*kappa);     if uniEPL == 1, fprintf('    ( %5.1f %% )',dkappa); end
  fprintf('\n   Lifetime util newborn  :   %7.4f',U(1))
  if uniEPL == 1
    fprintf('\n   Welfare gain (%%)       :     %7.4f',UU(1))
    fprintf('\n\n   [ Make sure to load the correct reference file for computing %% changes. ]\n')
  end

  fprintf(' \n')
  
end


%%   PART 4:   Model fit for the tenure distribution (vs data).

if DO_PART4 == 1

  % ---------------------------------------------------  DATA  ---------------------------------------------------------

  % I use the following criteria to select the sample from the OECD Stat webpage: Years 2000-2007, only males, ages 
  % 15-64, dependent employment, annual frequency, measured in thousand employees. Moreover, I exclude individuals who
  % have been employed for less than a month (otherwise the fit would be very bad). We can justify this by assuming that
  % these are not fully established matches yet.

  % 2000                  2001                    2002
  tot.y00  = 7618-466;    tot.y01  = 7906-453;    tot.y02  = 8183-427;   % Total number of individuals
  t2.y00   = 629;         t2.y01   = 631;         t2.y02   = 633;        % Tenure: Less or equal than 2 quarters
  t4.y00   = 627;         t4.y01   = 670;         t4.y02   = 670;        % Tenure: Less or equal than 4 quarters
  t12.y00  = 1193;        t12.y01  = 1175;        t12.y02  = 1245;       % Tenure: Less or equal than 3 years
  t20.y00  = 788;         t20.y01  = 967;         t20.y02  = 1073;       % Tenure: Less or equal than 5 years
  t40.y00  = 1064;        t40.y01  = 1125;        t40.y02  = 1167;       % Tenure: Less or equal than 10 years
  t41p.y00 = 2851;        t41p.y01 = 2886;        t41p.y02 = 2969;       % Tenure: More than 10 years

  % 2003                  2004                    2005                    2006                    2007
  tot.y03  = 8489-449;    tot.y04  = 8716-475;    tot.y05  = 9083-515;    tot.y06  = 9339-550;    tot.y07  = 9554-540;
  t2.y03   = 669;         t2.y04   = 699;         t2.y05   = 776;         t2.y06   = 821;         t2.y07   = 842;
  t4.y03   = 663;         t4.y04   = 705;         t4.y05   = 730;         t4.y06   = 806;         t4.y07   = 842;
  t12.y03  = 1280;        t12.y04  = 1301;        t12.y05  = 1278;        t12.y06  = 1374;        t12.y07  = 1487;
  t20.y03  = 1145;        t20.y04  = 1210;        t20.y05  = 1182;        t20.y06  = 1102;        t20.y07  = 1101;
  t40.y03  = 1295;        t40.y04  = 1403;        t40.y05  = 1535;        t40.y06  = 1564;        t40.y07  = 1622;
  t41p.y03 = 2986;        t41p.y04 = 2925;        t41p.y05 = 3063;        t41p.y06 = 3122;        t41p.y07 = 3120;

  % Take the average across all years in the sample.

  t2   = (1/8) * ( (t2.y00/tot.y00) + (t2.y01/tot.y01) + (t2.y02/tot.y02) + (t2.y03/tot.y03) + (t2.y04/tot.y04) + ...
                   (t2.y05/tot.y05) + (t2.y06/tot.y06) + (t2.y07/tot.y07) );
  t4   = (1/8) * ( (t4.y00/tot.y00) + (t4.y01/tot.y01) + (t4.y02/tot.y02) + (t4.y03/tot.y03) + (t4.y04/tot.y04) + ...
                   (t4.y05/tot.y05) + (t4.y06/tot.y06) + (t4.y07/tot.y07) );
  t12  = (1/8) * ( (t12.y00/tot.y00) + (t12.y01/tot.y01) + (t12.y02/tot.y02) + (t12.y03/tot.y03) + (t12.y04/tot.y04) + ...
                   (t12.y05/tot.y05) + (t12.y06/tot.y06) + (t12.y07/tot.y07) );
  t20  = (1/8) * ( (t20.y00/tot.y00) + (t20.y01/tot.y01) + (t20.y02/tot.y02) + (t20.y03/tot.y03) + (t20.y04/tot.y04) + ...
                   (t20.y05/tot.y05) + (t20.y06/tot.y06) + (t20.y07/tot.y07) );
  t40  = (1/8) * ( (t40.y00/tot.y00) + (t40.y01/tot.y01) + (t40.y02/tot.y02) + (t40.y03/tot.y03) + (t40.y04/tot.y04) + ...
                   (t40.y05/tot.y05) + (t40.y06/tot.y06) + (t40.y07/tot.y07) );
  t41p = (1/8) * ( (t41p.y00/tot.y00) + (t41p.y01/tot.y01) + (t41p.y02/tot.y02) + (t41p.y03/tot.y03) + (t41p.y04/tot.y04) + ...
                   (t41p.y05/tot.y05) + (t41p.y06/tot.y06) + (t41p.y07/tot.y07) );

  % Collect tenure distribution in vector.

  taudata = [ t2 ; t4 ; t12 ; t20 ; t40 ; t41p ];


  % ---------------------------------------------------  MODEL  --------------------------------------------------------

  tote  = sum(sum(ZZye+ZZoe));
  t2m   = sum(sum(ZZye(1:2,:)   +ZZoe(1:2,:)));
  t4m   = sum(sum(ZZye(3:4,:)   +ZZoe(3:4,:)));
  t12m  = sum(sum(ZZye(5:12,:)  +ZZoe(5:12,:)));
  t20m  = sum(sum(ZZye(13:20,:) +ZZoe(13:20,:)));
  t40m  = sum(sum(ZZye(21:40,:) +ZZoe(21:40,:)));
  t41pm = sum(sum(ZZye(41:end,:)+ZZoe(41:end,:)));

  taumodel = [ t2m ; t4m ; t12m ; t20m ; t40m ; t41pm ] ./ tote;

  100 * [ taudata , taumodel]                                % Compare model with data
  
end

clear DO_PART1 DO_PART2 DO_PART3 DO_PART4