% vfi.m
%
% Solve model by value function iteration. First solve the problem of an older worker. This amounts to solving for the
% value functions of being employed for a worker, the value of a job for a firm, and the equilibrium wage. Then solve
% the problem of a young worker. This amounts to solving for the value functions of being employed and unemployed for a
% worker, the value of a job for a firm, the equilibrium wage, and the equilibrium labor market tightness.

crito = 1;                                                 % Stopping criterion
crity = 1;                                                 % Stopping criterion


%%   (i) Value function iteration for older workers.

C1 = repmat(UoA(aux,:),1,N);      C2 = -repmat(philag,1,N);          % Some matrices (can be precomputed)
C3 = repmat(z',T,1);              C4 =  repmat(phi,1,N);             % Some matrices (can be precomputed)
C8 = zeros(T,N);

while crito > 1e-4
  C5 = -(1./woW - 1);                                                % Some matrix (can be precomputed)
  C6 = C3 - woF*(1+kappa);                                           % Some matrix (can be precomputed)
  for i2 = 1 : io1
    if QUITS == 1                                                    % Benchmark model: Delta-shock entitles to SP
      Wo = C5 + (1/(1+r)) * (1-chi) * ( (1-delta)*max( Wo(aux,:) , C1 )*P' + delta*C1 );
      Jo = C6 + (1/(1+r)) * (1-chi) * ( (1-delta)*max( Jo(aux,:) , C2 )*P' + delta*C2 );
    else                                                             % delta-shock does NOT entitle to SP
      Wo = C5 + (1/(1+r)) * (1-chi) * ( (1-delta)*max( Wo(aux,:) , C1 )*P' + delta*UoA(1) );
      Jo = C6 + (1/(1+r)) * (1-chi) *   (1-delta)*max( Jo(aux,:) , C2 )*P';
    end
  end
  splito = C8;
  CVW  = Wo - C5 - repmat(UoA,1,N);
  CVF  = Jo - C6;
  resW = 1./(1 + CVW);
  resF = (C3 + C4 + CVF)/(1+kappa);  
  wo1 = sqrt(resF.*resW*beta/(1-beta));
  wo1(resW > resF) = wo(resW > resF);
  splito(resW > resF) = 1;
  crito = 100*max(max(abs(wo1-wo)./wo));         wo = xx*wo1 + (1-xx)*wo;
  woF = wo;                                      woF(splito==1) = 4.9999;
  woW = wo;                                      woW(splito==1) = 0.001;
end


%%   (ii) Value function iteration for young workers.

if QUITS == 1
  CV_W = (1/(1+r))*gamma*( (1-delta)*max( Wo(aux,:) , repmat(UoA(aux,1),1,N) )*P' + delta*repmat(UoA(aux,1),1,N) );
  CV_J = (1/(1+r))*gamma*( (1-delta)*max( Jo(aux,:) , C2                     )*P' + delta*C2 );
else
  CV_W = (1/(1+r))*gamma*( (1-delta)*max( Wo(aux,:) , repmat(UoA(aux,1),1,N) )*P' + delta*UoA(1) );
  CV_J = (1/(1+r))*gamma*  (1-delta)*max( Jo(aux,:) , C2                     )*P';
end

maxity = 1500;                                             % Maximum number of iterations (show warning if hit)
ity    = 0;                                                % Initialize iteration counter
C7     = (1/(1+r))*(1-gamma);                              % Some number (can be precomputed)

while (crity > 1e-5) && (ity < maxity)                     % Outer loop over equilibrium wage function
  
  for i2 = 1 : iy1                                         % I iterate over theta a prespecified number of times
    
    q = A * theta^(-psi);
    jobfindprob = q*theta;
    expdur   = (1-gamma+gamma/chi)/(gamma*(1-jobfindprob) + jobfindprob);
    annfacy  = (r/(1+r)) * (1/(1-(1+r)^(-expdur)));        % Annuity factor: Young workers
    btildey  = b  + upsil*annfacy*phi;                     % Non-employment income for young workers
    btildeoB = bo + upsil*annfacy*phi;                     % Non-employment income for older workers
    UoB = -(1./btildeoB-1)/(1-(1/(1+r))*(1-chi));          % Non-employed older workers: laid off when young
    RRu = -(1./btildey-1);                                 % Some matrix (can be precomputed)
    RRe = -(1./wW-1);                                      % Single-period return matrix (can be precomputed)
    RRJ = C3 - wF*(1+kappa);                               % Single-period return matrix (can be precomputed)
    
    for i3 = 1 : iy2                                       % Iterate over value functions (given wage)      
      if (QUITS == 1)                                      % Benchmark: delta-shock entitles to SP
        U = RRu + (1/(1+r)) * ( (1-gamma)*(q*theta*W(1,z0idx) + (1-q*theta)*U) + gamma*UoB );
        W = RRe + C7*( (1-delta)*max(W(aux,:),repmat(U(aux,1),1,N))*P' + delta*repmat(U(aux,1),1,N) ) + CV_W;
        J = RRJ + C7*( (1-delta)*max(J(aux,:),C2                  )*P' + delta*C2) + CV_J;
      else                                                 % delta-shock does not entitle to SP
        U = RRu + (1/(1+r)) * ( (1-gamma)*(q*theta*W(1,z0idx) + (1-q*theta)*U) + gamma*UoB );
        W = RRe + C7*( (1-delta)*max(W(aux,:),repmat(U(aux,1),1,N))*P' + delta*U(1) ) + CV_W;
        J = RRJ + C7*  (1-delta)*max(J(aux,:),C2                  )*P' + CV_J;
      end
    end
    
    theta1 = ( (1/k)*(1/(1+r)) * A * max(J(1,z0idx),0) )^(1/psi);
    theta  = (1-xx)*theta + xx*theta1;
    
  end
  
  split = C8;                                              % Solve wage bargaining problem
  CVW   = W - RRe - repmat(U,1,N);
  CVF   = J - RRJ;
  resW = 1./(1 + CVW);
  resF = (C3 + C4 + CVF)/(1+kappa);  
  w1   = sqrt(resF.*resW*beta/(1-beta));
  w1(resW > resF) = w(resW > resF);              split(resW > resF) = 1;
  crity = 100*max(max(abs(w1-w)./w));            w = xx*w1 + (1-xx)*w;
  wF = w;                                        wF(split==1) = 4.9999;
  wW = w;                                        wW(split==1) = 0.001;
  ity = ity + 1;
  
end

if (q > 1),           disp('Careful: Job-filling probability is larger than 1!');   end
if (q*theta > 1),     disp('Careful: Job-finding probability is larger than 1!');   end  
if (ity == maxity),   disp('Hit maxity --> Discard!');                              end

clear C1 C2 C3 C4 C5 C6 C7 CV_J CV_W CVF CVW i1 i2 i3 J1 Jo1 resF resW RRe RRJ RRu TMPq theta1 w1 W1 wo1 Wo1 crito ...
      crity expdur ity j V0 U1 maxity philag wW wF woW woF