!---------------------------------------------------------------------------------------------------------------------------------------------------
!	module wage_function_young: obtain wage functions for young workers
!---------------------------------------------------------------------------------------------------------------------------------------------------

module wage_function_young

	use globals
	use misc_functions
	
	implicit none
	
	double precision :: contW, contJ, altU

contains
	

!	subroutine to solve for wages

subroutine solve_wage_young_workers()

	!	solve forwards
	
	do it = 1, NT
	do iz = 1, NZ
	do ic = 1, NC
				
		!	continuation values here
			
		call continuation_young_workers()
			
		!	reservation wages
	
		altU = Uy(it,ic)+(1.0d+0-young_sp)*(utils(By+nu*phi(it,ic))-utils(By))
		wmin = inv_utils(altU - contW)
		wmax = (phi(it,ic) + zGrid(iz) + contJ)/(1.0d+0+kappa(ip))
			
		if ( wmin .le. wmax ) then
			
			wwy(ip,it,iz,ic) = sqrt((zGrid(iz) + contJ + phi(it,ic))/((1.0d+0 + contW - altU)*(1.0d+0+kappa(ip))))
			!	wwy(ip,it,iz,ic) = solve_nash_foc()
		
		else
		
			wwy(ip,it,iz,ic) = (wmin+wmax)/2.0d+0
			
		end if
							
		!	fill the corresponding value functions
		
			Wy(it,iz,ic) = utils(wwy(ip,it,iz,ic)) + contW
			Jy(it,iz,ic) = zGrid(iz) - (1.0d+0+kappa(ip))*wwy(ip,it,iz,ic) + contJ
		
	end do
	end do
	end do
		
end subroutine


!	continuation values

subroutine continuation_young_workers()

	!	worker's side
	
	altU  = UyP(min(NT,it+1),ic)+(1.0d+0-young_sp)*(utils(By+nu*phi(min(NT,it+1),ic))-utils(By))
	contW = rho*(1.0d+0-gamma)*(delta*(UyP(1,ic) + delta_sp*(altU-UyP(1,ic))) &
				+(1.0d+0-delta)*dot_product(P(iz,:),max(Wy(min(NT,it+1),:,ic),altU))) &
			+ rho*gamma*(delta*(Uo(1,ic) + delta_sp*(Uo(min(NT,it+1),ic)-Uo(1,ic))) &
				+(1.0d+0-delta)*dot_product(P(iz,:),max(Wo(min(NT,it+1),:,ic),Uo(min(NT,it+1),ic))))

	!	firm's side
	
	contJ = - rho*delta*phi(min(NT,it+1),ic)*delta_sp &
			+ rho*(1.0d+0-gamma)*(1.0d+0-delta)*dot_product(P(iz,:),max(Jy(min(NT,it+1),:,ic),-phi(min(NT,it+1),ic))) &
			+ rho*gamma*(1.0d+0-delta)*dot_product(P(iz,:),max(Jo(min(NT,it+1),:,ic),-phi(min(NT,it+1),ic)))
	
	
end subroutine


!	value of being unemployed during the transition

subroutine value_unemployed_young()

	double precision :: Uy_t, jf_t
	integer :: ib
	
	!	current annfacY
	
	annfacY = (1.0d+0-rho)/(1.0d+0-rho**Edur)
	
	!	value of "lucky" unemployed
	
	do ic = 1, NC
	do it = 1, NT
	
		Ul(it,ic) = (utils(Bo + nu*annfacY*phi(it,ic)*young_sp) + leis + rho*khi*R)/(1.0d+0-rho*(1.0d+0-khi))
												
	end do
	end do

	!	value of young unemployed
	
	
	do ic = 1, NC
	do it = 1, NT

		!	value of being unemployed at t1
		
		jf_t  = min(A*theta(NP)**(1.0d+0-psi),theta(NP),1.0d+0)
		
		Uy_t = utils(By+nu*annfacY*phi(it,ic)*young_sp) + rho*(gamma*Ul(it,ic)+(1.0d+0-gamma)*jf_t*Wn(NP))
		Uy_t = Uy_t/(1.0d+0-rho+rho*(gamma+(1.0d+0-gamma)*jf_t))
		
		!	compute backwards to obtain current Uy
		
		do ib = NP-1, ip, -1
		
			jf_t = min(A*theta(ib)**(1.0d+0-psi),theta(ib),1.0d+0)
			Uy_t = utils(By+nu*annfacY*phi(it,ic)*young_sp) + rho*((1.0d+0-gamma)*(jf_t*Wn(ib+1)+(1.0d+0-jf_t)*Uy_t)+ gamma*Ul(it,ic))
			
		end do
		
		Uy(it,ic) = Uy_t
		
	end do
	end do
		
end subroutine


!	solve Nash problem using FOC (bisection method)

double precision function solve_nash_foc()

	double precision :: x, xa, xb, ya, yb, y
	logical :: ifstop
	
	!	minimum starting point
	xa = wmin
	ya = nash_foc(xa)
	
	!	maximum starting point
	xb = wmax
	yb = nash_foc(xb)
			
	ifstop = .true.
	do while (ifstop)
		
		x = (xa+xb)/2.0d+0
		y = nash_foc(x)
		if ( y*ya .lt. 0.0d+0 ) then
			xb = x
			yb = y
		else
			xa = x
			ya = y
		end if
		
		if (abs(xb-xa) .lt. TOL_nash) ifstop = .false.
		
	end do
	solve_nash_foc = (xa+xb)/2.0d+0
	
end function


!	write down Nash FOC

double precision function nash_foc(ww)
	
	double precision, intent(in) :: ww
	double precision :: valW, valJ
	
	!	worker's side
	
	valW = utils(ww) + contW
	altU = Uy(it,ic) + (1.0d+0-young_sp)*(utils(By+nu*phi(it,ic))-utils(By))

	!	firm's side
	
	valJ = zGrid(iz) - (1.0d+0+kappa(ip))*ww + contJ
		
	!	difference
	
	nash_foc = beta*(valJ + phi(it,ic))/(1.0d+0+kappa(ip)) - (1.0d+0-beta)*(valW-altU)/umarg(ww)
	
end function


end module