!---------------------------------------------------------------------------------------------------------------------------------------------------
!	module distribution: compute the time-invariant distribution
!---------------------------------------------------------------------------------------------------------------------------------------------------

module distribution

	use globals
	use misc_functions
	
	implicit none
	
contains


!	compute next period distribution

subroutine update_distribution()

	double precision :: NewLE(NT,NZ,NW,NC,2), NewLU(NT,NC,3), norm
	
	!	initialize size distribution
	norm = sum(LE) + sum(LU)
	LE  = LE/norm
	LU  = LU/norm
	
	!	transpose transition matrix
	
	P = transpose(P)
	
	!	set new measures to zero
	
	NewLE = 0.0d+0
	NewLU = 0.0d+0
		
	!	job-finding rate this period
	
	jfind = min(A*theta(ip)**(1.0d+0-psi),theta(ip),1.0d+0)
	
	!	employment
	
	iw_p = locate(max(wGrid(1), min(wGrid(NW),wwy(max(1,ip-1),1,iz0,NC))), wGrid) 	
	NewLE(1,iz0,iw_p,NC,1) = (1.0d+0-gamma)*jfind*sum(LU(:,:,1))
	
	do ic = 1, NC
		
		do iw = 1, NW
		
			do iz = 1, NZ
			do it = 2, NT
			
					ww_r = wage_rule(wGrid(iw), wwy(max(1,ip-1),it-1,iz,ic))
					iw_p = locate(max(wGrid(1), min(wGrid(NW),ww_r)), wGrid)		
			
					NewLE(it,:,iw_p,ic,1) = NewLE(it,:,iw_p,ic,1) + (1.0d+0-delta)*(1.0d+0-gamma)*P(:,iz)*LE(it-1,iz,iw,ic,1)
					NewLE(it,:,iw_p,ic,2) = NewLE(it,:,iw_p,ic,2) + (1.0d+0-delta)*gamma*P(:,iz)*LE(it-1,iz,iw,ic,1)
					
					ww_r = wage_rule(wGrid(iw), wwo(max(1,ip-1),it-1,iz,ic))
					iw_p = locate(max(wGrid(1), min(wGrid(NW),ww_r)), wGrid)
					
					NewLE(it,:,iw_p,ic,2) = NewLE(it,:,iw_p,ic,2) + (1.0d+0-delta)*(1.0d+0-khi)*P(:,iz)*LE(it-1,iz,iw,ic,2)
					
				
			end do
			end do
			
			do iz = 1, NZ
			
					ww_r = wage_rule(wGrid(iw), wwy(max(1,ip-1),NT,iz,ic))
					iw_p = locate(max(wGrid(1), min(wGrid(NW),ww_r)), wGrid)		
			
					NewLE(NT,:,iw_p,ic,1) = NewLE(NT,:,iw_p,ic,1) + (1.0d+0-delta)*(1.0d+0-gamma)*P(:,iz)*LE(NT,iz,iw,ic,1)
					NewLE(NT,:,iw_p,ic,2) = NewLE(NT,:,iw_p,ic,2) + (1.0d+0-delta)*gamma*P(:,iz)*LE(NT,iz,iw,ic,1)
					
					ww_r = wage_rule(wGrid(iw), wwo(max(1,ip-1),NT,iz,ic))
					iw_p = locate(max(wGrid(1), min(wGrid(NW),ww_r)), wGrid)
					
					NewLE(NT,:,iw_p,ic,2) = NewLE(NT,:,iw_p,ic,2) + (1.0d+0-delta)*(1.0d+0-khi)*P(:,iz)*LE(NT,iz,iw,ic,2)
					
				
			end do
			
		end do
				
		
		!	separations
		
		do it = 1, NT
		do iw = 1, NW
		
			if (sepy(ip,it,iw,ic) .le. NZ) then
			
				NewLU(it,ic,1) = NewLU(it,ic,1) + sum(NewLE(it,sepy(ip,it,iw,ic):NZ,iw,ic,1))
				NewLE(it,sepy(ip,it,iw,ic):NZ,iw,ic,1) = 0.0d+0
				
			end if
			
			if (sepo(ip,it,iw,ic) .le. NZ) then
			
				NewLU(it,ic,3) = NewLU(it,ic,3) + sum(NewLE(it,sepo(ip,it,iw,ic):NZ,iw,ic,2))
				NewLE(it,sepo(ip,it,iw,ic):NZ,iw,ic,2) = 0.0d+0
			
			end if
			
		end do
		end do
		
	end do
		
	!	unemployment
	
	NewLU(1,NC,1) = NewLU(1,NC,1) + khi*(sum(LU(:,:,2:3))+sum(LE(:,:,:,:,2))) &
								  + (1.0d+0-gamma)*delta*sum(LE(:,:,:,:,1))*(1.0d+0-delta_sp)
	NewLU(:,NC,1) = NewLU(:,NC,1) + (1.0d+0-gamma)*((1.0d+0-jfind)*sum(LU(:,:,1), dim = 2) &
								  + delta*sum(sum(sum(LE(:,:,:,:,1), dim = 4), dim = 3), dim = 2)*delta_sp)
	NewLU(:,:,2) = NewLU(:,:,2) + gamma*LU(:,:,1) + (1.0d+0-khi)*LU(:,:,2)
	
	do ic = 1, NC
	
		NewLU(1,ic,3) = NewLU(1,ic,3) + (gamma*delta*sum(LE(:,:,:,ic,1)) + (1.0d+0-khi)*delta*sum(LE(:,:,:,ic,2)))*(1.0d+0-delta_sp)
		NewLU(:,ic,3) = NewLU(:,ic,3) + (1.0d+0-khi)*LU(:,ic,3) &
								+ (gamma*delta*sum(sum(LE(:,:,:,ic,1), dim = 3), dim = 2) &
								+ (1.0d+0-khi)*delta*sum(sum(LE(:,:,:,ic,2), dim = 3), dim = 2))*delta_sp
								
	end do
	
	!	update distribution
	
	LE = NewLE
	LU = NewLU
	
	!	transpose transition matrix back
	
	P = transpose(P)
	
end subroutine





end module