% Create matrix UU containing the welfare gain for different combinations of initial eligibility and slope.

clearvars
tic
load rigid90.mat

DO_GRID   = 0;                                             % Simulate reforms for a grid of tau_u and rho_u
DO_SINGLE = 1;                                             % Simulate reform  for a specific combination

if     DO_GRID == 1,      tau_u_g =  1 : 1 : 5;            % Entry phase (in months)
                          rho_u_g =  30 : 1 : 36;          % Slope (in dwys)
elseif DO_SINGLE == 1,    tau_u_g =  3;                    % Entry phase (in months)
                          rho_u_g = 33;                    % Slope (in dwys)
end

% alfa = 0.25: tau = 3, rho = 21
% alfa = 0.50: tau = 2, rho = 21
% alfa = 0.75: tau = 1, rho = 23
% alfa = 0.90: tau = 1, rho = 23
% downwardrig: tau = 3, rho = 33

UU = zeros(length(tau_u_g),length(rho_u_g));               % Initialize matrix of results (UU contains welfare gain)
U0 = U(1);   uniEPL = 1;                                   % Save benchmark welfare and initialize EPL reform

for tau_u_idx = 1 : length(tau_u_g)                        % Iterate over grid of entry phase
  
  if DOWNWARDRIGID == 0
    kappa = 0.01 * 8.9;                                    % Initial guess for payroll tax
  else
    kappa = 0.01 * 11.22;                                  % Start with a higher value here (to avoid divergence)
  end

  for rho_u_idx = 1 : length(rho_u_g)                      % Iterate over grid of slopes
    fprintf('       tau_u_idx = %2.0d / %2.0d     rho_u_idx = %2.0d / %2.0d      Time = %2.1f min\n',tau_u_idx,...
       length(tau_u_g),rho_u_idx,length(rho_u_g),toc/60)
    tau_u = tau_u_g(tau_u_idx);
    rho_u = rho_u_g(rho_u_idx);
    wavg0 = wavg;
    kappa = 100 * kappa;                                   % Use previous tax rate as initial guess (except for 1st it)
    init;
    if DOWNWARDRIGID == 1
      z = [ linspace(0,0.32,5)' ; linspace(0.33,1,nz-5)' ];
      z0idx = sum(z <= z0);
    end
    critgov = 1;   itgov = 1;                              % Initialize
    while abs(critgov) > 3e-4 
      initx;   vfirigid;
      simulrigid;
      critgov = (Rev-Ben)/Rev
      if (itgov > 10) && (critgov > 0),  critgov = 1e-6;   end
      itgov = itgov + 1;
      kappa = kappa - 0.10 * critgov;
      wavg0 = 0.5*wavg + 0.5*wavg0;
    end
    tmp  = 1/(1-(1/(1+r))*(1-gamma))*(1+(1/(1+r))*gamma/(1-(1/(1+r))*(1-chi)));
    UU(tau_u_idx,rho_u_idx) = 100*((U0 - tmp)/(U(1) - tmp) - 1);
    fprintf(' %6.4f',UU(tau_u_idx,rho_u_idx))
  end

end
toc

if DO_GRID == 1
  UU
  [maxA,ind]      = max(UU(:));                            % Show best combination
  [maxrow,maxcol] = ind2sub(size(UU),ind);
  tau_u_g(maxrow)
  rho_u_g(maxcol)
  xelig = tau_u_g;
  xslop = rho_u_g;
  [ xmeshX , xmeshY ] = meshgrid(xelig,xslop);
  surf(xmeshX,xmeshY,UU') %Visualize function values
else
  showstatsrigid;
end

% save optimrigid90.mat