!---------------------------------------------------------------------------------------------------------------------------------------------------
!	module results_transition: characterize the transition path
!---------------------------------------------------------------------------------------------------------------------------------------------------

module results_transition

	use globals
	use misc_functions
	use distribution
	
	implicit none
	
	double precision :: sep_y, sep_o, sep_temp, sep_perm, sep_tot
	double precision :: avg_w_y, avg_z_y, avg_w_o, avg_z_o, avg_z
	double precision :: unemp_y, unemp_o, share_old, ten_dist(5)
	
contains


!	compute various quantities along the transition path

subroutine characterize_transition()
	
	!	initialize distribution function

	ic = 1
	LE = 0.0d+0
	LU = 0.0d+0
	open(1, file = 't0/dist_LEy.txt', status = 'old')
	open(2, file = 't0/dist_LEo.txt', status = 'old')
	open(3, file = 't0/dist_LU.txt', status = 'old')
	do it = 1, NT
		do iw = 1, NW
			read(1, *) (LE(it,iz,iw,ic,1), iz = NZ, 1, -1)
			read(2, *) (LE(it,iz,iw,ic,2), iz = NZ, 1, -1)
		end do
		read(3, *) LU(it,ic,1), LU(it,ic,2), LU(it,ic,3)
	end do
	
	close(1)
	close(2)
	close(3)
	
	
	open(1, file = 'results/transition_path.txt', status = 'unknown')
	open(2, file = 'results/theta_kappa_path.txt', status = 'unknown')
		
	!	compute the time-path for kappa
			
	do ip = 1, NP

		!	compute statistics to characterize transitions path
		
		call compute_statistics()
				
		!	write in text file
		
		write(1, '(I4,24(X,F7.3))') ip, 1.0d+2*kappa(ip), 1.0d+2*(By*sum(LU(:,:,1))+Bo*sum(LU(:,:,2:3)))/(avg_w*sum(LE)), &
									1.0d+2*sum(LU), unemp_y, unemp_o, &
								    theta(ip), sum(LU)/theta(ip), 1.0d+2*jfind, &
								    sep_y, sep_o, sep_temp, sep_perm, sep_tot, &
								    avg_w_y, avg_z_y, avg_w_o, avg_z_o, avg_z, &
								    ten_dist(1), ten_dist(2), ten_dist(3), ten_dist(4), ten_dist(5), share_old
		
		write(2, *) theta(ip), kappa(ip)
		
		!	move on
		
		call update_distribution()
					
	end do
		
	close(1)
	close(2)
	
	
end subroutine


!	subroutine to compute various statistics

subroutine compute_statistics()
	
	double precision ::  sep(NZ)
	
	!	average wages
	
	avg_w_y = 0.0d+0
	avg_w_o = 0.0d+0
	do it = 1, NT
	do iz = 1, NZ
	do iw = 1, NW
	do ic = 1, NC
		avg_w_y = avg_w_y + wage_rule(wGrid(iw), wwy(ip,it,iz,ic))*LE(it,iz,iw,ic,1)
		avg_w_o = avg_w_o + wage_rule(wGrid(iw), wwo(ip,it,iz,ic))*LE(it,iz,iw,ic,2)
	end do
	end do
	end do
	end do
	avg_w   = (avg_w_y+avg_w_o)/sum(LE)
	avg_w_y = avg_w_y/sum(LE(:,:,:,:,1))
	avg_w_o = avg_w_o/sum(LE(:,:,:,:,2))
	
	!	productivity levels
	
	avg_z_y = dot_product(zGrid, sum(sum(sum(LE(:,:,:,:,1), dim = 4), dim = 3), dim = 1))/sum(LE(:,:,:,:,1))
	avg_z_o = dot_product(zGrid, sum(sum(sum(LE(:,:,:,:,2), dim = 4), dim = 3), dim = 1))/sum(LE(:,:,:,:,2))
	avg_z   = dot_product(zGrid, sum(sum(sum(LE(:,:,:,:,1) + LE(:,:,:,:,2), dim = 4), dim = 3), dim = 1))/sum(LE)
	
	
	!	tenure distribution
	
	ten_dist(1) = 1.0d+2*sum(LE(1:3,:,:,:,:))/sum(LE)
	ten_dist(2) = 1.0d+2*sum(LE(1:7,:,:,:,:))/sum(LE)
	ten_dist(3) = 1.0d+2*sum(LE(8:19,:,:,:,:))/sum(LE)
	ten_dist(4) = 1.0d+2*sum(LE(20:39,:,:,:,:))/sum(LE)
	ten_dist(5) = 1.0d+2*sum(LE(40:NT,:,:,:,:))/sum(LE)
	
	share_old = 1.0d+2*sum(LE(:,:,:,1,:))/sum(LE)
	unemp_y   = 1.0d+2*sum(LU(:,:,1))/(sum(LU(:,:,1))+sum(LE(:,:,:,:,1)))
	unemp_o   = 1.0d+2*sum(LU(:,:,2:3))/(sum(LU(:,:,2:3))+sum(LE(:,:,:,:,2)))
	
	!	job-finding rate this period
	
	jfind = min(A*theta(ip)**(1.0d+0-psi),theta(ip),1.0d+0)
	
	
	!	separation rates

	!	young workers
	
	sep_y = 0.0d+0
	do ic = 1, NC
		do iw = 1, NW
		do iz = 1, NZ
		
			do it = 2, NT
		
				ww_r = wage_rule(wGrid(iw), wwy(max(1,ip-1),it-1,iz,ic))
				iw_p = locate(max(wGrid(1), min(wGrid(NW),ww_r)), wGrid)
	
				sep	= (1.0d+0-delta)*(1.0d+0-gamma)*P(iz,:)*LE(it-1,iz,iw,ic,1)
				if (sepy(ip,it,iw_p,ic) .le. NZ) sep_y = sep_y + sum(sep(sepy(ip,it,iw_p,ic):NZ))
				sep = (1.0d+0-delta)*gamma*P(iz,:)*LE(it-1,iz,iw,ic,1)
				if (sepo(ip,it,iw_p,ic) .le. NZ) sep_y = sep_y + sum(sep(sepo(ip,it,iw_p,ic):NZ))
		
			end do
			
			ww_r = wage_rule(wGrid(iw), wwy(max(1,ip-1),NT,iz,ic))
			iw_p = locate(max(wGrid(1), min(wGrid(NW),ww_r)), wGrid)
			
			sep = (1.0d+0-delta)*(1.0d+0-gamma)*P(iz,:)*LE(NT,iz,iw,ic,1)
			if (sepy(ip,NT,iw_p,ic) .le. NZ) sep_y = sep_y + sum(sep(sepy(ip,NT,iw_p,ic):NZ))
			sep = (1.0d+0-delta)*gamma*P(iz,:)*LE(NT,iz,iw,ic,1)
			if (sepo(ip,NT,iw_p,ic) .le. NZ) sep_y = sep_y + sum(sep(sepo(ip,NT,iw_p,ic):NZ))
		
		end do
		end do
	end do

	sep_y = sep_y + delta*sum(LE(:,:,:,:,1))
	
		
	!	old workers
	
	sep_o = 0.0d+0
	do ic = 1, NC
		do iw = 1, NW
		do iz = 1, NZ
		
			do it = 2, NT
				ww_r = wage_rule(wGrid(iw), wwo(max(1,ip-1),it-1,iz,ic))
				iw_p = locate(max(wGrid(1), min(wGrid(NW),ww_r)), wGrid)
						
				sep	= (1.0d+0-delta)*(1.0d+0-khi)*P(iz,:)*LE(it-1,iz,iw,ic,2)
				if (sepo(ip,it,iw_p,ic) .le. NZ) sep_o = sep_o + sum(sep(sepo(ip,it,iw_p,ic):NZ))		
			end do
	
			ww_r = wage_rule(wGrid(iw), wwo(max(1,ip-1),NT,iz,ic))
			iw_p = locate(max(wGrid(1), min(wGrid(NW),ww_r)), wGrid)
					
			sep = (1.0d+0-delta)*(1.0d+0-khi)*P(iz,:)*LE(NT,iz,iw,ic,2)
			if (sepo(ip,NT,iw_p,ic) .le. NZ) sep_o = sep_o + sum(sep(sepo(ip,NT,iw_p,ic):NZ))
	
		end do
		end do
		
		sep_o = sep_o + khi*sum(LE(:,:,:,ic,2))
		sep_o = sep_o + (1.0d+0-khi)*delta*sum(LE(:,:,:,ic,2))
	end do
	
		
	!	first 2 years
	
	sep_temp = 0.0d+0
	
	do ic = 1, NC
	
		do iw = 1, NW
		do iz = 1, NZ	
		do it = 2, 9
	
			ww_r = wage_rule(wGrid(iw), wwy(max(1,ip-1),it-1,iz,ic))
			iw_p = locate(max(wGrid(1), min(wGrid(NW),ww_r)), wGrid)
				
			sep = (1.0d+0-delta)*(1.0d+0-gamma)*P(iz,:)*LE(it-1,iz,iw,ic,1)
			if (sepy(ip,it,iw_p,ic) .le. NZ) sep_temp = sep_temp + sum(sep(sepy(ip,it,iw_p,ic):NZ))				
			sep = (1.0d+0-delta)*gamma*P(iz,:)*LE(it-1,iz,iw,ic,1)
			if (sepo(ip,it,iw_p,ic) .le. NZ) sep_temp = sep_temp + sum(sep(sepo(ip,it,iw_p,ic):NZ))
	
			ww_r = wage_rule(wGrid(iw), wwo(max(1,ip-1),it-1,iz,ic))
			iw_p = locate(max(wGrid(1), min(wGrid(NW),ww_r)), wGrid)
			
			sep = (1.0d+0-delta)*(1.0d+0-khi)*P(iz,:)*LE(it-1,iz,iw,ic,2)
			if (sepo(ip,it,iw_p,ic) .le. NZ) sep_temp = sep_temp + sum(sep(sepo(ip,it,iw_p,ic):NZ))
					
		end do
		end do
		end do
		
		sep_temp = sep_temp + khi*sum(LE(1:8,:,:,ic,2))
		sep_temp = sep_temp + delta*(sum(LE(1:8,:,:,ic,1)) + (1.0d+0-khi)*sum(LE(1:8,:,:,ic,2)))
				
	end do
	
	!	remaining separations
	
	sep_tot  = sep_y + sep_o
	sep_perm = sep_tot-sep_temp
	
	!	scale
	
	sep_y	 = 1.0d+2*sep_y/sum(LE(:,:,:,:,1))
	sep_o	 = 1.0d+2*sep_o/sum(LE(:,:,:,:,2))
	sep_temp = 1.0d+2*sep_temp/sum(LE(1:8,:,:,:,1)+LE(1:8,:,:,:,2))
	sep_perm = 1.0d+2*sep_perm/sum(LE(9:NT,:,:,:,1)+LE(9:NT,:,:,:,2))
	sep_tot  = 1.0d+2*sep_tot/sum(LE)


end subroutine


end module	