!---------------------------------------------------------------------------------------------------------------------------------------------------
!	module rigid_wage: obtain rigid wages for young and old workers
!---------------------------------------------------------------------------------------------------------------------------------------------------

module rigid_wage

	use globals
	use misc_functions
	
	implicit none
	
contains


!	get asset value with rigid wages for young workers

subroutine rigid_wage_value_young()

	do ic = 1, NC
	do it = 1, NT
	
		do iw = 1, NW
		do iz = 1, NZ
		
			ww_r = wage_rule(wGrid(iw), wwy(ip,it,iz,ic))
			iw_p = locate(max(wGrid(1), min(wGrid(NW),ww_r)), wGrid)
		
			Wy_w(it,iz,iw,ic) = utils(ww_r) + rho*(1.0d+0-gamma)*(delta*(UyP(1,ic) + delta_sp*(UyP(min(NT,it+1),ic)-UyP(1,ic))) &
				+(1.0d+0-delta)*dot_product(P(iz,:),max(Wy_w(min(NT,it+1),:,iw_p,ic)*(1.0d+0-sepy_J(min(NT,it+1),:,iw_p,ic)) &
															+sepy_J(min(NT,it+1),:,iw_p,ic)*UyP(min(NT,it+1),ic),UyP(min(NT,it+1),ic)))) &
									+ rho*gamma*(delta*(Uo(1,ic) + delta_sp*(Uo(min(NT,it+1),ic)-Uo(1,ic))) &
				+(1.0d+0-delta)*dot_product(P(iz,:),max(Wo_w(min(NT,it+1),:,iw_p,ic)*(1.0d+0-sepo_J(min(NT,it+1),:,iw_p,ic)) &
															+sepo_J(min(NT,it+1),:,iw_p,ic)*Uo(min(NT,it+1),ic),Uo(min(NT,it+1),ic))))
			
			Jy_w(it,iz,iw,ic) = zGrid(iz)-(1.0d+0+kappa(ip))*ww_r - rho*delta*phi(min(NT,it+1),ic)*delta_sp + rho*(1.0d+0-gamma)* &
								(1.0d+0-delta)*dot_product(P(iz,:),max(Jy_w(min(NT,it+1),:,iw_p,ic)*(1.0d+0-sepy_W(min(NT,it+1),:,iw_p,ic)) &
																			-sepy_W(min(NT,it+1),:,iw_p,ic)*phi(min(NT,it+1),ic),-phi(min(NT,it+1),ic))) &
				+ rho*gamma*(1.0d+0-delta)*dot_product(P(iz,:),max(Jo_w(min(NT,it+1),:,iw_p,ic)*(1.0d+0-sepo_W(min(NT,it+1),:,iw_p,ic)) &
																			-sepo_W(min(NT,it+1),:,iw_p,ic)*phi(min(NT,it+1),ic),-phi(min(NT,it+1),ic)))
	
		end do
		end do

	end do
	end do
		
end subroutine


!	get asset value with rigid wages for older workers

subroutine rigid_wage_value_old()

	!	compute forward
	
	do ic = 1, NC
	do it = 1, NT
	
		do iw = 1, NW		
		do iz = 1, NZ
	
			ww_r = wage_rule(wGrid(iw), wwo(ip,it,iz,ic))
			iw_p = locate(max(wGrid(1), min(wGrid(NW),ww_r)), wGrid)
			
			Wo_w(it,iz,iw,ic) = utils(ww_r) + rho*((1.0d+0-khi)*(delta*(Uo(1,ic) + delta_sp*(Uo(min(NT,it+1),ic)-Uo(1,ic))) &
				+ (1.0d+0-delta)*dot_product(P(iz,:),max(Wo_w(min(NT,it+1),:,iw_p,ic)*(1.0d+0-sepo_J(min(NT,it+1),:,iw_p,ic)) &
														+sepo_J(min(NT,it+1),:,iw_p,ic)*Uo(min(NT,it+1),ic),Uo(min(NT,it+1),ic)))) + khi*R)
						
			Jo_w(it,iz,iw,ic) = zGrid(iz)-(1.0d+0+kappa(ip))*ww_r + rho*(1.0d+0-khi)*(-delta*phi(min(NT,it+1),ic)*delta_sp &
				+ (1.0d+0-delta)*dot_product(P(iz,:),max(Jo_w(min(NT,it+1),:,iw_p,ic)*(1.0d+0-sepo_W(min(NT,it+1),:,iw_p,ic)) &
														-sepo_W(min(NT,it+1),:,iw_p,ic)*phi(min(NT,it+1),ic),-phi(min(NT,it+1),ic))))
			
		end do
		end do

	end do
	end do
		
end subroutine


end module