!---------------------------------------------------------------------------------------------------------------------------------------------------
!	module wage_function_old: obtain wage functions for older workers
!---------------------------------------------------------------------------------------------------------------------------------------------------

module wage_function_old

	use globals
	use misc_functions
	
	implicit none
	
	double precision :: contW, contJ
	
contains


!	subroutine to solve for wages

subroutine solve_wage_old_workers()

	!	solve forwards! (this is very important)
	
	do it = 1, NT
	do iz = 1, NZ
	do ic = 1, NC
	
		!	continuation values here
			
		call continuation_old_workers()

		!	reservation wages
				
		wmin = inv_utils(Uo(it,ic) - contW)
		wmax = (phi(it,ic) + zGrid(iz) + contJ)/(1.0d+0+kappa(ip))
		
		!	solve for the equilibrium wage
		
		if ( wmin .le. wmax ) then
		
			wwo(ip,it,iz,ic) = sqrt((zGrid(iz) + contJ + phi(it,ic))/((1.0d+0 + contW - Uo(it,ic))*(1.0d+0+kappa(ip))))
			!	wwo(ip,it,iz,ic) = solve_nash_foc()
			
		else
		
			wwo(ip,it,iz,ic) = (wmin+wmax)/2.0d+0
			
		end if
		
		!	fill the corresponding value functions
		
		Wo(it,iz,ic) = utils(wwo(ip,it,iz,ic)) + contW
		Jo(it,iz,ic) = zGrid(iz)-(1.0d+0+kappa(ip))*wwo(ip,it,iz,ic) + contJ
	
	end do
	end do
	end do
		
end subroutine


!	continuation values

subroutine continuation_old_workers()

	!	worker's side
	
	contW = rho*((1.0d+0-khi)*(delta*(Uo(1,ic) + delta_sp*(Uo(min(NT,it+1),ic)-Uo(1,ic))) &
			+ (1.0d+0-delta)*dot_product(P(iz,:),max(Wo(min(NT,it+1),:,ic),Uo(min(NT,it+1),ic)))) + khi*R)

	!	firm's side
	
	contJ = rho*(1.0d+0-khi)*(-delta*phi(min(NT,it+1),ic)*delta_sp &
			+ (1.0d+0-delta)*dot_product(P(iz,:),max(Jo(min(NT,it+1),:,ic),-phi(min(NT,it+1),ic))))
		
end subroutine


!	solve Nash problem using FOC (bisection method)

double precision function solve_nash_foc()

	double precision :: x, xa, xb, ya, yb, y
	logical :: ifstop
	
	!	minimum starting point
	xa = wmin
	ya = nash_foc(xa)
	
	!	maximum starting point
	xb = wmax
	yb = nash_foc(xb)
			
	ifstop = .true.
	do while (ifstop)
		
		x = (xa+xb)/2.0d+0
		y = nash_foc(x)
		if ( y*ya .lt. 0.0d+0 ) then
			xb = x
			yb = y
		else
			xa = x
			ya = y
		end if
		
		if (abs(xb-xa) .lt. TOL_nash) ifstop = .false.
		
	end do
	solve_nash_foc = (xa+xb)/2.0d+0
	
end function


!	write down Nash FOC

double precision function nash_foc(ww)
	
	double precision, intent(in) :: ww
	double precision :: valW, valJ
	
	!	worker's side
	
	valW = utils(ww) + contW

	!	firm's side
	
	valJ = zGrid(iz) - (1.0d+0+kappa(ip))*ww  + contJ
		
	!	difference
	
	nash_foc = beta*(valJ + phi(it,ic))/(1.0d+0+kappa(ip)) - (1.0d+0-beta)*(valW-Uo(it,ic))/umarg(ww)
	
end function


end module