!---------------------------------------------------------------------------------------------------------------------------------------------------
!	module distribution: compute the time-invariant distribution
!---------------------------------------------------------------------------------------------------------------------------------------------------

module distribution

	use globals
	use misc_functions
	
	implicit none
	
	
contains


!	retrieve separation decision

subroutine separation_rules()
	
	sepy = NZ+1
	sepo = NZ+1

	do it = 1, NT
	do iz = NZ, 1, -1
		
		if ( min(Jy(it,iz) + phi(it),Wy(it,iz)-(Uy(it)+(1.0d+0-young_sp)*(utils(By+nu*phi(it))-utils(By)))) .le. 0.0d+0 ) sepy(it) = iz
		if ( min(Jo(it,iz) + phi(it),Wo(it,iz)-Uo(it)) .le. 0.0d+0 ) sepo(it) = iz
		
	end do
	end do

end subroutine


!	compute invariant distribution

subroutine invariant()

	double precision :: NewLE(NT,NZ,2), NewLU(NT,3), norm
	double precision :: ERR_dist
	integer :: NUM_dist
	
	!	initialize size distribution
	
	norm = sum(LE) + sum(LU)
	LE  = LE/norm
	LU  = LU/norm
	
	!	transpose transition matrix
	
	P = transpose(P)
			
	!	begin iteration
	
	NUM_dist = 1
	ERR_dist = 1.0d+2
	
		
	do while (ERR_dist .gt. TOL_dist)
		
		!	set new measures to zero
		
		NewLE = 0.0d+0
		NewLU = 0.0d+0
		
		!	employment
		
		do it = 1, NT
			NewLE(1,:,1) = NewLE(1,:,1) + (1.0d+0-gamma)*alpha*jfind*acc(it,:)*P0*LU(it,1)
			NewLU(it,1) = (1.0d+0-gamma)*alpha*jfind*(1.0d+0-sum(acc(it,:)*P0))*LU(it,1)
		end do
		NewLE(1,iz0,1) = NewLE(1,iz0,1) + (1.0d+0-gamma)*(1.0d+0-alpha)*jfind*sum(LU(:,1))
		

		do it = 2, NT-1
		
			NewLE(it,:,1) = (1.0d+0-delta)*(1.0d+0-gamma)*matmul(P, LE(it-1,:,1))
			NewLE(it,:,2) = (1.0d+0-delta)*(gamma*matmul(P, LE(it-1,:,1)) + (1.0d+0-khi)*matmul(P, LE(it-1,:,2)))
			
		end do
		
		NewLE(NT,:,1) = (1.0d+0-delta)*(1.0d+0-gamma)*matmul(P, LE(NT-1,:,1) + LE(NT,:,1))
		NewLE(NT,:,2) = (1.0d+0-delta)*(gamma*matmul(P, LE(NT-1,:,1) + LE(NT,:,1)) + (1.0d+0-khi)*matmul(P, LE(NT-1,:,2) + LE(NT,:,2)))
		
		
		!	separations
		
		do it = 1, NT
		
			if (sepy(it) .le. NZ) then
			
				NewLU(it,1) = NewLU(it,1) + sum(NewLE(it,sepy(it):NZ,1))
				NewLE(it,sepy(it):NZ,1) = 0.0d+0
				
			end if
			
			if (sepo(it) .le. NZ) then
			
				NewLU(it,3) = NewLU(it,3) + sum(NewLE(it,sepo(it):NZ,2))
				NewLE(it,sepo(it):NZ,2) = 0.0d+0
			
			end if
			
		end do
		
		!	unemployment
		
		NewLU(1,1) = NewLU(1,1) + khi*(sum(LU(:,2:3)) + sum(LE(:,:,2))) + (1.0d+0-gamma)*delta*sum(LE(:,:,1))*(1.0d+0-delta_sp)
		NewLU(:,1) = NewLU(:,1) + (1.0d+0-gamma)*((1.0d+0-jfind)*LU(:,1) + delta*sum(LE(:,:,1), dim = 2)*delta_sp)
		NewLU(:,2) = NewLU(:,2) + gamma*LU(:,1) + (1.0d+0-khi)*LU(:,2)
		NewLU(1,3) = NewLU(1,3) + (gamma*delta*sum(LE(:,:,1)) + (1.0d+0-khi)*delta*sum(LE(:,:,2)))*(1.0d+0-delta_sp)
		NewLU(:,3) = NewLU(:,3) + (1.0d+0-khi)*LU(:,3) &
								+ (gamma*delta*sum(LE(:,:,1), dim = 2) + (1.0d+0-khi)*delta*sum(LE(:,:,2), dim = 2))*delta_sp
		
		!	check convergence
		
		ERR_dist = max(maxval(dabs(NewLE - LE)), maxval(dabs(NewLU - LU)))
		
		!	stop if too long
		
		if ( NUM_dist .eq. MAX_dist ) exit
		
		!	update distribution
		
		LE = NewLE
		LU = NewLU
		NUM_dist = NUM_dist + 1

	end do
		
		
	if (ERR_dist .gt. TOL_dist) then
		print *, 'force invariant() to exit'
		print *, 'last error term was ', ERR_dist
		print *, ' '
	end if
	if (isnan(minval(LU)+maxval(LU))) print *, 'X,NaN DETECTED IN DISTRIBUTION: invariant() failed'
	
	
	!	transpose transition matrix back
	
	P = transpose(P)
	
	
end subroutine





end module