% Model with dispersion in initial productivity z0.

clearvars
tic

alfa = 0.50;                                               % Probability of drawing stohcastic z0


%%   Set parameter values.

nz = 121;   stdz = 0.044;   kappa = 9.50;   uniEPL = 0;   z0disp = 1;

if alfa == 0.25
  z0 = 0.31;   b = 0.2358;   bo = 0.1689;   k = 0.2342;   wavg0 = 0.3999;   init;   A = 0.4318;
elseif alfa == 0.50  
  z0 = 0.35;   b = 0.2583;   bo = 0.1795;   k = 0.2446;   wavg0 = 0.4355;   init;   A = 0.4912;
elseif alfa == 0.75
  z0 = 0.44;   b = 0.3037;   bo = 0.2015;   k = 0.2492;   wavg0 = 0.5074;   init;   A = 0.6445;
end


%%   Solve the model.

xx_mu = 0.5;   critmu = 1;                                 % mu: Distribution of unemployed workers across tau
mu = zeros(1,T);   mu(1) = 0.6;   mu(end) = 0.4;           % mu: Initial guess

while critmu > 0.05
  vfistochz0;                                              % Solve value function iteration
  simulstochz0;                                            % Compute stationary distribution
  mu1 = ZZyu(:,1)'/sum(ZZyu(:,1));                         % Actual distribution of unemployed workers
  critmu = max(abs(mu-mu1));   %fprintf('%7.4f ',critmu)   % Convergence metric
  mu = xx_mu*mu1 + (1-xx_mu)*mu;                           % Update guess for mu
end

for calibrationit = 1 : 3
  critgov = 1;                                             % Initialize
  while abs(critgov) > 1e-4                                % Balance the government budget
    initx;
    critmu = 1;                                            % Initialize
    while critmu > 0.05                                    % Find fixed point for mu
      vfistochz0;   simulstochz0;                          % Solve the model
      mu1 = ZZyu(:,1)'/sum(ZZyu(:,1));
      critmu = max(abs(mu-mu1));   %fprintf('%7.4f ',critmu)
      mu = xx_mu*mu1 + (1-xx_mu)*mu;
    end
    critgov = (Rev-Ben)/Rev;                               % Excess revenues
    kappa = kappa - 0.12 * critgov;                        % Adjust payroll tax rate
    %fprintf('%7.4f ',100*kappa)                           % This may be useful to observe convergence
    wavg0 = 0.75*wavg + 0.25*wavg0;                        % Adjust guess for average wage
  end
  b  = b  - 0.28 * (b/wavgy  - 0.58);                      % Calibration: Set new value for b
  bo = bo - 0.28 * (bo/wavgo - 0.4540);   fprintf('x ')    % Calibration: Set new value for bo
end

showstats;
toc