!---------------------------------------------------------------------------------------------------------------------------------------------------
!	module wage_function_young: obtain wage functions for young workers
!---------------------------------------------------------------------------------------------------------------------------------------------------

module wage_function_young

	use globals
	use misc_functions
	
	implicit none
	
	double precision :: contW, contJ

contains


!	subroutine to solve for wages

subroutine solve_wage_young_workers()

	double precision :: ERR_bell, NewU(NT)
	integer :: NUM_bell, it_p
	
	
	!	begin iteration
	
	NUM_bell = 1
	ERR_bell = 1.0d+2
	
	do while (ERR_bell .gt. TOL_bell)
		
		!	update pjob
		
		pjob = alpha*matmul(acc, P0) + 1.0d+0-alpha
		acc  = 1.0d+0
			
		!	current value for Ul
		
		do it = 1, NT
		
			!	expected duration of jobless spell
			
			annfacY(it) = (1.0d+0-gamma+gamma/khi)/(gamma*(1.0d+0-jfind*pjob(it))+jfind*pjob(it))
			annfacY(it) = (1.0d+0-rho)/(1.0d+0-rho**annfacY(it))	
		
			Ul(it) = (utils(Bo + nu*annfacY(it)*phi(it)*young_sp) + leis + rho*khi*R)/(1.0d+0-rho*(1.0d+0-khi))
	
		end do	
		
		!	if young_sp = 0, alter Uy for bargaining 
		
		do it = 1, NT
			Uy(it) = Uy(it) + (1.0d+0-young_sp)*(utils(By+nu*phi(it))-utils(By))
		end do
		
		!	solve terminal problem
		
		call solve_terminal_problem()
		
		!	solve backwards from T-1
		
		do it = NT-1, 1, -1
		do iz = 1, NZ
				
			!	continuation values here
				
			call continuation_young_workers()
				
			!	reservation wages
		
			wmin = inv_utils(Uy(it) - contW)
			wmax = (phi(it) + zGrid(iz) + contJ)/(1.0d+0+kappa)
			
			if (( wmin .le. wmax ) .and. ( wmin .gt. 0.0d+0) ) then
						
				wwy(it,iz) = sqrt((zGrid(iz) + contJ + phi(it))/((1.0d+0 + contW - Uy(it))*(1.0d+0+kappa)))
				!	wwy(it,iz) = solve_nash_foc()
			
			else
			
				wwy(it,iz) = (wmin+wmax)/2.0d+0
				if (it .eq. 1) acc(it,iz)  = 0.0d+0
				
			end if
			
			!	fill the corresponding value functions
			
			Wy(it,iz) = utils(wwy(it,iz)) + contW
			Jy(it,iz) = zGrid(iz) - (1.0d+0+kappa)*wwy(it,iz) + contJ
			
		end do
		end do
		
		!	1st employment period
		
		it = 1
		do it_p = 1, NT
		do iz = 1, NZ
				
			!	continuation values here
				
			call continuation_young_workers()
				
			!	reservation wages
		
			wmin = inv_utils(Uy(it_p) - contW)
			wmax = (phi(1) + zGrid(iz) + contJ)/(1.0d+0+kappa)
			
			if ((wmin .le. wmax) .and. (wmin .gt. 0.0d+0) .and. (zGrid(iz)-(1.0d+0+kappa)*wwy0(it_p,iz)+contJ .gt. -phi(1))) then
						
				wwy0(it_p,iz) = sqrt((zGrid(iz) + contJ + phi(1))/((1.0d+0 + contW - Uy(it_p))*(1.0d+0+kappa)))
				!	wwy0(it,iz) = solve_nash_foc()
			
			else
			
				wwy0(it_p,iz) = (wmin+wmax)/2.0d+0
				acc(it_p,iz)  = 0.0d+0
				
			end if
								
			!	fill the corresponding value functions
			
			Wy0(it_p,iz) = utils(wwy0(it_p,iz)) + contW
			Jy0(it_p,iz) = zGrid(iz) - (1.0d+0+kappa)*wwy0(it_p,iz) + contJ
			
		end do
		end do
		

		!	if young_sp = 0, alter Uy for iterating
		
		do it = 1, NT
			Uy(it) = Uy(it) - (1.0d+0-young_sp)*(utils(By+nu*phi(it))-utils(By))
		end do
				
		!	new value for Uy
		
		do it = 1, NT
		
			NewU(it) = utils(By + nu*annfacY(it)*phi(it)*young_sp) &
					 + rho*((1.0d+0-gamma)*(jfind*(alpha*dot_product(max(Wy0(it,:),Uy(it)),P0) &
					 + (1.0d+0-alpha)*Wy(1,iz0)) + (1.0d+0-jfind)*Uy(it)) + gamma*Ul(it))
													
		end do
		
		!	check convergence
		
		ERR_bell = maxval(dabs(NewU - Uy))

		!	stop if too long
		
		if (NUM_bell .gt. MAX_bell) exit
		
		!	update value functions
		
		Uy = NewU
		NUM_bell = NUM_bell + 1
		
		
	end do
	
	!	print *, 'value functions for younger workers have converged'
	
	if (ERR_bell .gt. TOL_bell) then
		print *, 'force solve_bellman_workers() to exit'
		print *, 'last error term was ', ERR_bell
		print *, ' '
		ERR_cvg = 1
	end if		
		
end subroutine


!	continuation values

subroutine continuation_young_workers()

	!	worker's side
	
	contW = rho*(1.0d+0-gamma)*(delta*(Uy(1) + delta_sp*(Uy(min(NT,it+1))-Uy(1))) &
				+(1.0d+0-delta)*dot_product(P(iz,:),max(Wy(min(NT,it+1),:),Uy(min(NT,it+1))))) &
			+ rho*gamma*(delta*(Uo(1) + delta_sp*(Uo(min(NT,it+1))-Uo(1))) &
				+(1.0d+0-delta)*dot_product(P(iz,:),max(Wo(min(NT,it+1),:),Uo(min(NT,it+1)))))

	!	firm's side
	
	contJ = - rho*delta*phi(min(NT,it+1))*delta_sp &
			+ rho*(1.0d+0-gamma)*(1.0d+0-delta)*dot_product(P(iz,:),max(Jy(min(NT,it+1),:),-phi(min(NT,it+1)))) &
			+ rho*gamma*(1.0d+0-delta)*dot_product(P(iz,:),max(Jo(min(NT,it+1),:),-phi(min(NT,it+1))))
	
	
end subroutine


!	solve problem at time T

subroutine solve_terminal_problem()

	double precision :: NewJ(NZ), NewW(NZ), Newww(NZ), Wres(NZ), Jres(NZ)
	double precision :: ERR_bell
	integer :: NUM_bell
	
	!	set terminal period
	
	it = NT
	
	!	begin iterations
	
	NUM_bell = 1
	ERR_bell = 1.0d+2
	
	do while (ERR_bell .gt. TOL_bell)

		do iz = 1, NZ
		
			!	continuation values
			
			call continuation_young_workers()
					
			!	value function for workers
		
			NewW(iz) = utils(wwy(NT,iz)) + contW
			NewJ(iz) = zGrid(iz) - (1.0d+0+kappa)*wwy(NT,iz) + contJ
						
			!	reservation wage for workers
			
			Wres = max(Wy(NT,:),Uy(NT))
			Wres(iz) = Uy(NT)
			wmin = inv_utils(Uy(NT) &
							- rho*(1.0d+0-gamma)*(delta*(Uy(1) + delta_sp*(Uy(NT)-Uy(1))) &
										+ (1.0d+0-delta)*dot_product(P(iz,:),Wres)) &
							- rho*gamma*(delta*(Uo(1) + delta_sp*(Uo(NT)-Uo(1))) &
										+ (1.0d+0-delta)*dot_product(P(iz,:),max(Wo(NT,:),Uo(NT)))))

			!	reservation wage for firms
			
			Jres = max(Jy(NT,:),-phi(NT))
			Jres(iz) = -phi(NT)
			wmax = (phi(NT) + zGrid(iz) - rho*delta*phi(NT)*delta_sp &
										+ rho*(1.0d+0-gamma)*(1.0d+0-delta)*dot_product(P(iz,:),Jres) &
										+ rho*gamma*(1.0d+0-delta)*dot_product(P(iz,:),max(Jo(NT,:),-phi(NT))))/(1.0d+0+kappa)
			  												
			!	update wage function
			
			if ( wmin .le. wmax ) then
			
				Newww(iz) = sqrt((zGrid(iz) + contJ + phi(NT))/((1.0d+0 + contW - Uy(NT))*(1.0d+0+kappa)))
				!	Newww(iz) = solve_nash_foc()
				
			else
										
				Newww(iz) = (wmin+wmax)/2.0d+0
				
			end if
						
		end do
						
		!	check convergence
		
		ERR_bell = max(maxval(dabs(Wy(NT,:) - NewW)), maxval(dabs(Jy(NT,:) - NewJ)), maxval(dabs(wwy(NT,:) - Newww)))

		!	stop if too long
		
		if (NUM_bell .gt. MAX_bell) exit
		
		!	update value functions
		
		Wy(NT,:)  = NewW
		Jy(NT,:)  = NewJ
		wwy(NT,:) = Newww
		NUM_bell = NUM_bell + 1
		
	end do

	!print *, 'value functions for young workers have converged'
	
	if (ERR_bell .gt. TOL_bell) then
		print *, 'force solve_bellman() to exit (young workers)'
		print *, 'last error term was ', ERR_bell
		print *, ' '
		ERR_cvg = 1
	end if
	
end subroutine


!	solve Nash problem using FOC (bisection method)

double precision function solve_nash_foc()

	double precision :: x, xa, xb, ya, yb, y
	logical :: ifstop
	
	!	minimum starting point
	xa = wmin
	ya = nash_foc(xa)
	
	!	maximum starting point
	xb = wmax
	yb = nash_foc(xb)
			
	ifstop = .true.
	do while (ifstop)
		
		x = (xa+xb)/2.0d+0
		y = nash_foc(x)
		if ( y*ya .lt. 0.0d+0 ) then
			xb = x
			yb = y
		else
			xa = x
			ya = y
		end if
		
		if (abs(xb-xa) .lt. TOL_nash) ifstop = .false.
		
	end do
	solve_nash_foc = (xa+xb)/2.0d+0
	
end function


!	write down Nash FOC

double precision function nash_foc(ww)
	
	double precision, intent(in) :: ww
	double precision :: valW, valJ
	
	!	worker's side
	
	valW = utils(ww) + contW

	!	firm's side
	
	valJ = zGrid(iz) - (1.0d+0+kappa)*ww + contJ
		
	!	difference
	
	nash_foc = beta*(valJ + phi(it))/(1.0d+0+kappa) - (1.0d+0-beta)*(valW-Uy(it))/umarg(ww)
	
end function


end module