!---------------------------------------------------------------------------------------------------------------------------------------------------
!	module distribution: compute the time-invariant distribution
!---------------------------------------------------------------------------------------------------------------------------------------------------

module distribution

	use globals
	use misc_functions
	
	implicit none
	
contains


!	retrieve separation decision

subroutine separation_rules()

	sepy_0 = NZ+1
	sepo_0 = NZ+1

	do it = 1, NT
	do iz = NZ, 1, -1
		
		if ( min(Jy_0(it,iz) + phi(it), & 
					Wy_0(it,iz)-(Uy(it)+(1.0d+0-young_sp)*(utils(By+nu*phi(it))-utils(By)))) .le. 0.0d+0 ) sepy_0(it) = iz
		if ( min(Jo_0(it,iz) + phi(it),Wo_0(it,iz)-Uo(it)) .le. 0.0d+0 ) sepo_0(it) = iz
		
	end do
	end do

	sepy_1 = NZ+1
	sepo_1 = NZ+1

	do it = 1, NT
	do iz = NZ, 1, -1
		
		if ( min(Jy_1(it,iz) + phi(it), & 
					Wy_1(it,iz)-(Uy(it)+(1.0d+0-young_sp)*(utils(By+nu*phi(it))-utils(By)))) .le. 0.0d+0 ) sepy_1(it) = iz
		if ( min(Jo_1(it,iz) + phi(it),Wo_1(it,iz)-Uo(it)) .le. 0.0d+0 ) sepo_1(it) = iz
		
	end do
	end do

end subroutine


!	compute invariant distribution

subroutine invariant()

	double precision :: NewLE_0(NT,NZ,2), NewLE_1(NT,NZ,2), NewLU(NT,3), norm
	double precision :: ERR_dist
	integer :: NUM_dist
	
	!	initialize size distribution
	
	norm = sum(LE_0+LE_1) + sum(LU)
	LE_0  = LE_0/norm
	LE_1  = LE_1/norm
	LU  = LU/norm
	
	!	transpose transition matrix
	
	P = transpose(P)
			
	!	begin iteration
	
	NUM_dist = 1
	ERR_dist = 1.0d+2
	
		
	do while (ERR_dist .gt. TOL_dist)
		
		!	set new measures to zero
		
		NewLE_0 = 0.0d+0
		NewLE_1 = 0.0d+0
		NewLU = 0.0d+0
		
		!	employment
		
		NewLE_0(1,iz0,1) = (1.0d+0-gamma)*jfind*sum(LU(:,1))
				
		do it = 2, NT-1
		
			NewLE_0(it,:,1) = (1.0d+0-delta)*(1.0d+0-gamma)*matmul(P, (1.0d+0-pi_y(it-1,:))*LE_0(it-1,:,1))
			NewLE_0(it,:,2) = (1.0d+0-delta)*(gamma*matmul(P, (1.0d+0-pi_y(it-1,:))*LE_0(it-1,:,1)) &
												+ (1.0d+0-khi)*matmul(P, (1.0d+0-pi_o(it-1,:))*LE_0(it-1,:,2)))

			NewLE_1(it,:,1) = (1.0d+0-delta)*(1.0d+0-gamma)*matmul(P, pi_y(it-1,:)*LE_0(it-1,:,1)) + &
								(1.0d+0-delta)*(1.0d+0-gamma)*matmul(P, LE_1(it-1,:,1))
			NewLE_1(it,:,2) = (1.0d+0-delta)*(gamma*matmul(P, pi_y(it-1,:)*LE_0(it-1,:,1)) &
								+ (1.0d+0-khi)*matmul(P, pi_o(it-1,:)*LE_0(it-1,:,2))) &
							+ (1.0d+0-delta)*(gamma*matmul(P, LE_1(it-1,:,1)) + (1.0d+0-khi)*matmul(P, LE_1(it-1,:,2)))
						
		end do
		
		NewLE_0(NT,:,1) = (1.0d+0-delta)*(1.0d+0-gamma)*matmul(P, (1.0d+0-pi_y(NT-1,:))*LE_0(NT-1,:,1) +(1.0d+0-pi_y(NT,:))*LE_0(NT,:,1))
		NewLE_0(NT,:,2) = (1.0d+0-delta)*(gamma*matmul(P, (1.0d+0-pi_y(NT-1,:))*LE_0(NT-1,:,1) + (1.0d+0-pi_y(NT,:))*LE_0(NT,:,1)) &
							+ (1.0d+0-khi)*matmul(P, (1.0d+0-pi_o(NT-1,:))*LE_0(NT-1,:,2) + (1.0d+0-pi_o(NT,:))*LE_0(NT,:,2)))
		
		NewLE_1(NT,:,1) = (1.0d+0-delta)*(1.0d+0-gamma)*matmul(P, pi_y(NT-1,:)*LE_0(NT-1,:,1) + pi_y(NT,:)*LE_0(NT,:,1)) + &
							(1.0d+0-delta)*(1.0d+0-gamma)*matmul(P, LE_1(NT-1,:,1) + LE_1(NT,:,1))
		NewLE_1(NT,:,2) = (1.0d+0-delta)*(gamma*matmul(P, pi_y(NT-1,:)*LE_0(NT-1,:,1) + pi_y(NT,:)*LE_0(NT,:,1)) &
							+ (1.0d+0-khi)*matmul(P, pi_o(NT-1,:)*LE_0(NT-1,:,2) + pi_o(NT,:)*LE_0(NT,:,2))) + &
							(1.0d+0-delta)*(gamma*matmul(P, LE_1(NT-1,:,1) + LE_1(NT,:,1)) + (1.0d+0-khi)*matmul(P, LE_1(NT-1,:,2) + LE_1(NT,:,2)))
		
		
		!	separations
		
		do it = 1, NT
		
			if (sepy_0(it) .le. NZ) then
			
				NewLU(it,1) = NewLU(it,1) + sum(NewLE_0(it,sepy_0(it):NZ,1))
				NewLE_0(it,sepy_0(it):NZ,1) = 0.0d+0
				
			end if
			if (sepy_1(it) .le. NZ) then
			
				NewLU(it,1) = NewLU(it,1) + sum(NewLE_1(it,sepy_1(it):NZ,1))
				NewLE_1(it,sepy_1(it):NZ,1) = 0.0d+0
				
			end if
			
			if (sepo_0(it) .le. NZ) then
			
				NewLU(it,3) = NewLU(it,3) + sum(NewLE_0(it,sepo_0(it):NZ,2))
				NewLE_0(it,sepo_0(it):NZ,2) = 0.0d+0
			
			end if
			if (sepo_1(it) .le. NZ) then
			
				NewLU(it,3) = NewLU(it,3) + sum(NewLE_1(it,sepo_1(it):NZ,2))
				NewLE_1(it,sepo_1(it):NZ,2) = 0.0d+0
			
			end if
			
		end do
		
		!	unemployment
		
		NewLU(1,1) = NewLU(1,1) + khi*(sum(LU(:,2:3)) + sum(LE_0(:,:,2)+LE_1(:,:,2))) &
								+ (1.0d+0-gamma)*delta*sum(LE_0(:,:,1)+LE_1(:,:,1))*(1.0d+0-delta_sp)
		NewLU(:,1) = NewLU(:,1) + (1.0d+0-gamma)*((1.0d+0-jfind)*LU(:,1) + delta*sum(LE_0(:,:,1)+LE_1(:,:,1), dim = 2)*delta_sp)
		NewLU(:,2) = NewLU(:,2) + gamma*LU(:,1) + (1.0d+0-khi)*LU(:,2)
		NewLU(1,3) = NewLU(1,3) + (gamma*delta*sum(LE_0(:,:,1)+LE_1(:,:,1)) &
								+ (1.0d+0-khi)*delta*sum(LE_0(:,:,2)+LE_1(:,:,2)))*(1.0d+0-delta_sp)
		NewLU(:,3) = NewLU(:,3) + (1.0d+0-khi)*LU(:,3) &
								+ (gamma*delta*sum(LE_0(:,:,1)+LE_1(:,:,1), dim = 2) + (1.0d+0-khi)*delta*sum(LE_0(:,:,2)+LE_1(:,:,2), dim = 2))*delta_sp
		
		!	check convergence
		
		ERR_dist = max(maxval(dabs(NewLE_1 - LE_1)), maxval(dabs(NewLE_0 - LE_0)), maxval(dabs(NewLU - LU)))
		
		!	stop if too long
		
		if ( NUM_dist .eq. MAX_dist ) exit
		
		!	update distribution
		
		LE_0 = NewLE_0
		LE_1 = NewLE_1
		LU = NewLU
		NUM_dist = NUM_dist + 1

	end do	
		
	if (ERR_dist .gt. TOL_dist) then
		print *, 'force invariant() to exit'
		print *, 'last error term was ', ERR_dist
		print *, ' '
	end if
	if (isnan(minval(LU)+maxval(LU))) print *, 'X,NaN DETECTED IN DISTRIBUTION: invariant() failed'
	
	!	transpose transition matrix back
	
	P = transpose(P)
	
end subroutine

end module