!---------------------------------------------------------------------------------------------------------------------------------------------------
!	module misc_functions: various functions and subroutines
!---------------------------------------------------------------------------------------------------------------------------------------------------

module misc_functions

	use globals
	
	implicit none

contains


!	load model parameters

subroutine initial_parameters()

	double precision :: PInv(NZ,NZ), zmin, zmax, zinit, step, zGridInv(NZ)
	integer :: iz_p
	
	!	read parameters
		
	open(1, file = 'params.txt', status = 'old')
	
	read(1, *) delta_sp
	read(1, *) young_sp
	read(1, *) model_period
	read(1, *) rho
	rho = 1.0d+0/(1.0d+0 + rho)
	
	read(1, *) sigma
	read(1, *) leis
	
	!	life-cycle
	
	read(1, *) gamma
	read(1, *) khi
	
	gamma = 1.0d+0/(dble(model_period)*gamma)
	khi   = 1.0d+0/(dble(model_period)*khi)
		
	read(1, *) nu
	read(1, *) By
	read(1, *) Bo
	read(1, *) k
	
	!	retirement income
	
	read(1, *) R
	if (R .gt. 0.0d+0) R  = (utils(R) + leis)/(1.0d+0-rho*(1.0d+0-1.0d+0/dble(model_period*20)))
	
	!	productivity process

	read(1, *) zinit
	read(1, *) prob_z
	read(1, *) sig_z
		
	zmin  = 0.0d+0
	zmax  = 1.0d+0
	step  = (zmax-zmin)/dble(NZ-1)
	do iz = 1, NZ
		zGridInv(iz) = zmin + step*dble(iz-1)
	end do
		
	iz0 = locate(zinit, zGridInv)

	print *, ' '
	print *, 'Note: initial z', iz0, real(zGridInv(iz0))
	print *, ' '
	
	
	open(2, file = 'zGrid.txt', status = 'unknown')
	do iz = 1, NZ
		write(2, *) iz, zGridInv(iz) 
	end do
	close(2)
	
	PInv = 0.0d+0
	do iz = 1, NZ
		PInv(iz,iz) = 1.0d+0 - prob_z
  		PInv(iz, 1) = PInv(iz,1)+ prob_z*normal_cdf(0.5d+0*(zGridInv(1)+zGridInv(2))-zGridInv(iz),0.0d+0,sig_z)
  		do iz_p = 2, NZ-1
    		PInv(iz,iz_p) = PInv(iz,iz_p) + prob_z*(normal_cdf(0.5d+0*(zGridInv(iz_p)+zGridInv(iz_p+1)) - zGridInv(iz), 0.0d+0 ,sig_z) &
    											  - normal_cdf(0.5d+0*(zGridInv(iz_p-1)+zGridInv(iz_p)) - zGridInv(iz) ,0.0d+0 ,sig_z))
  		end do
  		PInv(iz,NZ) = PInv(iz,NZ) + prob_z*(1.0d+0-normal_cdf(0.5d+0*(zGridInv(NZ-1)+zGridInv(NZ))-zGridInv(iz),0.0d+0,sig_z))
	end do
	

	!	inverse order
	
	do iz = 1, NZ
		zGrid(NZ+1-iz) = zGridInv(iz)
	end do	
	iz0 = NZ+1-iz0
	
	do iz = 1, NZ
	do iz_p = 1, NZ
		P(NZ+1-iz,NZ+1-iz_p) = PInv(iz,iz_p)
	end do
	end do
	
	!	matching parameters
	
	read(1, *) delta
	read(1, *) A
	read(1, *) psi
	read(1, *) beta
	
	!	HC parameters
	
	read(1, *) H
	read(1, *) alpha0
	read(1, *) alpha1
	
	close(1)
	
	!	annuity factor for old workers
	
	annfacO = (1.0d+0-rho)/(1.0d+0-rho**(1.0d+0/khi))
	
end subroutine


!	locate x in xgrid

integer function locate(x, xgrid)
	implicit none

	double precision, intent(in) :: x, xgrid(:)
	
	integer :: N, ju, jm, jl, jndx
	
	N  = size(xgrid)
	jl = 0
	ju = N+1
	do while (ju - jl > 1)
		jm = (ju + jl)/2
		if (x .ge. xgrid(jm)) then
			jl = jm
		else
			ju = jm
		end if
	end do

	if (x .eq. xgrid(1)) then 
		jndx = 1
	else if (x .eq. xgrid(N)) then
		jndx = N - 1
	else 
		jndx = jl
	end if
	locate = jndx

end function


!	cdf of normal distribution (N(mu,std^2) evaluated at x)

double precision function normal_cdf(x, mu, std)
	implicit none
	double precision, intent(in) :: x, mu, std	
	normal_cdf = 1.0d+0 - erfc((x-mu)/(std*sqrt(2.0d+0)))/2.0d+0

end function


!	set initial guesses

subroutine set_initial_guesses()

	kappa = 10.49d-2
	theta = 1.00d+0
	avg_w = 0.5170d+0
	
	do iz = 1, NZ
		wwo_0(:,iz) = beta*zgrid(iz) + (1.0d+0-beta)*Bo
		wwy_0(:,iz) = beta*zgrid(iz) + (1.0d+0-beta)*By
		wwo_1(:,iz) = beta*zgrid(iz)*(1.0d+0+H) + (1.0d+0-beta)*Bo
		wwy_1(:,iz) = beta*zgrid(iz)*(1.0d+0+H) + (1.0d+0-beta)*By		
	end do
	
	eff_y = 0.0d+0
	eff_o = 0.0d+0
	pi_y = 0.0d+0
	pi_o = 0.0d+0
	
	do it = 1, NT
	do iz = 1, NZ
	
		Wo_0(it,iz) = utils(wwo_0(it,iz))/(1.0d+0-rho)
		Wy_0(it,iz) = utils(wwy_0(it,iz))/(1.0d+0-rho)
		Wo_1(it,iz) = utils(wwo_1(it,iz))/(1.0d+0-rho)
		Wy_1(it,iz) = utils(wwy_1(it,iz))/(1.0d+0-rho)
		
		Jo_0(it,iz) = (zGrid(iz)-wwo_0(it,iz))/(1.0d+0-rho)
		Jy_0(it,iz) = (zGrid(iz)-wwy_0(it,iz))/(1.0d+0-rho)
		Jo_1(it,iz) = (zGrid(iz)*(1.0d+0+H)-wwo_1(it,iz))/(1.0d+0-rho)
		Jy_1(it,iz) = (zGrid(iz)*(1.0d+0+H)-wwy_1(it,iz))/(1.0d+0-rho)		
		
	end do
	end do
	
	Uy = utils(By)/(1.0d+0-rho)
	LE_0 = 0.0d+0
	LE_1 = 0.0d+0
	LU = 0.0d+0
	LU(1,1) = 1.0d+0

end subroutine


!	set the layoff payments functions

subroutine initialize_layoff_payments()
	
	double precision :: phi0, phi1
	
	!	phi function
	
	phi = 0.0d+0
	
	!	benchmark phi function
	
	do it = 2, 9
		phi(it) = (8.0d+0/365.0d+0)*avg_w*dble(it-1)
	end do
	do it = 10, 114
		phi(it) = (45.0d+0/365.0d+0)*avg_w*dble(it-1)
	end do
	phi(115:NT) = (avg_w*4.0d+0)*3.5d+0

	!	optimal phi function
	
	phi0 =  5.0d+0
	phi1 =  20.0d+0
	do it = 1, NT
		phi(it) = max(0.0d+0, (phi1/dble(365))*avg_w*(dble(it-1)-phi0/3.0d+0))
	end do
	phi = 0.0d+0
	
	!	value function for old workers
	
	do it = 1, NT
		Uo(it) = (utils(Bo + nu*annfacO*phi(it)) + leis + rho*khi*R)/(1.0d+0-rho*(1.0d+0-khi))
	end do
		
end subroutine


!	utility function

double precision function utils(cc)
	
	double precision, intent(in) :: cc
	if (sigma .eq. 1.0d+0) utils = log(max(cc,TINY))
	if (sigma .gt. 1.0d+0) utils = (max(cc,TINY)**(1.0d+0-sigma) - 1.0d+0)/(1.0d+0-sigma)
	
end function


!	marginal utility

double precision function umarg(cc)
	
	double precision, intent(in) :: cc
	umarg = max(cc,TINY)**(-sigma)
	
end function


!	inverse utility function

double precision function inv_utils(uu)
	
	double precision, intent(in) :: uu
	if (sigma .eq. 1.0d+0) inv_utils = exp(uu)
	if (sigma .gt. 1.0d+0) then
		if (uu .lt. 1.0d+0/(sigma-1.0d+0)) inv_utils = (1.0d+0+(1.0d+0-sigma)*uu)**(1.0d+0/(1.0d+0-sigma))
		if (uu .ge. 1.0d+0/(sigma-1.0d+0)) inv_utils = 0.0d+0
	end if

	
end function


end module