% vfihumcap.m

crito = 1;                                                 % Stopping criterion
crity = 1;                                                 % Stopping criterion


%%   (i) Value function iteration for older workers.

C1 = repmat(UoA(aux,:),1,N);      C2 = -repmat(philag,1,N);          % Some matrices (can be precomputed)
C3 = max(repmat(z',T,1),1e-4);    C4 =  repmat(phi,1,N);             % Some matrices (can be precomputed)
C8 = zeros(T,N);

ito = 0;
eo  = 0.10*ones(T,N);                                                % Initialize policy function for effort

while crito > 1e-4
  C5  = -(1./woW - 1);                                               % Some matrix (can be precomputed)
  C6  = C3.*(1-eo) - woF*(1+kappa);                                  % Some matrix (can be precomputed)
  C5H = -(1./woWH - 1);                                              % Some matrix (can be precomputed)
  C6H = C3*(1+Hprem) - woFH*(1+kappa);                               % Some matrix (can be precomputed)
  pie = pi1 * eo.^pi2;                                               % Probability of acquiring human capital
  for i2 = 1 : io1
    Wo  = C5  + (1/(1+r)) * (1-chi) * ( (1-delta)*((1-pie).*max(Wo(aux,:),C1) + pie.*max(WoH(aux,:),C1))*P' + delta*C1 );
    Jo  = C6  + (1/(1+r)) * (1-chi) * ( (1-delta)*((1-pie).*max(Jo(aux,:),C2) + pie.*max(JoH(aux,:),C2))*P' + delta*C2 );
    WoH = C5H + (1/(1+r)) * (1-chi) * ( (1-delta)*max( WoH(aux,:) , C1 )*P' + delta*C1 );
    JoH = C6H + (1/(1+r)) * (1-chi) * ( (1-delta)*max( JoH(aux,:) , C2 )*P' + delta*C2 );
  end
  
  % Wage determination for matches with human capital (identical to baseline model).
  
  splitoH = C8;                                 CVWH  = WoH - C5H - repmat(UoA,1,N);
  CVFH  = JoH - C6H;                            resWH = 1./(1 + CVWH);
  resFH = (1/(1+kappa))*(C3*(1+Hprem) + C4 + CVFH);
  wo1H = sqrt(resFH.*resWH*beta/(1-beta));
  wo1H(resWH > resFH) = woH(resWH > resFH);     splitoH(resWH > resFH) = 1;
  
  % Wage and effort determination for matches without human capital.
  
  splito = C8;                                             % Initialize
  CVW    = Wo - C5 - repmat(UoA,1,N);                      % Identical as in baseline
  CVF    = Jo - C6;                                        % Identical as in baseline
  wo1    = sqrt((C3.*(1-eo) + CVF + C4)./((1+kappa)*(1 + CVW)));     % Wage determination equation (given e)
  resW   = 1./(1 + CVW);                                   % Identical to baseline
  resF   = (C3.*(1-eo) + CVF + C4)/(1+kappa);              % Augmented by (1-eo)
  
  wo1(resW > resF) = wo(resW > resF);                      % Take care of unprofitable matches
  splito(resW > resF) = 1;                                 % Take care of unprofitable matches
  
  tmp1 = (1/(1+r)) * (1-chi) * (1-delta)* ( max(WoH(aux,:),C1) - max(Wo(aux,:),C1) )*P';   % Wtilde_H - Wtilde
  tmp2 = (1/(1+r)) * (1-chi) * (1-delta)* ( max(JoH(aux,:),C2) - max(Jo(aux,:),C2) )*P';   % Jtilde_H - Jtilde

  uprime = 1./wo.^2;                                       % Here I take the previous guess for the wage
  tmp3   = max(pi1 * pi2 * (uprime.*tmp2 - (1+kappa)*tmp1),1e-5);     % This is the denominator (make sure it is positive)
  eo1    = min((uprime.*C3./tmp3).^(1/(pi2-1)),1);         % Implied optimal effort (given w)
  eo1(resW > resF) = eo(resW > resF);                      % Take care of unprofitable matches

  criteo  = 100*max(max(abs(eo1-eo)./eo));                 % Metric 1: Effort eo
  critwo1 = 100*max(max(abs(wo1-wo)./wo));                 % Metric 2: Wage wo
  critwo2 = 100*max(max(abs(wo1H-woH)./woH));              % Metric 3: Wage woH
  crito   = max(max(critwo1,critwo2),criteo);              % Take the maximum

  eo = xx*eo1 + (1-xx)*eo;
  wo = xx*wo1 + (1-xx)*wo;                      woH = xx*wo1H + (1-xx)*woH;
  woF = wo;                                     woF(splito==1)   = 4.9999;
  woW = wo;                                     woW(splito==1)   = 0.001;
  woFH = woH;                                   woFH(splitoH==1) = 4.9999;
  woWH = woH;                                   woWH(splitoH==1) = 0.001;
  ito = ito + 1;
end

pio = pie;


%%   (ii) Value function iteration for young workers.

CV_WH = (1/(1+r))*gamma*( (1-delta)*max( WoH(aux,:) , C1 )*P' + delta*C1 );
CV_JH = (1/(1+r))*gamma*( (1-delta)*max( JoH(aux,:) , C2 )*P' + delta*C2 );

maxity = 1500;                                             % Maximum number of iterations (show warning if hit)
ity    = 0;                                                % Initialize iteration counter
C7     = (1/(1+r))*(1-gamma);                              % Some number (can be precomputed)
e      = 0.10*ones(T,N);                                   % Initialize policy function for effort

while (crity > 1e-5) && (ity < maxity)                     % Outer loop over equilibrium wage function
  
  for i2 = 1 : iy1                                         % I iterate over theta a prespecified number of times
    
    q = A * theta^(-psi);
    jobfindprob = q*theta;
    expdur   = (1-gamma+gamma/chi)/(gamma*(1-jobfindprob) + jobfindprob);
    annfacy  = (r/(1+r)) * (1/(1-(1+r)^(-expdur)));        % Annuity factor: Young workers
    
    btildey  = b  + upsil*annfacy*phi;                     % Non-employment income for young workers
    btildeoB = bo + upsil*annfacy*phi;                     % Non-employment income for older workers
    UoB = -(1./btildeoB-1)/(1-(1/(1+r))*(1-chi));          % Non-employed older workers: laid off when young
    
    RRu  = -(1./btildey-1);                                % Some matrix (can be precomputed)
    RRe  = -(1./wW-1);                                     % Single-period return matrix (can be precomputed)
    RRJ  = C3.*(1-e) - wF*(1+kappa);                       % Single-period return matrix (can be precomputed)
    RReH = -(1./wWH-1);                                    % Single-period return matrix (can be precomputed)
    RRJH = C3*(1+Hprem) - wFH*(1+kappa);                   % Single-period return matrix (can be precomputed)
    pie  = pi1 * e.^pi2;                                   % Probability of acquiring human capital
    
    CV_W = (1/(1+r))*gamma*((1-delta)*((1-pie).*max(Wo(aux,:),C1) + pie.*max(WoH(aux,:),C1))*P' + delta*C1);
    CV_J = (1/(1+r))*gamma*((1-delta)*((1-pie).*max(Jo(aux,:),C2) + pie.*max(JoH(aux,:),C2))*P' + delta*C2);
    
    for i3 = 1 : iy2
      U  = RRu  + (1/(1+r)) * ( (1-gamma)*(q*theta*W(1,z0idx) + (1-q*theta)*U) + gamma*UoB );
      UX = repmat(U(aux,1),1,N);
      W  = RRe  + C7*( (1-delta)*((1-pie).*max(W(aux,:),UX) + pie.*max(WH(aux,:),UX))*P' + delta*UX) + CV_W;
      J  = RRJ  + C7*( (1-delta)*((1-pie).*max(J(aux,:),C2) + pie.*max(JH(aux,:),C2))*P' + delta*C2) + CV_J;
      WH = RReH + C7*( (1-delta)*max(WH(aux,:),UX)*P' + delta*UX ) + CV_WH;
      JH = RRJH + C7*( (1-delta)*max(JH(aux,:),C2)*P' + delta*C2 ) + CV_JH;
    end

    theta1 = ( (1/k)*(1/(1+r)) * A * max(J(1,z0idx),0) )^(1/psi);
    theta  = (1-xx)*theta + xx*theta1;
    
  end
  
  % Wage determination for matches with human capital (identical to baseline model).
  
  splitH = C8;                                   CVWH  = WH - RReH - repmat(U,1,N);
  CVFH  = JH - RRJH;                             resWH = 1./(1 + CVWH);  
  resFH = (1/(1+kappa))*(C3*(1+Hprem) + C4 + CVFH);
  w1H   = sqrt((C3*(1+Hprem) + CVFH + C4)./((1+kappa)*(1 + CVWH)));
  invalH = (resWH > resFH) + (w1H < 0) > 0;
  w1H(invalH) = wH(invalH);                      splitH(invalH) = 1;
  
  % Wage and effort determination for matches without human capital.

  split = C8;
  CVW   = W - RRe - repmat(U,1,N);
  CVF   = J - RRJ;
  w1    = sqrt((C3.*(1-e) + CVF + C4)./((1+kappa)*(1 + CVW)));
  resW  = 1./(1 + CVW);
  resF  = (C3.*(1-e) + CVF + C4)/(1+kappa);

  inval  = (resW > resF) + (w1 < 0) > 0;
  w1(inval) = w(inval);
  split(inval) = 1;
  
  % (Wtilde_H - Wtilde) und (Jtilde_H - Jtilde)
  tmp1 = (1/(1+r))*(1-gamma)*(1-delta)*(max(WH(aux,:) ,UX) - max(W(aux,:)  ,UX))*P' + ...
         (1/(1+r))*   gamma *(1-delta)*(max(WoH(aux,:),C1) - max(WoH(aux,:),C1))*P';
  tmp2 = (1/(1+r))*(1-gamma)*(1-delta)*(max(JH(aux,:) ,C2) - max(J(aux,:)  ,C2))*P' + ...
         (1/(1+r))*   gamma *(1-delta)*(max(Jo(aux,:) ,C2) - max(JoH(aux,:),C2))*P';

  uprime = 1./w.^2;                                                 % Here I take the previous guess for the wage
  tmp3   = max(pi1 * pi2 * (uprime.*tmp2 - (1+kappa)*tmp1),1e-5);   % This is the denominator (make sure it is positive)
  e1     = min((uprime.*C3./tmp3).^(1/(pi2-1)),1);                  % Implied optimal effort (given w)
  
  e1(inval) = e1(1,sum(split(1,:)) + 1);                            % Unprofitable matches: Set e to reasonable value

  crite  = 100*max(max(abs(e1-e)./e));                     % Metric 1: Effort eo
  critw1 = 100*max(max(abs(w1-w)./w));                     % Metric 2: Wage wo
  critw2 = 100*max(max(abs(w1H-wH)./wH));                  % Metric 3: Wage woH
  crity  = max(max(critw1,critw2),crite);                  % Take the maximum
  
  e   = xx*e1 + (1-xx)*e;
  w   = xx*w1  + (1-xx)*w;
  wH  = xx*w1H + (1-xx)*wH;
  wF  = w;                                       wF(split==1)   = 4.9999;
  wW  = w;                                       wW(split==1)   = 0.001;
  wFH = wH;                                      wFH(splitH==1) = 4.9999;
  wWH = wH;                                      wWH(splitH==1) = 0.001;
  ity = ity + 1;
  
end

ey  = e;
piy = pie;

if (q > 1),           disp('Careful: Job-filling probability is larger than 1!');   end
if (q*theta > 1),     disp('Careful: Job-finding probability is larger than 1!');   end  
if (ity == maxity),   disp('Hit maxity --> Discard!');                              end

clear C1 C2 C3 C4 C5 C6 C7 CV_J CV_W CVF CVW i1 i2 i3 J1 Jo1 resF resW RRe RRJ RRu TMPq theta1 w1 W1 wo1 Wo1 crito ...
      crity expdur inval ity j V0 U1 maxity philag wW wF woW woF crite criteo critw1 critw2 critwo1 critwo2 ito e e1 ...
      eo1 UX uprime tmp1 tmp2 tmp3 pie 