!---------------------------------------------------------------------------------------------------------------------------------------------------
!	module wage_function_old: obtain wage functions for older workers
!---------------------------------------------------------------------------------------------------------------------------------------------------

module wage_function_old

	use globals
	use misc_functions
	
	implicit none
	
	double precision :: contW_0, contJ_0, contW_1, contJ_1
	
contains


!	subroutine to solve for wages

subroutine solve_wage_old_workers()


	!	solve terminal problem

	call solve_terminal_problem()
	
	!	solve backwards from T-1
	
	do it = NT-1, 1, -1
	do iz = 1, NZ

		!	continuation values here
			
		call continuation_old_workers()

		
		!	reservation wages
				
		wmin = inv_utils(Uo(it) - contW_1)
		wmax = (phi(it) + zGrid(iz)*(1.0d+0+H) + contJ_1)/(1.0d+0+kappa)
		
		!	solve for the equilibrium wage
		
		if (( wmin .le. wmax ) .and. (1.0d+0 + contW_1 - Uo(it) .gt. 0.0d+0)) then
		
			wwo_1(it,iz) = sqrt((zGrid(iz)*(1.0d+0+H) + contJ_1 + phi(it))/((1.0d+0 + contW_1 - Uo(it))*(1.0d+0+kappa)))
			
		else
		
			wwo_1(it,iz) = (wmin+wmax)/2.0d+0
			
		end if
		
		!	fill the corresponding value functions
		
		Wo_1(it,iz) = utils(wwo_1(it,iz)) + contW_1
		Jo_1(it,iz) = zGrid(iz)*(1.0d+0+H)-(1.0d+0+kappa)*wwo_1(it,iz) + contJ_1
	
		
		!	optimal effort to acquire HC
		
		call optim_effort_old_workers()
		
		!	reservation wages
				
		wmin = inv_utils(Uo(it) - contW_0)
		wmax = (phi(it) + zGrid(iz)*(1.0d+0 - eff_o(it,iz)) + contJ_0)/(1.0d+0+kappa)
		
		!	solve for the equilibrium wage
		
		if (( wmin .le. wmax ) .and. (1.0d+0 + contW_0 - Uo(it) .gt. 0.0d+0) .and. (wmax .gt. 0.0d+0)) then
		
			wwo_0(it,iz) = sqrt((zGrid(iz)*(1.0d+0 - eff_o(it,iz)) + contJ_0 + phi(it))/((1.0d+0 + contW_0 - Uo(it))*(1.0d+0+kappa)))
			
		else
		
			wwo_0(it,iz) = (wmin+wmax)/2.0d+0
			
		end if
		
		!	fill the corresponding value functions
		
		Wo_0(it,iz) = utils(wwo_0(it,iz)) + contW_0
		Jo_0(it,iz) = zGrid(iz)*(1.0d+0 - eff_o(it,iz))-(1.0d+0+kappa)*wwo_0(it,iz) + contJ_0
				
	end do
	end do
		
end subroutine


!	continuation values

subroutine continuation_old_workers()

	!	worker's side
	
	contW_0 = rho*((1.0d+0-khi)*(delta*(Uo(1) + delta_sp*(Uo(min(NT,it+1))-Uo(1))) &
			+ (1.0d+0-delta)*((1.0d+0-pi_o(it,iz))*dot_product(P(iz,:),max(Wo_0(min(NT,it+1),:),Uo(min(NT,it+1)))) &
				+ pi_o(it,iz)*dot_product(P(iz,:),max(Wo_1(min(NT,it+1),:),Uo(min(NT,it+1))))))+ khi*R)
	contW_1 = rho*((1.0d+0-khi)*(delta*(Uo(1) + delta_sp*(Uo(min(NT,it+1))-Uo(1))) &
			+ (1.0d+0-delta)*dot_product(P(iz,:),max(Wo_1(min(NT,it+1),:),Uo(min(NT,it+1))))) + khi*R)

	!	firm's side
	
	contJ_0 = rho*(1.0d+0-khi)*(-delta*phi(min(NT,it+1))*delta_sp &
			+ (1.0d+0-delta)*((1.0d+0-pi_o(it,iz))*dot_product(P(iz,:),max(Jo_0(min(NT,it+1),:),-phi(min(NT,it+1)))) & 
				+ pi_o(it,iz)*dot_product(P(iz,:),max(Jo_1(min(NT,it+1),:),-phi(min(NT,it+1))))))
	contJ_1 = rho*(1.0d+0-khi)*(-delta*phi(min(NT,it+1))*delta_sp &
			+ (1.0d+0-delta)*dot_product(P(iz,:),max(Jo_1(min(NT,it+1),:),-phi(min(NT,it+1)))))
		
end subroutine


!	optimal effort to invest in HC (Nash bargained)

subroutine optim_effort_old_workers()

	double precision :: resW, resJ
	
	resW = dot_product(P(iz,:),max(Wo_1(min(NT,it+1),:),Uo(min(NT,it+1)))) &
		 - dot_product(P(iz,:),max(Wo_0(min(NT,it+1),:),Uo(min(NT,it+1))))
	resW = rho*(1.0d+0-khi)*(1.0d+0-delta)*resW*(1.0d+0+kappa)/umarg(wwo_0(it,iz))
	
	resJ = dot_product(P(iz,:),max(Jo_1(min(NT,it+1),:),-phi(min(NT,it+1)))) &
		 - dot_product(P(iz,:),max(Jo_0(min(NT,it+1),:),-phi(min(NT,it+1))))
	resJ = rho*(1.0d+0-khi)*(1.0d+0-delta)*resJ
	
	eff_o(it,iz) = 0.0d+0
	if ((resJ - resW > 0.0d+0)  .and. (zGrid(iz) .gt. 0.0d+0)) then
		eff_o(it,iz) = max(0.0d+0, min(1.0d0+0, (alpha1*alpha0*(resJ - resW)/zGrid(iz))**dble(1.0d+0/(1.0d+0-alpha1))))
	end if
	pi_o(it,iz) = alpha0*(eff_o(it,iz)**alpha1)
		
end subroutine


!	solve problem at time T

subroutine solve_terminal_problem()

	double precision :: NewJ_0(NZ), NewW_0(NZ), Newww_0(NZ), Wres_0(NZ), Jres_0(NZ)
	double precision :: NewJ_1(NZ), NewW_1(NZ), Newww_1(NZ), Wres_1(NZ), Jres_1(NZ)
	double precision :: ERR_bell
	integer :: NUM_bell
	
	!	set terminal period
	
	it = NT	
	
	!	begin iterations
	
	NUM_bell = 1
	ERR_bell = 1.0d+2
	
	do while (ERR_bell .gt. TOL_bell)

		do iz = 1, NZ

			!	continuation values
			
			call continuation_old_workers()
		
			!	value functions

			NewW_1(iz) = utils(wwo_1(it,iz)) + contW_1
			NewJ_1(iz) = zGrid(iz)*(1.0d+0+H) - (1.0d+0+kappa)*wwo_1(it,iz) + contJ_1
						
			!	reservation wage for workers
			
			Wres_1 = max(Wo_1(NT,:),Uo(NT))
			Wres_1(iz) = Uo(NT)
			wmin = inv_utils(Uo(NT) - rho*((1.0d+0-khi)*(delta*(Uo(1) + delta_sp*(Uo(NT)-Uo(1))) &
												+ (1.0d+0-delta)*dot_product(P(iz,:),Wres_1)) + khi*R))
			
			!	reservation wage for firms
			
			Jres_1 = max(Jo_1(NT,:),-phi(NT))
			Jres_1(iz) = -phi(NT)
			wmax = (phi(NT) + zGrid(iz)*(1.0d+0+H) + rho*(1.0d+0-khi)*(-delta*phi(NT)*delta_sp &
												+ (1.0d+0-delta)*dot_product(P(iz,:),Jres_1)))/(1.0d+0+kappa)
			
			!	update wage function

			if (( wmin .le. wmax ) .and. (1.0d+0 + contW_1 - Uo(NT) .gt. 0.0d+0)) then
				
				Newww_1(iz) = sqrt((zGrid(iz)*(1.0d+0+H) + contJ_1 + phi(NT))/((1.0d+0 + contW_1 - Uo(NT))*(1.0d+0+kappa)))
				
			else
							
				Newww_1(iz) = (wmin+wmax)/2.0d+0
				
			end if

			!	optimal effort to acquire HC
			
			call optim_effort_old_workers()
			
			!	value functions

			NewW_0(iz) = utils(wwo_0(it,iz)) + contW_0
			NewJ_0(iz) = zGrid(iz)*(1.0d+0 - eff_o(NT,iz)) - (1.0d+0+kappa)*wwo_0(it,iz) + contJ_0
						
			!	reservation wage for workers
			
			Wres_0 = max(Wo_0(NT,:),Uo(NT))
			Wres_0(iz) = Uo(NT)
			wmin = inv_utils(Uo(NT) - rho*((1.0d+0-khi)*(delta*(Uo(1) + delta_sp*(Uo(NT)-Uo(1))) &
						+ (1.0d+0-delta)*((1.0d+0-pi_o(NT,iz))*dot_product(P(iz,:),Wres_0)+pi_o(NT,iz)*dot_product(P(iz,:),Wres_1))) + khi*R))
			
			!	reservation wage for firms
			
			Jres_0 = max(Jo_0(NT,:),-phi(NT))
			Jres_0(iz) = -phi(NT)
			wmax = (phi(NT) + zGrid(iz)*(1.0d+0 - eff_o(NT,iz)) + rho*(1.0d+0-khi)*(-delta*phi(NT)*delta_sp &
						+ (1.0d+0-delta)*((1.0d+0-pi_o(NT,iz))*dot_product(P(iz,:),Jres_0)+pi_o(NT,iz)*dot_product(P(iz,:),Jres_1))))/(1.0d+0+kappa)
			
			!	update wage function

			if (( wmin .le. wmax ).and.(1.0d+0+contW_0-Uo(NT) .gt. 0.0d+0) &
						.and.(zGrid(iz)*(1.0d+0 - eff_o(NT,iz))+contJ_0+phi(NT) .gt. 0.0d+0)) then
				
				Newww_0(iz) = sqrt((zGrid(iz)*(1.0d+0 - eff_o(NT,iz)) + contJ_0 + phi(NT))/((1.0d+0 + contW_0 - Uo(NT))*(1.0d+0+kappa)))
				
			else
							
				Newww_0(iz) = (wmin+wmax)/2.0d+0
				
			end if
			

		end do
		
		!	check convergence
		
		ERR_bell = max(maxval(dabs(Wo_0(it,:) - NewW_0)), maxval(dabs(Jo_0(it,:) - NewJ_0)), maxval(dabs(wwo_0(it,:) - Newww_0)), &
					maxval(dabs(Wo_1(it,:) - NewW_1)), maxval(dabs(Jo_1(it,:) - NewJ_1)), maxval(dabs(wwo_1(it,:) - Newww_1)))

		!	stop if too long
		
		if (NUM_bell .gt. MAX_bell) exit
		
		!	update value functions
		
		Wo_0(it,:)  = NewW_0
		Jo_0(it,:)  = NewJ_0
		wwo_0(it,:) = Newww_0
		Wo_1(it,:)  = NewW_1
		Jo_1(it,:)  = NewJ_1
		wwo_1(it,:) = Newww_1
		NUM_bell = NUM_bell + 1
		
	end do

	!	print *, 'value functions for older workers have converged'
	
	if (ERR_bell .gt. TOL_bell) then
		print *, 'force solve_bellman() to exit (older workers)'
		print *, 'last error term was ', ERR_bell
		print *, ' '
		ERR_cvg = 1
	else
		ERR_cvg = 0
	end if
	
end subroutine


end module