!---------------------------------------------------------------------------------------------------------------------------------------------------
!	module wage_function_young: obtain wage functions for young workers
!---------------------------------------------------------------------------------------------------------------------------------------------------

module wage_function_young

	use globals
	use misc_functions
	
	implicit none
	
	double precision :: contW_0, contJ_0, contW_1, contJ_1

contains


!	subroutine to solve for wages

subroutine solve_wage_young_workers()

	double precision :: ERR_bell, NewU(NT)
	integer :: NUM_bell
	
	!	expected duration of jobless spell
	
	annfacY = (1.0d+0-gamma+gamma/khi)/(gamma*(1.0d+0-jfind)+jfind)
	annfacY = (1.0d+0-rho)/(1.0d+0-rho**annfacY)
	
	!	current value for Ul
	
	do it = 1, NT
	
		Ul(it) = (utils(Bo + nu*annfacY*phi(it)*young_sp) + leis + rho*khi*R)/(1.0d+0-rho*(1.0d+0-khi))

	end do	
	
	
	!	begin iteration
	
	NUM_bell = 1
	ERR_bell = 1.0d+2
	
	do while (ERR_bell .gt. TOL_bell)	
		
		!	if young_sp = 0, alter Uy for bargaining 
		
		do it = 1, NT
			Uy(it) = Uy(it) + (1.0d+0-young_sp)*(utils(By+nu*phi(it))-utils(By))
		end do
		
		!	solve terminal problem
		
		call solve_terminal_problem()
		
		!	solve backwards from T-1
		
		do it = NT-1, 1, -1
		do iz = 1, NZ
				
			!	continuation values here
				
			call continuation_young_workers()
			
			
			!	reservation wages
		
			wmin = inv_utils(Uy(it) - contW_1)
			wmax = (phi(it) + zGrid(iz)*(1.0d+0+H) + contJ_1)/(1.0d+0+kappa)
			
			if (( wmin .le. wmax ) .and. (1.0d+0 + contW_1 - Uy(it) .gt. 0.0d+0)) then
						
				wwy_1(it,iz) = sqrt((zGrid(iz)*(1.0d+0+H) + contJ_1 + phi(it))/((1.0d+0 + contW_1 - Uy(it))*(1.0d+0+kappa)))
			
			else
			
				wwy_1(it,iz) = (wmin+wmax)/2.0d+0
				
			end if
								
			!	fill the corresponding value functions
			
			Wy_1(it,iz) = utils(wwy_1(it,iz)) + contW_1
			Jy_1(it,iz) = zGrid(iz)*(1.0d+0+H) - (1.0d+0+kappa)*wwy_1(it,iz) + contJ_1

			
			!	optimal effort to acquire HC
			
			call optim_effort_young_workers()
			
			!	reservation wages
		
			wmin = inv_utils(Uy(it) - contW_0)
			wmax = (phi(it) + zGrid(iz)*(1.0d+0 - eff_y(it,iz)) + contJ_0)/(1.0d+0+kappa)
			
			if (( wmin .le. wmax ) .and. (1.0d+0 + contW_0 - Uy(it) .gt. 0.0d+0) .and. (wmax .gt. 0.0d+0)) then
						
				wwy_0(it,iz) = sqrt((zGrid(iz)*(1.0d+0 - eff_y(it,iz)) + contJ_0 + phi(it))/((1.0d+0+ contW_0 - Uy(it))*(1.0d+0+kappa)))
			
			else
			
				wwy_0(it,iz) = (wmin+wmax)/2.0d+0
				
			end if
								
			!	fill the corresponding value functions
			
			Wy_0(it,iz) = utils(wwy_0(it,iz)) + contW_0
			Jy_0(it,iz) = zGrid(iz)*(1.0d+0 - eff_y(it,iz)) - (1.0d+0+kappa)*wwy_0(it,iz) + contJ_0
						
		end do
		end do
		
		!	if young_sp = 0, alter Uy for iterating
		
		do it = 1, NT
			Uy(it) = Uy(it) - (1.0d+0-young_sp)*(utils(By+nu*phi(it))-utils(By))
		end do
				
		!	new value for Uy
		
		do it = 1, NT
		
			NewU(it) = utils(By +nu*annfacY*phi(it)*young_sp) + rho*((1.0d+0-gamma)*(jfind*Wy_0(1,iz0)+(1.0d+0-jfind)*Uy(it))+gamma*Ul(it))
													
		end do
		
		!	check convergence
		
		ERR_bell = maxval(dabs(NewU - Uy))

		!	stop if too long
		
		if (NUM_bell .gt. MAX_bell) exit
		
		!	update value functions
		
		Uy = NewU
		NUM_bell = NUM_bell + 1
		
		
	end do
	
	!	print *, 'value functions for younger workers have converged'
	
	if (ERR_bell .gt. TOL_bell) then
		print *, 'force solve_bellman_workers() to exit'
		print *, 'last error term was ', ERR_bell
		print *, ' '
		ERR_cvg = 1
	end if		
		
end subroutine


!	continuation values

subroutine continuation_young_workers()

	!	worker's side
	
	contW_0 = rho*(1.0d+0-gamma)*(delta*(Uy(1) + delta_sp*(Uy(min(NT,it+1))-Uy(1))) &
				+(1.0d+0-delta)*((1.0d+0-pi_y(it,iz))*dot_product(P(iz,:),max(Wy_0(min(NT,it+1),:),Uy(min(NT,it+1)))) + &
									pi_y(it,iz)*dot_product(P(iz,:),max(Wy_1(min(NT,it+1),:),Uy(min(NT,it+1))))) ) &
			+ rho*gamma*(delta*(Uo(1) + delta_sp*(Uo(min(NT,it+1))-Uo(1))) &
				+(1.0d+0-delta)*((1.0d+0-pi_y(it,iz))*dot_product(P(iz,:),max(Wo_0(min(NT,it+1),:),Uo(min(NT,it+1)))) + & 
									pi_y(it,iz)*dot_product(P(iz,:),max(Wo_1(min(NT,it+1),:),Uo(min(NT,it+1))))) )

	contW_1 = rho*(1.0d+0-gamma)*(delta*(Uy(1) + delta_sp*(Uy(min(NT,it+1))-Uy(1))) &
				+(1.0d+0-delta)*dot_product(P(iz,:),max(Wy_1(min(NT,it+1),:),Uy(min(NT,it+1))))) &
			+ rho*gamma*(delta*(Uo(1) + delta_sp*(Uo(min(NT,it+1))-Uo(1))) &
				+(1.0d+0-delta)*dot_product(P(iz,:),max(Wo_1(min(NT,it+1),:),Uo(min(NT,it+1)))))
									
	!	firm's side

	contJ_0 = - rho*delta*phi(min(NT,it+1))*delta_sp &
			+ rho*(1.0d+0-gamma)*(1.0d+0-delta)*((1.0d+0-pi_y(it,iz))*dot_product(P(iz,:),max(Jy_0(min(NT,it+1),:),-phi(min(NT,it+1)))) + &
													pi_y(it,iz)*dot_product(P(iz,:),max(Jy_1(min(NT,it+1),:),-phi(min(NT,it+1))))) &
			+ rho*gamma*(1.0d+0-delta)*((1.0d+0-pi_y(it,iz))*dot_product(P(iz,:),max(Jo_0(min(NT,it+1),:),-phi(min(NT,it+1)))) + &
										pi_y(it,iz)*dot_product(P(iz,:),max(Jo_1(min(NT,it+1),:),-phi(min(NT,it+1)))))
	contJ_1 = - rho*delta*phi(min(NT,it+1))*delta_sp &
			+ rho*(1.0d+0-gamma)*(1.0d+0-delta)*dot_product(P(iz,:),max(Jy_1(min(NT,it+1),:),-phi(min(NT,it+1)))) &
			+ rho*gamma*(1.0d+0-delta)*dot_product(P(iz,:),max(Jo_1(min(NT,it+1),:),-phi(min(NT,it+1))))
	
	
end subroutine


!	optimal effort to invest in HC (Nash bargained)

subroutine optim_effort_young_workers()

	double precision :: resW, resJ
	
	resW = gamma*dot_product(P(iz,:),max(Wo_1(min(NT,it+1),:),Uo(min(NT,it+1)))) &
			+ (1.0d+0-gamma)*dot_product(P(iz,:),max(Wy_1(min(NT,it+1),:),Uy(min(NT,it+1)))) &
		- (gamma*dot_product(P(iz,:),max(Wo_0(min(NT,it+1),:),Uo(min(NT,it+1)))) &
			+ (1.0d+0-gamma)*dot_product(P(iz,:),max(Wy_0(min(NT,it+1),:),Uy(min(NT,it+1)))))
	resW = rho*(1.0d+0-delta)*resW*(1.0d+0+kappa)/umarg(wwy_0(it,iz))
	
	resJ = gamma*dot_product(P(iz,:),max(Jo_1(min(NT,it+1),:),-phi(min(NT,it+1)))) &
			+ (1.0d+0-gamma)*dot_product(P(iz,:),max(Jy_1(min(NT,it+1),:),-phi(min(NT,it+1)))) &
		- (gamma*dot_product(P(iz,:),max(Jo_0(min(NT,it+1),:),-phi(min(NT,it+1)))) &
			+ (1.0d+0-gamma)*dot_product(P(iz,:),max(Jy_0(min(NT,it+1),:),-phi(min(NT,it+1)))))
	resJ = rho*(1.0d+0-delta)*resJ
	
	eff_y(it,iz) = 0.0d+0
	if ((resJ - resW > 0.0d+0) .and. (zGrid(iz) .gt. 0.0d+0)) then
		eff_y(it,iz) = max(0.0d+0, min(1.0d0+0, (alpha1*alpha0*(resJ - resW)/zGrid(iz))**dble(1.0d+0/(1.0d+0-alpha1))))
	end if
	pi_y(it,iz) = alpha0*(eff_y(it,iz)**alpha1)
		
end subroutine


!	solve problem at time T

subroutine solve_terminal_problem()

	double precision :: NewJ_0(NZ), NewW_0(NZ), Newww_0(NZ), Wres_0(NZ), Jres_0(NZ)
	double precision :: NewJ_1(NZ), NewW_1(NZ), Newww_1(NZ), Wres_1(NZ), Jres_1(NZ)
	double precision :: ERR_bell
	integer :: NUM_bell
	
	!	set terminal period
	
	it = NT
	
	!	begin iterations
	
	NUM_bell = 1
	ERR_bell = 1.0d+2
	
	do while (ERR_bell .gt. TOL_bell)

		do iz = 1, NZ
		
			!	continuation values
			
			call continuation_young_workers()
					
			!	value function for workers
		
			NewW_1(iz) = utils(wwy_1(NT,iz)) + contW_1
			NewJ_1(iz) = zGrid(iz)*(1.0d+0+H) - (1.0d+0+kappa)*wwy_1(NT,iz) + contJ_1
						
			!	reservation wage for workers
			
			Wres_1 = max(Wy_1(NT,:),Uy(NT))
			Wres_1(iz) = Uy(NT)
			wmin = inv_utils(Uy(NT) &
							- rho*(1.0d+0-gamma)*(delta*(Uy(1) + delta_sp*(Uy(NT)-Uy(1))) &
										+ (1.0d+0-delta)*dot_product(P(iz,:),Wres_1)) &
							- rho*gamma*(delta*(Uo(1) + delta_sp*(Uo(NT)-Uo(1))) &
										+ (1.0d+0-delta)*dot_product(P(iz,:),max(Wo_1(NT,:),Uo(NT)))))

			!	reservation wage for firms
			
			Jres_1 = max(Jy_1(NT,:),-phi(NT))
			Jres_1(iz) = -phi(NT)
			wmax = (phi(NT) + zGrid(iz)*(1.0d+0+H) - rho*delta*phi(NT)*delta_sp &
										+ rho*(1.0d+0-gamma)*(1.0d+0-delta)*dot_product(P(iz,:),Jres_1) &
										+ rho*gamma*(1.0d+0-delta)*dot_product(P(iz,:),max(Jo_1(NT,:),-phi(NT))))/(1.0d+0+kappa)
			  												
			!	update wage function
			
			if (( wmin .le. wmax ) .and. (1.0d+0 + contW_1 - Uy(NT) .gt. 0.0d+0)) then
			
				Newww_1(iz) = sqrt((zGrid(iz)*(1.0d+0+H) + contJ_1 + phi(NT))/((1.0d+0 + contW_1 - Uy(NT))*(1.0d+0+kappa)))
				
			else
										
				Newww_1(iz) = (wmin+wmax)/2.0d+0
				
			end if
			
			
			!	optimal effort to acquire HC
			
			call optim_effort_young_workers()
			
			!	value function for workers
		
			NewW_0(iz) = utils(wwy_0(NT,iz)) + contW_0
			NewJ_0(iz) = zGrid(iz)*(1.0d+0 - eff_y(NT,iz)) - (1.0d+0+kappa)*wwy_0(NT,iz) + contJ_0
						
			!	reservation wage for workers
			
			Wres_0 = max(Wy_0(NT,:),Uy(NT))
			Wres_0(iz) = Uy(NT)
			wmin = inv_utils(Uy(NT) &
				- rho*(1.0d+0-gamma)*(delta*(Uy(1) + delta_sp*(Uy(NT)-Uy(1))) &
					+ (1.0d+0-delta)*((1.0d+0-pi_y(NT,iz))*dot_product(P(iz,:),Wres_0)+pi_y(NT,iz)*dot_product(P(iz,:),Wres_1))) &
				- rho*gamma*(delta*(Uo(1) + delta_sp*(Uo(NT)-Uo(1))) &
					+ (1.0d+0-delta)*((1.0d+0-pi_y(NT,iz))*dot_product(P(iz,:),max(Wo_0(NT,:),Uo(NT)))&
										+pi_y(NT,iz)*dot_product(P(iz,:),max(Wo_1(NT,:),Uo(NT))))))

			!	reservation wage for firms
			
			Jres_0 = max(Jy_0(NT,:),-phi(NT))
			Jres_0(iz) = -phi(NT)
			wmax = (phi(NT) + zGrid(iz)*(1.0d+0 - eff_y(NT,iz)) - rho*delta*phi(NT)*delta_sp &
							+ rho*(1.0d+0-gamma)*(1.0d+0-delta)*((1.0d+0-pi_y(NT,iz))*dot_product(P(iz,:),Jres_0) &
										+ pi_y(NT,iz)*dot_product(P(iz,:),Jres_1)) &
							+ rho*gamma*(1.0d+0-delta)*((1.0d+0-pi_y(NT,iz))*dot_product(P(iz,:),max(Jo_0(NT,:),-phi(NT)))&
															+ pi_y(NT,iz)*dot_product(P(iz,:),max(Jo_1(NT,:),-phi(NT)))))/(1.0d+0+kappa)
			  												
			!	update wage function
			
			if (( wmin .le. wmax ).and.(1.0d+0+contW_0-Uy(NT) .gt. 0.0d+0) &
						.and.(zGrid(iz)*(1.0d+0-eff_y(NT,iz))+contJ_0+phi(NT) .gt. 0.0d+0)) then
			
				Newww_0(iz) = sqrt((zGrid(iz)*(1.0d+0 - eff_y(NT,iz)) + contJ_0 + phi(NT))/((1.0d+0 + contW_0 - Uy(NT))*(1.0d+0+kappa)))
				
			else
										
				Newww_0(iz) = (wmin+wmax)/2.0d+0
				
			end if
			
			
						
		end do
						
		!	check convergence
		
		ERR_bell = max(maxval(dabs(Wy_0(NT,:) - NewW_0)), maxval(dabs(Jy_0(NT,:) - NewJ_0)), maxval(dabs(wwy_0(NT,:) - Newww_0)), &
					maxval(dabs(Wy_1(NT,:) - NewW_1)), maxval(dabs(Jy_1(NT,:) - NewJ_1)), maxval(dabs(wwy_1(NT,:) - Newww_1)))

		!	stop if too long
		
		if (NUM_bell .gt. MAX_bell) exit
		
		!	update value functions
		
		Wy_0(NT,:)  = NewW_0
		Jy_0(NT,:)  = NewJ_0
		wwy_0(NT,:) = Newww_0
		Wy_1(NT,:)  = NewW_1
		Jy_1(NT,:)  = NewJ_1
		wwy_1(NT,:) = Newww_1		
		NUM_bell = NUM_bell + 1
		
	end do

	!print *, 'value functions for young workers have converged'
	
	if (ERR_bell .gt. TOL_bell) then
		print *, 'force solve_bellman() to exit (young workers)'
		print *, 'last error term was ', ERR_bell
		print *, ' '
		ERR_cvg = 1
	end if
	
end subroutine


end module